#ifndef __IntegerRange_hpp__
#define __IntegerRange_hpp__

#include <ostream>

class IntegerRange {

  int lowerBound_;  ///< lower bound of range (range includes this).
  int upperBound_;  ///< upper bound of range (range doesn't include this).

public:

  ///
  /// @brief Initialize a new instance of the IntegerRange class 
  /// with an empty range.
  ///
  IntegerRange();

  ///
  /// @brief Initialize a new instance of the IntegerRange as a copy 
  /// of another instance.
  ///
  IntegerRange( const IntegerRange & integerRange );

  ///
  /// @brief Initialize a new instance of the IntegerRange class with 
  /// a specified lower and upper bound.
  ///
  IntegerRange( int lowerBound, int upperBound );

  ///
  /// @brief Assignment operator.
  ///
  const IntegerRange & operator=( const IntegerRange & integerRange );

  ///
  /// @brief Equality operator.
  ///
  bool operator==( const IntegerRange & integerRange ) const;

  ///
  /// @brief Inequality operator.
  ///
  bool operator!=( const IntegerRange & integerRange ) const;

  ///
  /// @brief Get the lower bound of the instance.
  ///
  int GetLowerBound() const;

  ///
  /// @brief Get the upper bound of the instance.
  ///
  int GetUpperBound() const;

  ///
  /// @brief Check if the range is empty.
  ///
  bool IsEmpty() const;

  ///
  /// @brief Get the size of the range.
  ///
  unsigned int GetSize() const;

};

///
/// @brief Get the intersection of two IntegerRange instances.
///
const IntegerRange Intersect( const IntegerRange & integerRange0, 
		const IntegerRange & integerRange1 );

///
/// @brief Get the smallest integer range that contains two ranges.
///
const IntegerRange Contain( const IntegerRange & integerRange0, 
		const IntegerRange & integerRange1 );

///
/// @brief Output the range to an ostream instance.
///
std::ostream & operator<<( std::ostream & os, 
		const IntegerRange & integerRange );

#endif // __IntegerRange_hpp__
