#ifndef CMATRIX_ALREADY_INCLUDED
#define CMATRIX_ALREADY_INCLUDED

#include <common.h>
#include <cvector.h>

#define MAT_DEG_TO_RAD(D) ((D)/180.0*M_PI)
#define MAT_RAD_TO_DEG(R) ((R)/M_PI*180.0)


typedef float CMatrix[4][4];


/*****************************************************************************
MatCopy

Copy the matrix given.

*****************************************************************************/
#define MatCopy(result, mat)			\
if (1) {					\
   register int _i, _j;				\
						\
   for (_i = 3; _i >= 0; _i--) 			\
     for (_j = 3; _j >= 0; _j--) 		\
        (result)[_i][_j] = (mat)[_i][_j];	\
} else


/*****************************************************************************
MatTranspose

Tranpose the matrix and return it in the result matrix.

*****************************************************************************/
#define MatTranspose(result, mat)		\
if (1) {					\
   register int  _i, _j;			\
						\
   IFDEBUG((float *)(result) == (float *)(mat))	\
      fprintf(stderr,"MatTranspose() Must use 2 matrices!\n");\
						\
   for (_i = 3; _i >= 0; _i--) 			\
     for (_j = 3; _j >= 0; _j--) 		\
       (result)[_j][_i] = (mat)[_i][_j];	\
} else


/*****************************************************************************
MatMultPos

Multiplies a point by a matrix, setting the result point. It assumes all 
homogeneous coordinates are 1. The two points involved may be the same.

*****************************************************************************/
#define MatMultPos(result, mat, in)		\
if (1) {					\
   register int   _i,_j;			\
   register float _pos[4];			\
   register float _res[4];			\
						\
   _pos[0] = in[0];				\
   _pos[1] = in[1];				\
   _pos[2] = in[2];				\
   _pos[3] = 1.0;				\
						\
   for (_i = 0; _i < 4; _i++) {			\
      _res[_i] = 0.0;				\
      for (_j = 0; _j < 4; _j++)		\
         _res[_i] += mat[_i][_j] * _pos[_j];	\
   }						\
						\
   if (_res[3] != 1.0 && _res[3] != 0.0) {	\
      _res[0] /= _res[3];			\
      _res[1] /= _res[3];			\
      _res[2] /= _res[3];			\
   }						\
						\
   result[0] = _res[0];				\
   result[1] = _res[1];				\
   result[2] = _res[2];				\
} else


/*****************************************************************************
MatMultPosD

Multiplies a point by a matrix, setting the result point. It assumes all 
homogeneous coordinates are 1. The two points involved may be the same.

-- use doubles!

*****************************************************************************/
#define MatMultPosD(result, mat, in)		\
if (1) {					\
   register int   _i,_j;			\
   register double _pos[4];			\
   register double _res[4];			\
						\
   _pos[0] = in[0];				\
   _pos[1] = in[1];				\
   _pos[2] = in[2];				\
   _pos[3] = 1.0;				\
						\
   for (_i = 0; _i < 4; _i++) {			\
      _res[_i] = 0.0;				\
      for (_j = 0; _j < 4; _j++)		\
         _res[_i] += mat[_i][_j] * _pos[_j];	\
   }						\
						\
   if (_res[3] != 1.0 && _res[3] != 0.0) {	\
      _res[0] /= _res[3];			\
      _res[1] /= _res[3];			\
      _res[2] /= _res[3];			\
   }						\
						\
   result[0] = _res[0];				\
   result[1] = _res[1];				\
   result[2] = _res[2];				\
} else


/*****************************************************************************
MatMultVec

Multiplies a vector by a matrix, setting the result vector. It assumes all 
homogeneous coordinates are 0. The two vectors involved may be the same.

*****************************************************************************/
#define MatMultVec(result, mat, in)		\
if (1) {					\
   register int   _i,_j;			\
   register float _pos[4];			\
   register float _res[4];			\
						\
   _pos[0] = in[0];				\
   _pos[1] = in[1];				\
   _pos[2] = in[2];				\
   _pos[3] = 0.0;				\
						\
   for (_i = 0; _i < 4; _i++) {			\
      _res[_i] = 0.0;				\
      for (_j = 0; _j < 4; _j++)		\
         _res[_i] += mat[_i][_j] * _pos[_j];	\
   }						\
						\
   if (_res[3] != 1.0 && _res[3] != 0.0) {	\
      _res[0] /= _res[3];			\
      _res[1] /= _res[3];			\
      _res[2] /= _res[3];			\
   }						\
						\
   result[0] = _res[0];				\
   result[1] = _res[1];				\
   result[2] = _res[2];				\
} else


/* Prototypes */
extern int  MatInvert( CMatrix result, CMatrix mat );
extern int  MatInvertFull( CMatrix result, CMatrix mat );
extern int  MatInvertAnglePreserving(CMatrix out, CMatrix in);

extern void MatZero( CMatrix mat );
extern void MatIdentity( CMatrix mat );
/* extern void MatCopy( CMatrix result, CMatrix mat ); */
extern void MatMultiply( CMatrix result, CMatrix mat1, CMatrix mat2 );
/* extern void MatTranspose( CMatrix result, CMatrix mat ); */

extern void MatSetTranslate( CMatrix mat, CVector pos );
extern void MatGetTranslate( CVector pos, CMatrix mat );
extern void MatTranslate( CMatrix mat, CVector pos );
extern void MatTranslate( CMatrix mat, float tx, float ty, float tz );
extern void MatScale( CMatrix mat, CVector vec );
extern void MatScaleUniform( CMatrix mat, float s );
extern void MatRotate( CMatrix m, CVector axis, float radians );
extern void MatRotateAxis( CMatrix m, int axis, float radians );

extern void MatCreateOrthoRotate(CMatrix mat, CVector, CVector, CVector);
extern void MatCreateViewMatrix(CMatrix mat, CVector, CVector, CVector);
extern void MatCreateFrustum(CMatrix mat, float l,float r,float b,
                                         float t,float n,float f);
extern void MatCreatePerspective(CMatrix mat, float fov, float asp, 
                                 float nz, float fz);
extern void MatPrint( char *msge, CMatrix mat );

#endif
