#ifndef VBALL_ALREADY_INCLUDED
#define VBALL_ALREADY_INCLUDED

#include <common.h>
#include <cvector.h>
#include <cmatrix.h>

/* no UNC joysticks */
#define NOJOY

#ifndef NOJOY
#define VBALL_DFLT_DELTAROT		0.15
#define VBALL_DFLT_DELTATRANS		1000.0 /* should be max dimension/10 */
#endif


/*****************************************************************************
VBall

This class defines a virtual trackball. The algorithm is copied from
Graphics Gems. 

*****************************************************************************/
class VBall {
   private:
      CMatrix  ctm;		/* current view matrix to use */
      CVector  ctr;		/* center of trackball */
      CVector  ppos;		/* previous position on trackball surface */
      int      first;		/* where the first update is to occur */
      float    radius;		/* radius of trackball */
      int      width,height;	/* width x height of window */
      int      oldx,oldy,oldz;	/* previous position of mouse in window */
      int      walkMode;	/* whether in walk mode or not */
      int      initJoy;         /* joy sticks need initialization flag */
      float    deltaRot;        /* delta rotation factor for joysticks */
      float    deltaTrans;      /* delta translation factor for joysticks */

      int  intersectSphere(CVector base, CVector ctr, float radius,
                           CVector dir, CVector pt);
      int  rotateToPoint(CVector p, CVector axis, float *radians);
      void initJoysticks(void);
      void makeMat(CMatrix mat, float rx, float ry, float rz,
                   float tx, float ty, float tz);

   public:
      VBall(float radius, int width, int height);
     ~VBall() {};

      /* reset the trackball */
      void reset(void)
      {
         first = TRUE;
      }

      /* walk rotate trackball about y axis */
      void walkRotate(int deltaX, float scale);

      /* rotate trackball to delta x,y */
      void rotate(int x, int y, float scale);

      /* xy translate trackball to delta x,y */
      void xytranslate(int x, int y);

      /* z translate trackball according delta x */
      void ztranslate(int y);

      /* scale trackball according max(delta x, delta y) */
      void scale(int x, int y);

      /* scale trackball according delta x */
      void xscale(int x);

      /* scale trackball according delta y */
      void yscale(int y);

      /* spin the trackball matrix */
      void spin(int axis, float radians);

      /* set walk mode flag */
      void setWalk(int f) 
      {
         walkMode = f;
      }

      /* set radius of model size */
      void setRadius(float rad)
      {
         VecSet(ctr,0,0,0);
         first  = TRUE;
         radius = rad;
      }


      /* update from joysticsk */
      int  updateJoysticks(void);

      /* force level eye */
      void setLevelEye();

      /* inquire walk mode flag */
      /* inquire walk mode flag */
      int  walk(void)
      {
         return (walkMode);
      }

      /* update window size */
      void resize(int w, int h)
      {
         width = w;
         height = h;
      }

      /* return the actual trackball matrix */
      CMatrix *matrix(void) 
      { 
         return (&ctm);
      }
};


#endif
