#include "AnchorReconstructor.hpp"
#include <math.h>
#include <iostream>

using std::cout;
using std::endl;

#define FTOL 1.0e-4

float func( SceneData & sceneData, const FeaturePair & pair, int imageIDs[], int imageNumber, float p[] );
void dfunc( SceneData & sceneData, const FeaturePair & pair, int imageIDs[], int imageNumber, float p[], float xi[]);
bool frprmn( SceneData & sceneData, const FeaturePair & pair, int imageIDs[], int imageNumber, float p[], int n, float ftol, int *iter, float *fret, 
             float (*func)( SceneData &, const FeaturePair &, int [], int, float []), 
			 void (*dfunc)( SceneData &, const FeaturePair &, int [], int, float [], float []));


float scale;

/*****************************************************************************
*****************************************************************************/
AnchorReconstructor::AnchorReconstructor( SceneData & sceneData, 
		float cosAngleThreshold ): sceneData_( sceneData ),
      cosAngleThreshold_( cosAngleThreshold )
{}


/*****************************************************************************
*****************************************************************************/
AnchorReconstructor::AnchorReconstructor( 
		const AnchorReconstructor & anchorReconstructor ): 
      sceneData_( anchorReconstructor.sceneData_ ),
      cosAngleThreshold_( anchorReconstructor.cosAngleThreshold_ )
{}


/*****************************************************************************
*****************************************************************************/
bool AnchorReconstructor::ChooseReconImages( const FeaturePair & pair, 
		int imageIDs[], int imageNumber ) const
{ 
  int i, j, n, lowerBound;
  float cosAngle[2];

  n = pair.GetImageRange().GetSize();
  std::vector<int> validImage( n );
  lowerBound = pair.GetImageRange().GetLowerBound();  

  for ( i=0, j=0; i<n; i++ )
  { if( sceneData_.GetViewpointValidOrientationList()[lowerBound+i] )
	{ validImage[j] = lowerBound + i;
	  j++;
	}
  }

  if ( j<imageNumber )
    return false;

  n = j / imageNumber;  

  for ( i=0; i<imageNumber; i++ )
    imageIDs[i] = validImage[i * n];

  return true;
}


/*****************************************************************************
*****************************************************************************/
bool AnchorReconstructor::EstimateAnchorPair( const FeaturePair & pair, 
		int imageIDs[], float p[], float cosAngle[2], int imageNumber ) const
{ 
  VM3Vector *coord2[2], *coord3[2], Q[2], *C;
  float k[2][2], cosAngleTemp;
  int i, j, m, jMax, mMax;

  for ( i=0; i<2; i++ )
  { coord2[i] = (VM3Vector *) malloc( imageNumber * sizeof( VM3Vector ) );
    coord3[i] = (VM3Vector *) malloc( imageNumber * sizeof( VM3Vector ) );
  }

  C = (VM3Vector *) malloc( imageNumber * sizeof( VM3Vector ) );
  
  for ( i=0; i<2; i++ )
    cosAngle[i] = 1.0;  

  for ( i=0; i<imageNumber; i++ )
    C[i] = sceneData_.GetViewpointLocationList()[imageIDs[i]];    

  for ( i=0; i<imageNumber; i++ )
  { 
	 coord2[0][i].set( 
			pair.GetFirstFeature().GetLocation(imageIDs[i]).x_ * 
				sceneData_.GetCameraParameters().realLength,
         pair.GetFirstFeature().GetLocation(imageIDs[i]).y_ * 
		   	sceneData_.GetCameraParameters().realWidth,
         -sceneData_.GetCameraParameters().focus );
    coord2[1][i].set( 
			pair.GetSecondFeature().GetLocation(imageIDs[i]).x_ * 
				sceneData_.GetCameraParameters().realLength,
         pair.GetSecondFeature().GetLocation(imageIDs[i]).y_ * 
			   sceneData_.GetCameraParameters().realWidth,
         -sceneData_.GetCameraParameters().focus );

	 for ( j=0; j<2; j++ )
       coord3[j][i] = sceneData_.GetViewpointOrientationList()[imageIDs[i]] * 
			 coord2[j][i];
  }

  for ( i=0; i<2; i++ )
  { for ( j=0; j<imageNumber-1; j++ )
    { for ( m=j+1; m<imageNumber; m++ )
      { cosAngleTemp = coord3[i][j].dot( coord3[i][m] ) / 
			( coord3[i][j].length() * coord3[i][m].length() );

		if ( ABS( cosAngle[i] ) > ABS( cosAngleTemp ) )
        {  cosAngle[i] = cosAngleTemp;
           jMax = j;
		   mMax = m;
		}
	  }
	}

    if ( cosAngle[i] > cosAngleThreshold_ )
	  break;

    k[0][i] = ( coord3[i][jMax].dot( C[mMax] - C[jMax] )*
			       coord3[i][mMax].dot( coord3[i][mMax] ) -
                coord3[i][mMax].dot( C[mMax] - C[jMax] )*
					 coord3[i][jMax].dot( coord3[i][mMax] ) ) /
              ( coord3[i][jMax].dot( coord3[i][jMax] )*
					 coord3[i][mMax].dot( coord3[i][mMax] ) -
                coord3[i][jMax].dot( coord3[i][mMax] )*
					 coord3[i][jMax].dot( coord3[i][mMax] ) );

    k[1][i] = ( k[0][i] * coord3[i][jMax].dot( coord3[i][jMax] ) - 
			       coord3[i][jMax].dot( C[mMax]-C[jMax] ) ) / 
		          coord3[i][jMax].dot( coord3[i][mMax] );

    Q[i] = C[jMax] + k[0][i] * coord3[i][jMax];
	 Q[i] = Q[i] + C[mMax] + k[1][i] * coord3[i][mMax];
	 Q[i] = Q[i] / 2;
  }

  if ((cosAngle[0] > cosAngleThreshold_ )||( cosAngle[1] > cosAngleThreshold_))
  { for ( i=0; i<2; i++ )
    { free( coord2[i] );
      free( coord3[i] );
	}
	free( C );
    return false; 
  }

  //Q[0] = sceneData_.GetReconObjectList()[pair.GetFirstFeature().GetID()].GetReconLocation();
  //Q[1] = sceneData_.GetReconObjectList()[pair.GetSecondFeature().GetID()].GetReconLocation();

  scale = ( Q[0] - C[0] ).length();
  
  Q[0] = Q[0];// / scale;
  Q[1] = Q[1];// / scale;
  
  p[1] = Q[0].x();
  p[2] = Q[0].y();
  p[3] = Q[0].z();
  p[4] = Q[1].x();
  p[5] = Q[1].y();
  p[6] = Q[1].z();

  for ( i=0; i<imageNumber; i++)
	  C[i] = C[i];// / scale;

  for ( i=0; i<imageNumber; i++)
  { p[7+2*i] = sqrt( ( Q[0]-C[i] ).dot( Q[0]-C[i] ) / 
		               coord2[0][i].dot( coord2[0][i] ) );
    p[8+2*i] = sqrt( ( Q[1]-C[i] ).dot( Q[1]-C[i] ) / 
			            coord2[1][i].dot( coord2[1][i] ) );
	p[2*imageNumber+7+3*i] = C[i].x();
	p[2*imageNumber+8+3*i] = C[i].y();
	p[2*imageNumber+9+3*i] = C[i].z();
  }

  for ( i=0; i<2; i++ )
  { free( coord2[i] );
    free( coord3[i] );
  }
  free( C );

#if 0
  const Feature & f1 = pair.GetFirstFeature();
  const Feature & f2 = pair.GetSecondFeature();
  const std::vector<VM3Vector> & viewpointLocationList = sceneData_.GetViewpointLocationList();
  const std::vector<VM4Matrix> & viewpointOrientationList = sceneData_.GetViewpointOrientationList();
  cout << "Reconstructing anchor pair: " << f1.GetID() << " " << f2.GetID() << endl;

  {
    i= imageIDs[jMax];
    const VM3Vector & vpLoc = viewpointLocationList[i];
    VM4Matrix vpOri = viewpointOrientationList[i];
    const Feature::Location & l1 = f1.GetLocation( i );
    const Feature::Location & l2 = f2.GetLocation( i );
    cout << " Image 1:" << i << endl;
    cout
      << "  (" << vpLoc.x() << ", " << vpLoc.y() << ", " << vpLoc.z() << ")" << endl
      << endl;
    vpOri.print( stdout );
    cout
      << endl
      << "  (" << l1.x_ << ", " << l1.y_ << ") (" << l2.x_ << ", " << l2.y_ << ")" << endl;
  }

  {
    i= imageIDs[mMax];
    const VM3Vector & vpLoc = viewpointLocationList[i];
    VM4Matrix vpOri = viewpointOrientationList[i];
    const Feature::Location & l1 = f1.GetLocation( i );
    const Feature::Location & l2 = f2.GetLocation( i );
    cout << " Image 2:" << i << endl;
    cout
      << "  (" << vpLoc.x() << ", " << vpLoc.y() << ", " << vpLoc.z() << ")" << endl
      << endl;
    vpOri.print( stdout );
    cout
      << endl
      << "  (" << l1.x_ << ", " << l1.y_ << ") (" << l2.x_ << ", " << l2.y_ << ")" << endl;
  }
#endif

  return true;
}


/*****************************************************************************
*****************************************************************************/
bool AnchorReconstructor::ReconstructAnchorPair( const FeaturePair & pair, 
		int imageNumber ) const
{ 
  int *imageIDs, iter;
  float *p, cosAngle[2], fret;
  VM3Vector reconAnchor[2];

  imageIDs = (int *) malloc( imageNumber * sizeof( int ) );
  p = (float *) malloc( ( 6 + imageNumber * 5 + 1 ) * sizeof( float ) );

  try {

    if ( !ChooseReconImages( pair, imageIDs, imageNumber ) )
	{ free( imageIDs );
	  free( p );
      return false;
	}

    if ( !EstimateAnchorPair( pair, imageIDs, p, cosAngle, imageNumber ) )
	{ free( imageIDs );
	  free( p );
      return false;
	}

    if ( !frprmn( sceneData_, pair, imageIDs, imageNumber, p, 
				 6+imageNumber*2, FTOL, &iter, &fret, func, dfunc ) )
    { free( imageIDs );
      free( p );
      return false;
    }

	//printf("%d %f\n",iter,fret);

    reconAnchor[0].set( p[1], p[2], p[3] );
    reconAnchor[1].set( p[4], p[5], p[6] );

    sceneData_.GetReconObjectList()[pair.GetFirstFeature().GetID()].SetAnchorReconstruction( reconAnchor[0], cosAngle[0] );
    sceneData_.GetReconObjectList()[pair.GetSecondFeature().GetID()].SetAnchorReconstruction( reconAnchor[1], cosAngle[1] );
  }

  catch( const std::exception & e ) {

    std::cerr << e.what() << std::endl;
	free( imageIDs );
	free( p );
    return false;
  }
  free( imageIDs );
  free( p );
  
  return true;
}

