#include "Feature.hpp"
#include <stdexcept>

using namespace std;


/*****************************************************************************
*****************************************************************************/
Feature::Feature():
  id_( -1 )
{}


/*****************************************************************************
*****************************************************************************/
Feature::Feature( int id, size_t firstViewFrame, 
		const vector<Location> locations ):
  id_( id ),
  firstViewFrame_( firstViewFrame ),
  locations_( locations )
{}


/*****************************************************************************
*****************************************************************************/
Feature::Feature( int id, int objectId, size_t firstViewFrame, 
		const vector<Location> locations ):
  id_( id ),
  objectId_( objectId ),
  firstViewFrame_( firstViewFrame ),
  locations_( locations )
{}


/*****************************************************************************
*****************************************************************************/
Feature::Feature( const Feature & feature ):
  id_( feature.id_ ),
  objectId_( feature.objectId_ ),
  firstViewFrame_( feature.firstViewFrame_ ),
  locations_( feature.locations_ )
{}


/*****************************************************************************
*****************************************************************************/
const Feature & Feature::operator=( const Feature & feature )
{
  id_ = feature.id_;
  objectId_ = feature.objectId_;
  firstViewFrame_ = feature.firstViewFrame_;
  locations_ = feature.locations_;
  return *this;
}


/*****************************************************************************
*****************************************************************************/
int Feature::GetID() const
{
  return id_;
}


/*****************************************************************************
*****************************************************************************/
int Feature::GetObjectID() const
{
  return objectId_;
}


/*****************************************************************************
*****************************************************************************/
void Feature::overwriteLocationList(int newFirstFrame, std::vector<Location> locList)
{
	firstViewFrame_ = newFirstFrame;
	locations_.swap(locList);
}
/*****************************************************************************
*****************************************************************************/
const Feature::Location & Feature::GetLocation( int frame ) const
{
  int index = frame - firstViewFrame_;
  if( index < 0 || index >= locations_.size() )
    throw runtime_error( "A feature location was requested at an "
			                "invalid frame number." );

  return locations_[index];
}


/*****************************************************************************
*****************************************************************************/
const IntegerRange Feature::GetRange() const
{
  return IntegerRange( firstViewFrame_, firstViewFrame_ + locations_.size() );
}


/*****************************************************************************
*****************************************************************************/
int Feature::GetRangeSize() const
{
  return locations_.size();
}


/*****************************************************************************
*****************************************************************************/
ostream & operator<<( ostream & os, const Feature & feature )
{
  return os << "<Feature: " << feature.GetID()  << feature.GetRange() << ">";
}
