#include "IntegerRange.hpp"
#include <numeric>

using namespace std;

namespace {

  template<typename T> inline T min( T a, T b )
  {
    return a < b ? a : b;
  }

  template<typename T> inline T max( T a, T b )
  {
    return a > b ? a : b;
  }

}

IntegerRange::IntegerRange():
  lowerBound_( 0 ),
  upperBound_( 0 )
{}

IntegerRange::IntegerRange( const IntegerRange & integerRange ):
  lowerBound_( integerRange.lowerBound_ ),
  upperBound_( integerRange.upperBound_ )
{}

IntegerRange::IntegerRange( int lowerBound, int upperBound ):
  lowerBound_( lowerBound ),
  upperBound_( max( lowerBound, upperBound ) )
{}

const IntegerRange & IntegerRange::operator=( const IntegerRange & integerRange )
{
  lowerBound_ = integerRange.lowerBound_;
  upperBound_ = integerRange.upperBound_;
  return *this;
}

bool IntegerRange::operator==( const IntegerRange & integerRange ) const
{
  return lowerBound_ == integerRange.lowerBound_ && upperBound_ == integerRange.upperBound_;
}

bool IntegerRange::operator!=( const IntegerRange & integerRange ) const
{
  return lowerBound_ != integerRange.lowerBound_ || upperBound_ != integerRange.upperBound_;
}

int IntegerRange::GetLowerBound() const
{
  return lowerBound_;
}

int IntegerRange::GetUpperBound() const
{
  return upperBound_;
}

bool IntegerRange::IsEmpty() const
{
  return lowerBound_ == upperBound_;
}

unsigned int IntegerRange::GetSize() const
{
  return upperBound_ - lowerBound_;
}

const IntegerRange Intersect( const IntegerRange & integerRange0, 
		const IntegerRange & integerRange1 )
{
  return IntegerRange( max( integerRange0.GetLowerBound(), 
			                   integerRange1.GetLowerBound() ),
                       min( integerRange0.GetUpperBound(), 
								    integerRange1.GetUpperBound() ) );
}

const IntegerRange Contain( const IntegerRange & integerRange0, 
		const IntegerRange & integerRange1 )
{
  return IntegerRange( min( integerRange0.GetLowerBound(), 
			                   integerRange1.GetLowerBound() ),
                       max( integerRange0.GetUpperBound(), 
								    integerRange1.GetUpperBound() ) );
}

ostream & operator<<( ostream & os, const IntegerRange & integerRange )
{
  return os << "[" << integerRange.GetLowerBound() << ", " << 
	  integerRange.GetUpperBound() << ")";
}
