#include "Disable4786.hpp"
#include "SceneData.hpp"
#include "ranlib.h"
#include <sstream>
#include <iostream>
#include <cstdlib>
#include <ctime>
#include <fstream> //BOBEDIT

#define EPSILON 1e-3
#define FTOL 1.0e-6

float funcAll( SceneData & sceneData, float p[] );
void dfuncAll( SceneData & sceneData, float p[], float xi[]);
bool frprmnAll( SceneData & sceneData, float p[], int n, float ftol, int *iter, float *fret, 
                float (*funcAll)( SceneData &, float []), 
			    void (*dfuncAll)( SceneData &, float [], float []));
bool powellAll( SceneData & sceneData, float p[], float **xi, int n, float ftol, int *iter, 
		float *fret, float (*func)( SceneData &, float [] ) );

using namespace std;

namespace {

  float frand()
  {
    return float( rand() ) / float( RAND_MAX );
  }

}
/*****************************************************************************
*****************************************************************************/
std::vector<int> & SceneData::GetAbsoluteImageIDList()
{
	return absoluteImageIDList_;
}

const std::vector<int> & SceneData::GetAbsoluteImageIDList() const
{
	return (const std::vector<int> &)absoluteImageIDList_;
}


/*****************************************************************************
*****************************************************************************/
SceneData::SceneData()
{}


/*****************************************************************************
*****************************************************************************/
SceneData::SceneData( const SceneData & sceneData ):
  viewpointLocationList_( sceneData.viewpointLocationList_ ),
  absoluteImageIDList_( sceneData.absoluteImageIDList_ ),
  viewpointOrientationList_( sceneData.viewpointOrientationList_ ),  
  viewpointValidOrientationList_( sceneData.viewpointValidOrientationList_ ),
  featureList_( sceneData.featureList_ ),
  reconObjectList_( sceneData.reconObjectList_ ),
  reconViewpointLocationList_( sceneData.reconViewpointLocationList_ ),
  SBAReconLocationList_( sceneData.SBAReconLocationList_ ),
  //triangulateReconLocationList_( sceneData.triangulateReconLocationList_ ),
  projErrorList_( sceneData.projErrorList_ ),
  objectLocationList_( sceneData.objectLocationList_), //TODO: should this be present? Question by Bob, feb 14, 2006
  anchorReconstructor_( 0 ),
  nonanchorReconstructor_( 0 ),
  anchorPlanner_( 0 ),
  cameraParameters_( sceneData.cameraParameters_ )
{}

/*****************************************************************************
*****************************************************************************/
SceneData::SceneData(std::vector<VM3Vector> viewpointLocationList, std::vector<int> absoluteImageIDList, std::vector<bool> viewpointValidOrientationList,  
			std::vector<VM4Matrix> viewpointOrientationList, std::vector<Feature> featureList, std::vector<ReconObject> reconObjectList,  
			std::vector<VM3Vector> reconViewpointLocationList, std::vector<VM3Vector> SBAReconLocationList, 
			//std::vector<VM3Vector> triangulateReconLocationList,
			std::vector<float> projErrorList, std::vector<VM3Vector> objectLocationList,   
			AnchorReconstructor * anchorReconstructor, NonanchorReconstructor * nonanchorReconstructor, AnchorPlanner * anchorPlanner,
			CameraParameters cameraParameters)
{
	viewpointLocationList_ = viewpointLocationList;
	absoluteImageIDList_ = absoluteImageIDList;
	viewpointValidOrientationList_ = viewpointValidOrientationList;
	viewpointOrientationList_ = viewpointOrientationList;
	featureList_ = featureList;
	reconObjectList_ = reconObjectList;
	reconViewpointLocationList_ = reconViewpointLocationList;
	SBAReconLocationList_ = SBAReconLocationList;
	//triangulateReconLocationList_ = triangulateReconLocationList;
	projErrorList_ = projErrorList;
	objectLocationList_ = objectLocationList;
	
	anchorReconstructor_ = anchorReconstructor;
	nonanchorReconstructor_ = nonanchorReconstructor;
	anchorPlanner_ = anchorPlanner;
	cameraParameters_ = cameraParameters;
}
/*****************************************************************************
*****************************************************************************/
namespace {
  class SyntaxException: public runtime_error
  {
    SyntaxException( const string & what ): runtime_error( what ) {}
  };

  void CheckSyntax( const string & encountered, const string & expected )
  {
    if( encountered != expected )
      throw SyntaxException( string( "encountered \"" ) + encountered + string( "\" but expected \"" ) + expected  + string( "\"." ) );
  }

  stringstream ReadLine( istream & input, unsigned & lineNum )
  {
    while( 1 ) {
      ++lineNum;

      if( !input )
        throw SyntaxException( "Unexpected end of file." );

      char line[1024];
      input.getline( line, 1024 );
      if( strlen( line ) > 0 ) {
        return stringstream( line );
      }
    }
  }
}


/*****************************************************************************
*****************************************************************************/
int isOutlier( int i, int numOutliers, int * outliers )
{
	for ( int j=0; j<numOutliers; j++ )
	{
		if ( i == outliers[j] )
			break;
	}

	if ( j == numOutliers )
		return FALSE;
	else
		return TRUE;
}

/*****************************************************************************
*****************************************************************************/
void SceneData::ReconstructAnchors( float featurePairLifeWeight, 
		float featurePairDistanceWeight, float continuationThreshold, 
		int anchorImageNumber, float estimatedTrackingError, 
		float cameraPositionNoise, float cameraOrientationNoise )
{
  {
    std::vector<ReconObject> newReconObjects( featureList_.size() );
    reconObjectList_.swap( newReconObjects );
    for( int i=0; i<reconObjectList_.size(); ++i )
      reconObjectList_[i].SetFeatureID( i );
  }

  if( anchorReconstructor_ == 0 )
    anchorReconstructor_ = new AnchorReconstructor( *this );

  if( anchorPlanner_ == 0 )
    anchorPlanner_ = new AnchorPlanner( *this, *anchorReconstructor_ );

  anchorPlanner_->PlanAnchors( featurePairLifeWeight, 
		  featurePairDistanceWeight, continuationThreshold, anchorImageNumber );  
  {
    const deque<FeaturePair*> & anchorList = anchorPlanner_->GetAnchorList( );
	 std::vector<FeaturePair*> tmp( anchorList.size() );
    copy( anchorList.begin(), anchorList.end(), tmp.begin() );
    anchorList_.swap( tmp );
  }

  for ( int i = 0; i<reconObjectList_.size(); i++ )
  { if ( reconObjectList_[i].GetReconFlag() == ReconObject::RECON_ANCHOR )
    { int j, k, n, lowerBound;

      projErrorList_[i] = 0;

      n = featureList_[i].GetRangeSize();
      std::vector<int> validImage( n );
      lowerBound = featureList_[i].GetRange().GetLowerBound();
      for ( j=0, k=0; j<n; j++ ) {
        if( viewpointValidOrientationList_[lowerBound+j] ) {
          validImage[k] = lowerBound + j;
          k++;
        }
      }

      validImage.resize( k );

      for ( j=0; j<validImage.size(); j++ )
      { VM4Matrix inverseViewpointOrientation( 
				viewpointOrientationList_[validImage[j]] );
	     VM4Matrix viewpointOrientation( 
				viewpointOrientationList_[validImage[j]] );

	     inverseViewpointOrientation.invertAnglePreserving();

        VM3Vector C_Q = inverseViewpointOrientation * 
			  ( reconObjectList_[i].GetReconLocation() - 
				 viewpointLocationList_[validImage[j]] );

	     float z = C_Q.z();
        VM3Vector C_q1 = C_Q / z * ( -cameraParameters_.focus );


		VM3Vector C_q2( featureList_[i].GetLocation(validImage[j]).x_ * cameraParameters_.realLength,
			            featureList_[i].GetLocation(validImage[j]).y_ * cameraParameters_.realWidth,
					    -cameraParameters_.focus );

		C_q1 = viewpointOrientation * C_q1;
		C_q2 = viewpointOrientation * C_q2;

        float tempError = ( C_q1 - C_q2 ).length() + estimatedTrackingError * cameraParameters_.realWidth;

		float cos1 = C_q1.dot( C_q1 - C_q2 ) / ( C_q1.length() * ( C_q1 - C_q2 ).length() );
		float sin1 = sqrt( ABS( 1 - cos1 * cos1 ) );
		float cos2 = cos( cameraOrientationNoise );
		float sin2 = sin( cameraOrientationNoise );
		float sin3 = sin1 * cos2 + cos1 * sin2;
		float cos4 = C_q2.dot( C_q2 - C_q1 ) / ( C_q2.length() * ( C_q2 - C_q1 ).length() );
		float sin4 = sqrt( ABS( 1 - cos4 * cos4 ) );

        tempError += C_q1.length() / sin3 * sin2;
		//tempError += cameraPositionNoise;
		tempError += C_q1.length() / C_Q.length() * cameraPositionNoise;

		tempError = tempError * C_Q.length() / C_q1.length() * sin4;

        projErrorList_[i] += tempError;
        
	  }

	  float sinAngle = sqrt( ABS( 1 - reconObjectList_[i].GetCosAngle() / 2 ) );
	  projErrorList_[i] = projErrorList_[i] / validImage.size() / sinAngle;
	}
  }

  /*int i, j, k, featureID1 = -1, featureID2 = -1;
  float realLength = cameraParameters_.realLength;
  float realWidth = cameraParameters_.realWidth;
  float focus = cameraParameters_.focus;
  VM3Vector coord2[2];

  for ( i=0; i<anchorList_.size(); i++ )
  { if ( anchorList_[i] && ( ( featureID1 != anchorList_[i]->GetFirstFeature().GetID() ) || ( featureID2 != anchorList_[i]->GetSecondFeature().GetID() ) ) )
    { featureID1 = anchorList_[i]->GetFirstFeature().GetID();
      featureID2 = anchorList_[i]->GetSecondFeature().GetID();

	  int n = anchorList_[i]->GetImageRange().GetSize();
      std::vector<int> validImage( n );
      int lowerBound = anchorList_[i]->GetImageRange().GetLowerBound();
      for ( k=0, j=0; k<n; k++ )
      { if( viewpointValidOrientationList_[lowerBound+k] )
        { validImage[j] = lowerBound + k;
          j++;
        }
      }
      validImage.resize( j );
      projErrorList_[featureID1] = 0;
      projErrorList_[featureID2] = 0;
      for ( k=0; k<validImage.size(); k++ )
      { coord2[0].set( featureList_[featureID1].GetLocation(validImage[k]).x_ * realLength,
          featureList_[featureID1].GetLocation(validImage[k]).y_ * realWidth,
          -focus );
      coord2[1].set( featureList_[featureID2].GetLocation(validImage[k]).x_ * realLength,
          featureList_[featureID2].GetLocation(validImage[k]).y_ * realWidth,
          -focus );

      float C_q1 = coord2[0].length();
      float C_q2 = coord2[1].length();
      float q1_q2 = ( coord2[0] - coord2[1] ).length();

      float cos1 = coord2[0].dot( coord2[0] - coord2[1] )/( C_q1 * q1_q2 );
      float sin1 = sqrt( 1 - cos1 * cos1 );

      VM3Vector C_Q1 = reconObjectList_[featureID1].GetReconLocation() - viewpointLocationList_[validImage[k]];
      VM3Vector C_Q2 = reconObjectList_[featureID2].GetReconLocation() - viewpointLocationList_[validImage[k]];

      float cos2 = C_Q1.dot( C_Q2 )/( C_Q1.length() * C_Q2.length() );
      float sin2 = sqrt( 1 - cos2 * cos2 );

      float sin3 = sin1 * cos2 + cos1 * sin2;
      float q1_q3 = C_q1 * sin2 / sin3;

      float tempError = ( ABS( q1_q3 - q1_q2 ) + estimatedTrackingError * realLength ) * C_Q2.length() / C_q2 * sin3;

      //if ( projError[featureID2] < tempError )
      projErrorList_[featureID2] += tempError;

      cos1 = coord2[1].dot( coord2[1] - coord2[0] )/( C_q2 * q1_q2 );
      sin1 = sqrt( 1 - cos1 * cos1 );

      sin3 = sin1 * cos2 + cos1 * sin2;
      float q2_q3 = C_q2 * sin2 / sin3;

      tempError = ( ABS( q2_q3 - q1_q2 ) + estimatedTrackingError * realLength ) * C_Q1.length() / C_q1 * sin3;

      //if ( projError[featureID1] < tempError )
      projErrorList_[featureID1] += tempError;
      }

      float sinAngle = sqrt( ( 1 - reconObjectList_[featureID1].GetCosAngle() ) / 2 );
      projErrorList_[featureID1] = projErrorList_[featureID1] / validImage.size() / sinAngle;

      sinAngle = sqrt( ( 1 - reconObjectList_[featureID2].GetCosAngle() ) / 2 );
      projErrorList_[featureID2] = projErrorList_[featureID2] / validImage.size() / sinAngle;
    }
  }*/
}

/*****************************************************************************
*****************************************************************************/
void SceneData::ReconstructNonanchors( int nonanchorImageNumber, float estimatedTrackingError, float cameraPositionNoise, float cameraOrientationNoise )
{
  int i, j, k, n, ii;
  int outliers[100];
  int numOutliers = 0;
  outliers[0] = 0;
  FILE * fp;
  if ( fp = fopen( "outliers.txt", "r" ) )
  {
      fscanf( fp, "%d\n\n", &numOutliers );
	  for ( ii=0; ii<numOutliers; ii++ )
		  fscanf( fp, "%d\n", outliers+ii );
	  fclose( fp );
  }

  if( nonanchorReconstructor_ == 0 )
    nonanchorReconstructor_ = new NonanchorReconstructor( *this );

  for ( i=0; i<featureList_.size(); i++ ) {
    try {
		if ( reconObjectList_[i].GetReconFlag() == ReconObject::RECON_NONE && !isOutlier( i, numOutliers, outliers ) )
        nonanchorReconstructor_->ReconstructNonanchorFeature( featureList_[i], projErrorList_[i], nonanchorImageNumber );
    }
    catch( const std::exception & e ) {
      cerr << e.what() << endl;
    }
  }
  
  float error = 0.0;
  int imageNum = 0;

  for ( i = 0; i<reconObjectList_.size(); i++ )
  { 
    if ( reconObjectList_[i].GetReconFlag() == ReconObject::RECON_NONANCHOR )
    { int lowerBound;
	  VM3Vector *C_q2;

      projErrorList_[i] = 0;

      n = featureList_[i].GetRangeSize();
      std::vector<int> validImage( n );
      lowerBound = featureList_[i].GetRange().GetLowerBound();
      for ( j=0, k=0; j<n; j++ ) {
        if( viewpointValidOrientationList_[lowerBound+j] ) {
          validImage[k] = lowerBound + j;
          k++;
        }
      }

      validImage.resize( k );
	  
	  C_q2 = (VM3Vector *) malloc( validImage.size() * sizeof( VM3Vector ) );

      for ( j=0; j<validImage.size(); j++ )
      { VM4Matrix viewpointOrientation( viewpointOrientationList_[validImage[j]] );
	    VM4Matrix inverseViewpointOrientation( viewpointOrientationList_[validImage[j]] );

	    inverseViewpointOrientation.invertAnglePreserving();

		VM3Vector C_Q = inverseViewpointOrientation* ( 
				 reconObjectList_[i].GetReconLocation() - 
				 viewpointLocationList_[validImage[j]] );

	    float z = C_Q.z();
		VM3Vector C_q1 = C_Q / z * ( -cameraParameters_.focus );

		C_q2[j].set( featureList_[i].GetLocation(validImage[j]).x_ * cameraParameters_.realLength,
			         featureList_[i].GetLocation(validImage[j]).y_ * cameraParameters_.realWidth,
					 -cameraParameters_.focus );

		C_q1 = viewpointOrientation * C_q1;
		C_q2[j] = viewpointOrientation * C_q2[j];

		//cout<<C_q1.x()-C_q2[j].x()<<" "<<C_q1.y()-C_q2[j].y()<<" "<<C_q1.z()-C_q2[j].z()<<endl;

        float tempError = ( C_q1 - C_q2[j] ).length();
		
		tempError += estimatedTrackingError * cameraParameters_.realWidth;

		float cos1 = C_q1.dot( C_q1 - C_q2[j] ) / ( C_q1.length() * ( C_q1 - C_q2[j] ).length() );
		float sin1 = sqrt( ABS( 1 - cos1 * cos1 ) );
		float cos2 = cos( cameraOrientationNoise );
		float sin2 = sin( cameraOrientationNoise );
		float sin3 = sin1 * cos2 + cos1 * sin2;
		float cos4 = C_q2[j].dot( C_q2[j] - C_q1 ) / ( C_q2[j].length() * ( C_q2[j] - C_q1 ).length() );
		float sin4 = sqrt( ABS( 1 - cos4 * cos4 ) );

		//tempError += cameraPositionNoise;
		tempError += C_q1.length() / sin3 * sin2;
		tempError += C_q1.length() / C_Q.length() * cameraPositionNoise;

		tempError = tempError * C_Q.length() / C_q1.length() * sin4;
		
		projErrorList_[i] += tempError;
        
	  }

	  float cosAngleTemp, cosAngle = 1;

	  for ( j=0; j<validImage.size()-1; j++ )
      { for ( k=j+1; k<validImage.size(); k++ )
        { cosAngleTemp = C_q2[j].dot( C_q2[k] ) / ( C_q2[j].length() * C_q2[k].length() );
          if ( ABS( cosAngle ) > ABS( cosAngleTemp ) )
          {  cosAngle = cosAngleTemp;
	      }
        }
      }

      float sinAngle = sqrt( ABS( ( 1 - cosAngle ) / 2 ) );
	  projErrorList_[i] = projErrorList_[i] / validImage.size();

	  free( C_q2 );
	}
  }
}
/*****************************************************************************
*****************************************************************************/
bool SceneData::OutputDistanceBetweenScenes( SceneData sceneData, ostream outflow )
{
	int i=0;
	if (objectLocationList_.size() != sceneData.GetObjectLocationList().size())
		return false;
	

	outflow.precision(6);
	outflow << "NumObjects " << objectLocationList_.size() << endl;
	outflow << "Below is the difference between the 3-D reconstructed objects (not a percentage)." << endl << endl;
	for (i=0; i < objectLocationList_.size(); i++)
	{
		outflow << i << " " << objectLocationList_[i].x() - sceneData.GetObjectLocationList()[i].x() << " "
	             << objectLocationList_[i].y() - sceneData.GetObjectLocationList()[i].y() << " "
				 << objectLocationList_[i].z() - sceneData.GetObjectLocationList()[i].z() << endl;
	}


	return true;
}


/*****************************************************************************
*****************************************************************************/
void SceneData::LoadFeatureData( std::istream & input, float trackingNoise, 
		float cameraPositionNoise, float cameraOrientationNoise )
{
  unsigned lineNum = 0;



  try {

    //
    // Seed the RNG.
    //
    srand( time( 0 ) );
	
    //
    // Read the parameters of the camera.
    //
    stringstream ss = ReadLine( input, lineNum );
    string cameraParameters_Str;
	 float xAngle, yAngle;
	 ss >> cameraParameters_Str >> xAngle >> yAngle;
	 CheckSyntax( cameraParameters_Str, "CameraParameters" );



    cameraParameters_.focus = 1;
    cameraParameters_.realLength = cameraParameters_.focus * tan( xAngle / 2.0 );
    cameraParameters_.realWidth = cameraParameters_.focus * tan( yAngle / 2.0 );

    //
    // Read the number of viewpoint locations.
    //
    unsigned numViewpoints, firstFrame;
    {
      stringstream ss = ReadLine( input, lineNum );
      string numViewpointsStr;
      string firstFrameStr;
      ss >> numViewpointsStr >> numViewpoints >> firstFrameStr >> firstFrame;
      CheckSyntax( numViewpointsStr, "NumViewpoints" );
      CheckSyntax( firstFrameStr, "FirstFrame" );
    }
	
    //
    // Resize the viewpoint location & orientation lists.
    //
    {
      std::vector<VM3Vector> newLocations( numViewpoints );
	  std::vector<VM3Vector> newReconLocations( numViewpoints );
	  std::vector<int> newAbsoluteImageIDs( numViewpoints );//BOBEDIT Feb14
      std::vector<VM4Matrix> newOrientations( numViewpoints );
      std::vector<bool> newValidOrientations( numViewpoints );
      viewpointLocationList_.swap( newLocations );
	  reconViewpointLocationList_.swap( newReconLocations );
	  absoluteImageIDList_.swap( newAbsoluteImageIDs );//BOBEDIT Feb14
      viewpointOrientationList_.swap( newOrientations );
      viewpointValidOrientationList_.swap( newValidOrientations );
    }


    //
    // Read the viewpoint locations.
    //
    for( int i=0; i<numViewpoints; ++i ) {
      stringstream ss = ReadLine( input, lineNum );
      //cout << "Read line \"" << ss.str() << "\"" << endl;
      //cout.flush();

      float lX, lY, lZ;  // Viewpoint location
	  float VX, VY, VZ;  // View direction
	  float UX=0, UY=1, UZ=0;  // Up direction
	  int imageID;
      ss >> imageID >> lX >> lY >> lZ >> VX >> VY >> VZ >> UX >> UY >> UZ;
      viewpointLocationList_[i].set( lX, lY, lZ );
	  absoluteImageIDList_[i] = ( imageID );


      float sqDist = 1.0f;
      if( i > 0 )
        sqDist = ( viewpointLocationList_[i] - viewpointLocationList_[i-1] ).lengthSquared();

      //
	  // Get the rotation matrix from the heading in the file.
	  //

	  VM3Vector dirZ( VX, VY, VZ );
	  dirZ.normalize();
	  VM3Vector dirY( UX, UY, UZ );
	  dirY = dirY - ( dirY.dot( dirZ ) * dirZ );
	  dirY.normalize();
	  VM3Vector dirX( dirY.cross( dirZ ) );
	  dirX.normalize();
	  viewpointOrientationList_[i].set( 0, 0, dirX.x() );
	  viewpointOrientationList_[i].set( 1, 0, dirX.y() );
	  viewpointOrientationList_[i].set( 2, 0, dirX.z() );
	  viewpointOrientationList_[i].set( 0, 1, dirY.x() );
	  viewpointOrientationList_[i].set( 1, 1, dirY.y() );
	  viewpointOrientationList_[i].set( 2, 1, dirY.z() );
	  viewpointOrientationList_[i].set( 0, 2, dirZ.x() );
	  viewpointOrientationList_[i].set( 1, 2, dirZ.y() );
	  viewpointOrientationList_[i].set( 2, 2, dirZ.z() );
	  viewpointOrientationList_[i].set( 0, 3, 0 );
	  viewpointOrientationList_[i].set( 1, 3, 0 );
	  viewpointOrientationList_[i].set( 2, 3, 0 );
	  viewpointOrientationList_[i].set( 3, 0, 0 );
	  viewpointOrientationList_[i].set( 3, 1, 0 );
	  viewpointOrientationList_[i].set( 3, 2, 0 );
	  viewpointOrientationList_[i].set( 3, 3, 1 );
	  
	  if( sqDist > EPSILON * EPSILON ) {
		  //
          // Mark the orientation as valid.
          //
          viewpointValidOrientationList_[i] = true;
      }
      else
		if( sqDist <= EPSILON * EPSILON ) 
			 viewpointValidOrientationList_[i] = false;
	  }

	viewpointValidOrientationList_[0] = viewpointValidOrientationList_[1];

    //
    // Add noise to the positions & rotations as needed.
    //
    for( int k=0; k<numViewpoints; ++k ) {

		 //
         // Add noise to the positions.
         //
         float lX = viewpointLocationList_[k].x();
         float lY = viewpointLocationList_[k].y();
         float lZ = viewpointLocationList_[k].z();
		 //
		 //uniform distribution
		 //
         //lX += ( 2.0f * frand() - 1.0f ) * cameraPositionNoise;
         //lY += ( 2.0f * frand() - 1.0f ) * cameraPositionNoise;
         //lZ += ( 2.0f * frand() - 1.0f ) * cameraPositionNoise;
		 //
		 //normal distribution
		 //
         lX += gennor( 0, cameraPositionNoise);
         lY += gennor( 0, cameraPositionNoise);
         lZ += gennor( 0, cameraPositionNoise);

		 reconViewpointLocationList_[k].set( lX, lY, lZ );

		 if (cameraOrientationNoise != 0.0) {
          //
          // Add a noise rotation to the orientation.
          //
          if( viewpointValidOrientationList_[k] ) {
             VM3Vector rotNoiseAxis( 2.0f * frand() - 1.0f, 2.0f * frand() - 1.0f, 2.0f * frand() - 1.0f );
             rotNoiseAxis.normalize();
             float rotNoiseAngle = gennor( 0, cameraOrientationNoise );
             VM4Matrix rotNoiseMatrix;
             rotNoiseMatrix.rotate( rotNoiseAxis, rotNoiseAngle );
             viewpointOrientationList_[k] = 
		          rotNoiseMatrix * viewpointOrientationList_[k];
		    }
       }
    }


    //
    // Read the number of features.
    //
    unsigned numFeatures;
    {
      stringstream ss = ReadLine( input, lineNum );
      //cout << "Read line \"" << ss.str() << "\"" << endl;
      //cout.flush();
      string numFeaturesStr;
      ss >> numFeaturesStr >> numFeatures;
      CheckSyntax( numFeaturesStr, "NumFeatures" );
      //cout << "  NumFeatures: " << numFeatures << endl;
      //cout.flush();
    }


    //
    // Resize the feature and reconstructed object list.
    //
	{
      std::vector<Feature> newFeatures( numFeatures );
      std::vector<ReconObject> newReconObjects( numFeatures );
	  std::vector<VM3Vector> newSBAReconLocations( numFeatures );
	  //std::vector<VM3Vector> newTriangulateReconLocations( numFeatures );
      std::vector<float> newProjError( numFeatures );
	  std::vector<VM3Vector> newObjectLocationList( numFeatures );

      featureList_.swap( newFeatures );
      reconObjectList_.swap( newReconObjects );
	  SBAReconLocationList_.swap( newSBAReconLocations );
	  //triangulateReconLocationList_.swap( newTriangulateReconLocations );
	  projErrorList_.swap( newProjError );
	  objectLocationList_.swap( newObjectLocationList );
	}
	
	
	//
    // Read the features.
    //
	for( int j=0; j<numFeatures; ++j ) {
//		cout << "j: " << j << endl;
      //
      // Read the first image ID.
      //      
	  
	  unsigned firstImageID;
      {
        stringstream ss = ReadLine( input, lineNum );
        //cout << "Read line \"" << ss.str() << "\"" << endl;
        //cout.flush();
        string firstImageIDStr;
        ss >> firstImageIDStr >> firstImageID;
        CheckSyntax( firstImageIDStr, "FirstImageID" );
        //cout << "  First Image ID: " << firstImageID << endl;

        //cout.flush();
      }

      //
      // Read the number of projections for the feature.
      //
      unsigned numProjections;
      {
        stringstream ss = ReadLine( input, lineNum );
        string numProjectionsStr;
        ss >> numProjectionsStr >> numProjections;
        CheckSyntax( numProjectionsStr, "NumProjections" );
        //cout.flush();
      }
	  
      //
      // Read the projections.
      //
      std::vector<Feature::Location> projectionLocationList( numProjections );
      for( int k=0; k<numProjections; ++k ) {
        stringstream ss = ReadLine( input, lineNum );
        //cout << "Read line \"" << ss.str() << "\"" << endl;
        float x, y;
        ss >> x >> y;
		if ( k < numProjections )
		{
			x += ( frand() * 2.0f - 1.0f ) * trackingNoise;
			y += ( frand() * 2.0f - 1.0f ) * trackingNoise;
			projectionLocationList[k].x_ = x;
			projectionLocationList[k].y_ = y;
		}
        //cout << "  Got Location " << x << ", " << y << endl;
        //cout.flush();
      }

      //
      // Create the Feature instance.
      //
      featureList_[j] = Feature( j, firstImageID-firstFrame, 
				                     projectionLocationList );
    }
    //
    // Read the number of objects if it's there.
    //
    unsigned numObjects = 0;
	bool areObjects = true;
    try {
      stringstream ss = ReadLine( input, lineNum );
      string numObjectsStr;
      ss >> numObjectsStr >> numObjects;
      if( !numObjectsStr.empty() )
        CheckSyntax( numObjectsStr, "NumObjects" );
    }
    catch( SyntaxException & e ) {areObjects = false;} // Don't do anything if get EOF exception.

	if (areObjects == true)
	{
	 //
	    // Read the object data.
	    //
	    {
	      std::vector<VM3Vector> tmp;
	      objectLocationList_.swap( tmp );
	    }
	    if( numObjects == numFeatures ) {
	      objectLocationList_.resize( numObjects );
		  for( int i=0; i<numObjects; ++i ) {
	        stringstream ss = ReadLine( input, lineNum );
	        float x, y, z;
	        ss >> x >> y >> z;
	        objectLocationList_[i].set( x, y, z );
	      }
		}
	}

    //
    // Delete the old anchor planner.
    //
    if( anchorPlanner_ != 0 ) {
      delete anchorPlanner_;
      anchorPlanner_ = 0;
    }

    //
    // Estimate the orientations.
    //
    //EstimateViewpointOrientations();

    //
    // Weed out the invalid features.
    //
    //RemoveBadFeatures();

  }
  catch( const SyntaxException & e ) {
    stringstream ss;
    ss << "Syntax error encountered at line " << lineNum << ": " << e.what();
    throw runtime_error( ss.str() );
  }

}


/*****************************************************************************
*****************************************************************************/
void SceneData::SaveFeatureData( ostream & output ) const
{
  //
  // Write the viewpoint locations.
  //	
  int i, j;
  
  output << "CameraParameters" << " " <<atan( cameraParameters_.realLength / cameraParameters_.focus ) * 2.0
	                           << " " <<atan( cameraParameters_.realWidth / cameraParameters_.focus ) * 2.0
							   << endl << endl;
	  
	  
  output << "NumViewpoints " << viewpointLocationList_.size() << " FirstFrame 0" << endl;
  for( i=0; i<viewpointLocationList_.size(); ++i ) {
		output << i << " " << viewpointLocationList_[i].x() << " "
                       << viewpointLocationList_[i].y() << " "
                       << viewpointLocationList_[i].z() << " "
					   << viewpointOrientationList_[i]( 0, 2 ) << " "
					   << viewpointOrientationList_[i]( 1, 2 ) << " "
					   << viewpointOrientationList_[i]( 2, 2 ) << " "
					   << viewpointOrientationList_[i]( 0, 1 ) << " "
					   << viewpointOrientationList_[i]( 1, 1 ) << " "
					   << viewpointOrientationList_[i]( 2, 1 ) << " "
					   << endl;
  }
  output << endl;

  //
  // Write the features.
  //
  output << "NumFeatures " << featureList_.size() << endl;
  for( i=0; i<featureList_.size(); ++i ) 
  {	  
		  IntegerRange imageRange = featureList_[i].GetRange();
		  output << endl << "FirstImageID " << imageRange.GetLowerBound() << " " << i << endl;
		  output << "NumProjections " << featureList_[i].GetRangeSize() << endl;
		  for( j=imageRange.GetLowerBound(); j<imageRange.GetUpperBound(); ++j ) 
		  {
			  const Feature::Location & loc = featureList_[i].GetLocation( j );
			  output << loc.x_ << " " << loc.y_ << endl;
		  }
  }

  output << endl;

  if ( HasObjectLocations() )
  {
	  output << "NumObjects " << objectLocationList_.size() << endl;
		for ( i=0; i<reconObjectList_.size(); i++ )
			  output << reconObjectList_[i].GetReconLocation().x() << " "
	             << reconObjectList_[i].GetReconLocation().y() << " "
				 << reconObjectList_[i].GetReconLocation().z() << " "
				 << i << endl;
  }
}


/*****************************************************************************
*****************************************************************************/
bool VectorEqual( VM3Vector & a, VM3Vector & b )
{  return ( ( ABS( a.x() - b.x() ) < EPSILON ) && 
    ( ABS( a.y() - b.y() ) < EPSILON ) &&
    ( ABS( a.z() - b.z() ) < EPSILON ) );
}


/*****************************************************************************
*****************************************************************************/
void SceneData::EstimateViewpointOrientations()
{ int i, j;  
  float r, x, z;  
  VM3Vector robotDirection, cameraDirection;
  VM4Matrix turnLeft(
      0, 0, 1, 0,
      0, 1, 0, 0,
      -1, 0, 0, 0,
      0, 0, 0, 1);

  VM4Matrix turnRight(
      0, 0, -1, 0,
      0, 1, 0, 0,
      1, 0, 0, 0,
      0, 0, 0, 1);

  VM4Matrix noTurn(
      1, 0, 0, 0,
      0, 1, 0, 0,
      0, 0, 1, 0,
      0, 0, 0, 1);

  for ( i=0; i<viewpointLocationList_.size()-1; i++ )  
  { if ( VectorEqual( viewpointLocationList_[i+1], viewpointLocationList_[i] ) ) 
    { viewpointValidOrientationList_[i] = false;
      continue;
    }
    viewpointValidOrientationList_[i] = true;
    robotDirection = viewpointLocationList_[i+1] - viewpointLocationList_[i];
    cameraDirection = turnRight * robotDirection;    
    x = cameraDirection.x();
    z = cameraDirection.z();
    r = sqrt( x * x + z * z );
    viewpointOrientationList_[i].zero();
    viewpointOrientationList_[i].set( 0, 0, -z/r );
    viewpointOrientationList_[i].set( 0, 2, -x/r );
    viewpointOrientationList_[i].set( 1, 1, 1 );
    viewpointOrientationList_[i].set( 2, 0, x/r );
    viewpointOrientationList_[i].set( 2, 2, -z/r );
    viewpointOrientationList_[i].set( 3, 3, 1 );
  }

  viewpointValidOrientationList_[i] = false;
}


/*****************************************************************************
*****************************************************************************/
void SceneData::RemoveBadFeatures()
{
  std::vector<Feature>::iterator first, last, firstBadFeature;
  first = featureList_.begin();
  last = firstBadFeature = featureList_.end();

  for( ; first != last; ++first ) {
    if( IsBadFeature( *first ) )
      swap( *first, *(--firstBadFeature) );
  }

  cout << "Eliminated " << featureList_.end() - firstBadFeature << 
	  " bad features." << endl;
  featureList_.erase( firstBadFeature, featureList_.end() );
}


/*****************************************************************************
*****************************************************************************/
bool SceneData::IsBadFeature( const Feature & feature ) const
{
  IntegerRange imageRange = feature.GetRange();
  const float halfImageWidth3D = cameraParameters_.realWidth;
  const float halfImageHeight3D = cameraParameters_.realLength;
  const float nearDistance = cameraParameters_.focus;
  const float threshold = 0.1;

  for( int i=imageRange.GetLowerBound(); i<imageRange.GetUpperBound()-1; ++i ) {
    //
    // We can only do this if we have orientations for the two images.
    //
    if( viewpointValidOrientationList_[i] && 
			 viewpointValidOrientationList_[i+1] ) {
      const Feature::Location & proj1 = feature.GetLocation( i );
      const Feature::Location & proj2 = feature.GetLocation( i+1 );

      //
      // Get 3-space location of the first projection on the image plane and the
      // direction vector from the CoP to the point.
      // 
      VM3Vector dir1( halfImageWidth3D * proj1.x_, 
				          halfImageHeight3D * proj1.y_, 
							 nearDistance );
      dir1 = viewpointOrientationList_[i] * dir1;
      dir1.normalize();

      //
      // Get the 3-space x- and y-directions of the second image's plane.
      //
      const VM4Matrix & orient2 = viewpointOrientationList_[i+1];
      VM3Vector x2( orient2( 0, 0 ), orient2( 1, 0 ), orient2( 2, 0 ) );
      VM3Vector y2( orient2( 0, 1 ), orient2( 1, 1 ), orient2( 2, 1 ) );

      //
      // Project the first image's direction vector into the second image.
      //
      VM2Vector projDir1( dir1.dot( x2 ) / halfImageWidth3D, 
				              dir1.dot( y2 ) / halfImageHeight3D );
      projDir1.normalize();

      //
      // Project the first CoP onto the second image plane.
      //
      VM3Vector & relativeCop1 = viewpointLocationList_[i] - 
			viewpointLocationList_[i+1];
      VM2Vector projCop1( relativeCop1.dot( x2 ) / halfImageWidth3D, 
				              relativeCop1.dot( y2 ) / halfImageHeight3D );

      //
      // Compute epipolar line & find the distance of the 2nd projection to it.
      //
      VM2Vector epiNormal( -projDir1.y(), projDir1.x() );
      float epiConst = - epiNormal.dot( projCop1 );
      VM2Vector proj2Vec( proj2.x_, proj2.y_ );
      float dist2 = epiNormal.dot( proj2Vec ) + epiConst;

      //
      // If second projection is too far off epipolar line, then feature bad.
      //
      if( dist2 > threshold )
        return true;
    }
  }

  return false;
}

/*****************************************************************************
*****************************************************************************/
void SceneData::WriteViewpointOrientation( VM4Matrix Orientation, int frameID )
{
	viewpointOrientationList_[frameID] = Orientation;
}

/*****************************************************************************
*****************************************************************************/
void SceneData::WriteViewpointLocation( VM3Vector Location, int frameID )
{
	viewpointLocationList_[frameID] = Location;
}

/*****************************************************************************
 *****************************************************************************/
void SceneData::OutputBundleFile( )
{ 
	int i,j;
    	
	char file1[256];
	sprintf( file1, "pts1.txt" );
	FILE* fp1 = fopen( file1, "w" );
	if ( fp1 == 0 )
		fprintf( stderr, "UNable to open pts1.txt\n" );

	fprintf( fp1, "# X Y Z  nframes  frame0 x0 y0  frame1 x1 y1 ...\n" );

	for ( i=0; i<reconObjectList_.size(); i++ )
	{ 
		if ( reconObjectList_[i].GetReconFlag() != ReconObject::RECON_NONE )
		{  
			int rangeSize = featureList_[i].GetRangeSize();
			int lowerBound = featureList_[i].GetRange().GetLowerBound();
			VM3Vector reconLocation = reconObjectList_[i].GetReconLocation();
			fprintf( fp1, "%f %f %f  %d  ", reconLocation.x(), reconLocation.y(), -reconLocation.z(), rangeSize );
			
			for ( j=0; j<rangeSize; j++ )
				fprintf( fp1, "%d %f %f ", lowerBound+j, featureList_[i].GetLocation( lowerBound+j ).x_*512, 
				                                         featureList_[i].GetLocation( lowerBound+j ).y_*384 );
			
			fprintf( fp1, "\n" );
	  }
	}

	fclose( fp1 );

	char file3[256];
	sprintf( file3, "cams.txt" );
	FILE* fp3 = fopen( file3, "w" );
	if ( fp3 == 0 )
		fprintf( stderr, "UNable to open cams.txt\n" );

	for ( i=0; i<viewpointLocationList_.size(); i++ )
	{  
		VM3Vector newViewpointLocation = reconViewpointLocationList_[i];
		VM4Matrix newViewpointOrientation = viewpointOrientationList_[i];
		newViewpointOrientation.invertAnglePreserving();
		newViewpointLocation = newViewpointOrientation * reconViewpointLocationList_[i];
		newViewpointLocation.set( -newViewpointLocation.x(), -newViewpointLocation.y(), -newViewpointLocation.z() );

	   float result[4];
	   VM4Matrix m( viewpointOrientationList_[i] );

       float trace = m(0,0) + m(1,1) + m(2,2);
       if( trace > 0.0f ) 
	   {
         result[0] = 0.5f * sqrt( trace + 1.0f );
         float inv4W = 0.25f / result[0];
         result[1] = inv4W * ( m(2,1) - m(1,2) );
         result[2] = inv4W * ( m(0,2) - m(2,0) );
         result[3] = inv4W * ( m(1,0) - m(0,1) );
       }
       else 
	   {
         if( m(0,0) > m(1,1) && m(0,0) > m(2,2) ) 
		 {
           result[1] = 0.5f * sqrt( m(0,0) - m(1,1) - m(2,2) + 1.0f );
           float inv4X = 0.25f / result[1];
           result[0] = inv4X * ( m(2,1) - m(1,2) );
           result[2] = inv4X * ( m(1,0) + m(0,1) );
           result[3] = inv4X * ( m(2,0) + m(0,2) );
         }
         else if( m(1,1) > m(0,0) && m(1,1) > m(2,2) ) 
		 {
           result[2] = 0.5f * sqrt( m(1,1) - m(0,0) - m(2,2) + 1.0f );
           float inv4Y = 0.25f / result[2];
           result[0] = inv4Y * ( m(0,2) - m(2,0) );
           result[1] = inv4Y * ( m(1,0) + m(0,1) );
           result[3] = inv4Y * ( m(2,1) + m(1,2) );
         }
         else 
		 {
           result[3] = 0.5f * sqrt( m(2,2) - m(0,0) - m(1,1) + 1.0f );
           float inv4Z = 0.25f / result[3];
           result[0] = inv4Z * ( m(1,0) - m(0,1) );
           result[1] = inv4Z * ( m(2,0) + m(0,2) );
           result[2] = inv4Z * ( m(2,1) + m(1,2) );
         }
       }

       fprintf( fp3, "%f %f %f %f %f %f %f\n", result[0], result[1], result[2], -result[3], 
	                                      newViewpointLocation.x(), newViewpointLocation.y(), -newViewpointLocation.z() );
	}
	fclose( fp3 );
}

/*****************************************************************************
*****************************************************************************/
float SceneData::InputBundleResult( char * file )
{
	float s, u, v, w, x, y, z, error;
	FILE * fp;
	fp = fopen( file, "r" );
	
	SBAReconLocationList_.resize( reconObjectList_.size() );
	error = 0.0;
	
	for ( int i=0; i<reconViewpointLocationList_.size(); i++ )
	{
		fscanf( fp, "%f %f %f %f %f %f %f\n", &s, &u, &v, &w, &x, &y, &z );
		w=-w;
		VM4Matrix rotation( 1-2*v*v-2*w*w, 2*u*v-2*w*s,   2*u*w+2*s*v,   0,
			                2*u*v+2*w*s,   1-2*u*u-2*w*w, 2*v*w-2*s*u,   0,
							2*u*w-2*s*v,   2*v*w+2*s*u,   1-2*u*u-2*v*v, 0,
							0,             0,             0,             1 );
		
		VM3Vector viewpointLocation( -x, -y, z );
		reconViewpointLocationList_[i] = rotation * viewpointLocation;
	}

	int count = 0;	
	for ( i=0; i<reconObjectList_.size(); i++ )
	{
		if ( reconObjectList_[i].GetReconFlag() != ReconObject::RECON_NONE )
		{
			fscanf( fp, "%f %f %f\n", &x, &y, &z );
			SBAReconLocationList_[i].set( x, y, z );
			if ( HasObjectLocations() )
			{
				float length = ( SBAReconLocationList_[i] - objectLocationList_[i] ).length();
				error += length;
				if ( length > 50 )
					count++;
			}
		}
	}
	
	//printf( "outliers for BA: %d\n", count );

	if ( HasObjectLocations() )
		error /= objectLocationList_.size();

	fclose( fp );
	return error;
}

/*****************************************************************************
*****************************************************************************/
void SceneData::AddPositionNoise( float positionNoise )
{
	int i;
	
	//cerr << "hmm1" << SBAReconLocationList_.size() << endl;
	SBAReconLocationList_.resize( reconObjectList_.size() );
	

	//cerr << "hmm2" << SBAReconLocationList_.size() << " " << objectLocationList_.size() << endl;
	for ( i=0; i<reconObjectList_.size(); i++ )
	{
		reconObjectList_[i].SetNonanchorReconstruction( objectLocationList_[i], 0, 0, 0 );
		SBAReconLocationList_[i] = objectLocationList_[i];
		
		if ( positionNoise != 0.0 )
		{
			VM3Vector randPosition( gennor( 0, positionNoise), 
			                        gennor( 0, positionNoise), 
									gennor( 0, positionNoise) );
			
			reconObjectList_[i].SetNonanchorReconstruction( objectLocationList_[i] + randPosition, 0, 0, 0 );
			SBAReconLocationList_[i] = reconObjectList_[i].GetReconLocation();
		}
	}
}

void SceneData::Triangulate( )
{
	triangulateReconLocationList_.resize( objectLocationList_.size() );
	
	for ( int i=0; i<featureList_.size(); i++ )
	{
		  int lowerBound = featureList_[i].GetRange().GetLowerBound();
		  int upperBound = featureList_[i].GetRange().GetUpperBound();

		  VM3Vector C1 = viewpointLocationList_[lowerBound];
		  VM3Vector C2 = viewpointLocationList_[upperBound-1];
		  
		  VM3Vector coord1( featureList_[i].GetLocation( lowerBound ).x_ * cameraParameters_.realLength,
                            featureList_[i].GetLocation( lowerBound ).y_ * cameraParameters_.realWidth,
                            -cameraParameters_.focus );
		  VM3Vector coord2( featureList_[i].GetLocation( upperBound-1 ).x_ * cameraParameters_.realLength,
                            featureList_[i].GetLocation( upperBound-1 ).y_ * cameraParameters_.realWidth,
                            -cameraParameters_.focus );
		  
		  coord1 = viewpointOrientationList_[lowerBound] * coord1;
		  coord2 = viewpointOrientationList_[upperBound-1] * coord2;
		  
		  float k1 = ( coord1.dot( C2 - C1 ) * coord2.dot( coord2 ) - coord2.dot( C2 - C1 ) * coord1.dot( coord2 ) ) /
                     ( coord1.dot( coord1 ) * coord2.dot( coord2 ) - coord1.dot( coord2 ) * coord1.dot( coord2 ) );
		  
		  float k2 = ( k1 * coord1.dot( coord1 ) - coord1.dot( C2-C1 ) ) / coord1.dot( coord2 );

		  VM3Vector reconLocation = C1 + k1 * coord1;
		  reconLocation = reconLocation + C2 + k2 * coord2;
		  reconLocation = reconLocation / 2;

		  reconObjectList_[i].SetNonanchorReconstruction( reconLocation, 0, 0, 0 );
		  triangulateReconLocationList_[i] = reconLocation;
	}
}

/*****************************************************************************
*****************************************************************************/
#if 1
float SceneData::ReconstructBySubsets( int imageNumber )
{
	std::vector<int> anchorPairList;
	anchorPairList.resize( featureList_.size() ); 
	
	for ( int i=0; i<anchorPairList.size(); i++ )
	{
		int maxLifeFeatureID = 0;
		for ( int j=0; j<anchorPairList.size(); j++ )
		{
			if ( j!=i )
			{
				FeaturePair tempFeaturePair( featureList_[i], featureList_[j] );
				if ( tempFeaturePair.GetImageRange().GetSize() > maxLifeFeatureID )
				{
					maxLifeFeatureID = tempFeaturePair.GetImageRange().GetSize();
					anchorPairList[i] = j;
				}
			}
		}
		//printf("%d %d\n",anchorPairList[i],maxLifeFeatureID);
	}

	float err = 0.0;
	int count = 0;
	for ( i=0; i<reconObjectList_.size(); i++ )
	{
		FeaturePair pair( featureList_[i], featureList_[anchorPairList[i]] );
		
		if( anchorReconstructor_ == 0 )
			anchorReconstructor_ = new AnchorReconstructor( *this );

		anchorReconstructor_->ReconstructAnchorPair( pair, imageNumber );

		float length = ( reconObjectList_[i].GetReconLocation() - objectLocationList_[i] ).length();
		err += length;

		if ( length > 50 )
			count++;
	}

	printf( "outliers for IM: %d\n", count );

	return err / reconObjectList_.size();
}
#endif

#if 0
float SceneData::ReconstructAll1( )
{
	for ( i=0; i<16; i++ )
	{
		int iter, j, k;
		float *p, fret;
		VM2Point **imageLocation;
		VM3Vector Q[6], C[3], coord2[6][3], reconAnchor;

		p = ( float * ) malloc( 46 * sizeof( float ) );
		imageLocation = ( VM2Point ** ) malloc( 6 * sizeof( VM2Point * ) );

		for ( j=0; j<6; j++ )
			imageLocation[j] = ( VM2Point * ) malloc( 3 * sizeof( VM2Point ) );

		for ( j=0; j<6; j++ )
		{
			Q[j] = reconObjectList_[6*i+j].GetReconLocation();
			
			p[1+3*j] = Q[j].x();
			p[2+3*j] = Q[j].y();
			p[3+3*j] = Q[j].z();
		}

		for ( j=0; j<3; j++ )
		{
			C[j] = viewpointLocationList_[16*j+i];

			p[19+3*j] = C[j].x();
			p[20+3*j] = C[j].y();
			p[21+3*j] = C[j].z();
		}

		for ( j=0; j<6; j++ )
		{
			for ( k=0; k<3; k++ )
			{
				imageLocation[j][k].set( featureList_[6*i+j].GetLocation(16*k+i).x_, featureList_[6*i+j].GetLocation(16*k+i).y_ );
				
				coord2[j][k].set( featureList_[6*i+j].GetLocation(16*k+i).x_ * cameraParameters_.realLength,
					              featureList_[6*i+j].GetLocation(16*k+i).y_ * cameraParameters_.realWidth,
								  -cameraParameters_.focus );

				p[28+3*j+k] = sqrt( ( Q[j]-C[k] ).dot( Q[j]-C[k] ) / coord2[j][k].dot( coord2[j][k] ) );
			}
		}
		
		frprmn( *this, imageLocation, p, 45, FTOL, &iter, &fret, func, dfunc );

		for ( j=0; j<6; j++ )
		{
			reconAnchor.set( p[1+3*j], p[2+3*j], p[3+3*j] );			
			reconObjectList_[6*i+j].SetAnchorReconstruction( reconAnchor, 0 );
		}

		free( p );

		for ( j=0; j<6; j++ )
			free( imageLocation[j] );

		free ( imageLocation );
	}

	float err = 0.0;
	int count = 0;
	for ( i=0; i<reconObjectList_.size(); i++ )
	{
		float length = ( reconObjectList_[i].GetReconLocation() - objectLocationList_[i] ).length();
		err += length;
		if ( length > 50 )
			count++;
	}

	printf( "outlier for IM: %d\n", count );

	return err / reconObjectList_.size();
}
#endif

struct ImageProjection
{
	VM3Vector *projection;
	VM3Vector *object;
	float *gamma;
	int *featureID;
	int size;
};

float SceneData::ReconstructAllPoints( )
{
	int i, iter, j, gammaID=0;
	float *p, fret;
	VM3Vector *cameraCenter, reconPoint;
	ImageProjection *imageProjection;

	cameraCenter = ( VM3Vector * ) malloc( reconViewpointLocationList_.size() * sizeof( VM3Vector ) );
	imageProjection = ( ImageProjection * ) malloc( viewpointLocationList_.size() * sizeof( ImageProjection ) );

	for ( i=0; i<viewpointLocationList_.size(); i++ )
		cameraCenter[i] = reconViewpointLocationList_[i];

	float scale = ( reconObjectList_[0].GetReconLocation() - cameraCenter[0] ).length();
	
	for ( i=0; i<viewpointLocationList_.size(); i++ )
	{
		imageProjection[i].projection = ( VM3Vector * ) malloc( featureList_.size() * sizeof( VM3Vector ) );
		imageProjection[i].object = ( VM3Vector * ) malloc( featureList_.size() * sizeof( VM3Vector ) );
		imageProjection[i].gamma = ( float * ) malloc( featureList_.size() * sizeof( float ) );
		imageProjection[i].featureID = ( int * ) malloc( featureList_.size() * sizeof( int ) );
		imageProjection[i].size = 0;
	  
		for ( j=0; j<featureList_.size(); j++ )
		{
			if ( featureList_[j].GetRange().GetLowerBound() <= i && featureList_[j].GetRange().GetUpperBound() > i )
			{
				imageProjection[i].projection[imageProjection[i].size].set( featureList_[j].GetLocation(i).x_ * cameraParameters_.realLength,
				                                                            featureList_[j].GetLocation(i).y_ * cameraParameters_.realWidth,
														                    -cameraParameters_.focus );
				imageProjection[i].object[imageProjection[i].size] = reconObjectList_[j].GetReconLocation();
				imageProjection[i].gamma[imageProjection[i].size] = sqrt( ABS( ( imageProjection[i].object[imageProjection[i].size]-cameraCenter[i] ).dot
					                                                           ( imageProjection[i].object[imageProjection[i].size]-cameraCenter[i] ) 
					                                               / scale / scale / imageProjection[i].projection[imageProjection[i].size].dot
																     ( imageProjection[i].projection[imageProjection[i].size] ) ) );
				imageProjection[i].featureID[imageProjection[i].size] = j;
				

				/*printf("%d %d\n", i, imageProjection[i].featureID[imageProjection[i].size]);
				imageProjection[i].object[imageProjection[i].size].print(stdout);
				cameraCenter[i].print(stdout);
				printf("%f\n", imageProjection[i].gamma[imageProjection[i].size] );
				imageProjection[i].projection[imageProjection[i].size].print(stdout);
				printf("\n");*/

				imageProjection[i].size++;
				gammaID++;
			}
		}
	}

	
	
	p = ( float * ) malloc( ( 1+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID ) * sizeof( float ) );

	for ( i=0; i<viewpointLocationList_.size(); i++ )
	{
		p[1+3*featureList_.size()+3*i] = cameraCenter[i].x() / scale;
		p[2+3*featureList_.size()+3*i] = cameraCenter[i].y() / scale;
		p[3+3*featureList_.size()+3*i] = cameraCenter[i].z() / scale;
	}

	for ( i=0; i<featureList_.size(); i++ )
	{
		p[1+3*i] = reconObjectList_[i].GetReconLocation().x() / scale;
		p[2+3*i] = reconObjectList_[i].GetReconLocation().y() / scale;
		p[3+3*i] = reconObjectList_[i].GetReconLocation().z() / scale;
	}

	int currentGammaID = 0;	
	
	for ( i=0; i < viewpointLocationList_.size(); i++ )
	{
		for ( j=0; j < imageProjection[i].size; j++ )
		{
			p[1+3*featureList_.size()+3*viewpointLocationList_.size()+currentGammaID] = imageProjection[i].gamma[j];
			currentGammaID++;
		}
	}

	//cout <<gammaID<<endl;	

	/*float **xi;
	
	xi = ( float ** ) malloc( ( 2+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID ) * sizeof( float * ) );
	
	for ( i=1; i<2+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID; i++ )
		xi[i] = ( float * ) malloc( ( 2+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID ) * sizeof( float ) );
	
	for ( i=1; i<2+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID; i++ )
	{
		for ( j=1; j<2+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID; j++ )
		{
			if ( i==j )
				xi[i][j] = 1;
			else
				xi[i][j] = 0;
		}
	}
	
	if ( powellAll( *this, p, xi, 1+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID, FTOL, &iter, &fret, funcAll ) )*/
	
	if ( frprmnAll( *this, p, 1+3*featureList_.size()+3*viewpointLocationList_.size()+gammaID, FTOL, &iter, &fret, funcAll, dfuncAll ) )
		printf( "\nOptimization Success!\n\n" );
	else
		printf( "\nOptimization Fail!\n\n" );

	for ( i=0; i<featureList_.size(); i++ )
	{
		reconPoint.set( p[1+3*i] * scale, p[2+3*i] * scale, p[3+3*i] * scale );
		reconObjectList_[i].SetAnchorReconstruction( reconPoint, 0 );
	}

	for ( i=0; i<viewpointLocationList_.size(); i++ )
	{
		reconPoint.set( p[1+3*featureList_.size()+3*i] * scale, p[2+3*featureList_.size()+3*i] * scale, p[3+3*featureList_.size()+3*i] * scale );
		reconViewpointLocationList_[i] = reconPoint;
	}
	
	free( p );

	for ( i=0; i<viewpointLocationList_.size(); i++ )
	{
		free( imageProjection[i].projection );
		free( imageProjection[i].object );
		free( imageProjection[i].gamma );
		free( imageProjection[i].featureID );
	}

	free( imageProjection );
	free( cameraCenter );

	float err = 0.0;
	int count = 0;
	for ( i=0; i<reconObjectList_.size(); i++ )
	{
		float length = ( reconObjectList_[i].GetReconLocation() - objectLocationList_[i] ).length();
		err += length;
		if ( length > 50 )
			count++;
	}

	//printf( "outlier for IM: %d\n", count );

	return err / reconObjectList_.size();
}