#ifndef __SceneData_hpp__
#define __SceneData_hpp__

#include "AnchorReconstructor.hpp"
#include "NonanchorReconstructor.hpp"
#include "AnchorPlanner.hpp"
#include "Feature.hpp"
#include "FeaturePair.hpp"
#include "ReconObject.hpp"
#include "CameraParameters.hpp"
#include "vm3vector.h"
#include "vm3matrix.h"
#include "vm4matrix.h"
#include <vector>
#include <iostream>

class AnchorReconstructor;
class AnchorPlanner;
class NonanchorReconstructor;

/*****************************************************************************
*****************************************************************************/
class SceneData {  

  std::vector<VM3Vector> viewpointLocationList_;
  std::vector<VM3Vector> reconViewpointLocationList_;
  std::vector<int> absoluteImageIDList_; //BOBEDIT Feb 14
  std::vector<bool> viewpointValidOrientationList_;  
  std::vector<VM4Matrix> viewpointOrientationList_;  

  std::vector<Feature> featureList_;  	
  std::vector<ReconObject> reconObjectList_;

  std::vector<VM3Vector> SBAReconLocationList_;
  std::vector<VM3Vector> triangulateReconLocationList_;
  std::vector<float> projErrorList_;	
  std::vector<VM3Vector> objectLocationList_;  	
  std::vector<FeaturePair*> anchorList_;  

  AnchorReconstructor * anchorReconstructor_;
  NonanchorReconstructor * nonanchorReconstructor_;  
  AnchorPlanner * anchorPlanner_;

  CameraParameters cameraParameters_;

public:  
  SceneData();  	
  SceneData( const SceneData & sceneData );  
  SceneData(std::vector<VM3Vector> viewpointLocationList, std::vector<int> absoluteImageIDList, std::vector<bool> viewpointValidOrientationList,  
			std::vector<VM4Matrix> viewpointOrientationList, std::vector<Feature> featureList, std::vector<ReconObject> reconObjectList,  
			std::vector<VM3Vector> reconViewpointLocationList, std::vector<VM3Vector> SBAReconLocationList, 
			//std::vector<VM3Vector> triangulateReconLocationList,
			std::vector<float> projErrorList, std::vector<VM3Vector> objectLocationList,   
			AnchorReconstructor * anchorReconstructor, NonanchorReconstructor * nonanchorReconstructor, AnchorPlanner * anchorPlanner,
			CameraParameters cameraParameters);

  bool OutputDistanceBetweenScenes( SceneData sceneData, std::ostream outflow );

  std::vector<VM3Vector> & GetViewpointLocationList();  	
  const std::vector<VM3Vector> & GetViewpointLocationList() const;

  std::vector<VM3Vector> & GetReconViewpointLocationList();  	
  const std::vector<VM3Vector> & GetReconViewpointLocationList() const;

  std::vector<int> & GetAbsoluteImageIDList();
  const std::vector<int> & GetAbsoluteImageIDList() const;

  std::vector<VM4Matrix> & GetViewpointOrientationList();  	
  const std::vector<VM4Matrix> & GetViewpointOrientationList() const;  

  std::vector<bool> & GetViewpointValidOrientationList();  	
  const std::vector<bool> & GetViewpointValidOrientationList() const;  

  std::vector<Feature> & GetFeatureList();  	
  const std::vector<Feature> & GetFeatureList() const;  

  std::vector<ReconObject> & GetReconObjectList();  	
  const std::vector<ReconObject> & GetReconObjectList() const;


  std::vector<VM3Vector> & GetSBAReconLocationList();  	
  const std::vector<VM3Vector> & GetSBAReconLocationList() const;  

  std::vector<VM3Vector> & GetTriangulateReconLocationList();  	
  const std::vector<VM3Vector> & GetTriangulateReconLocationList() const;  

  std::vector<float> & GetProjErrorList();  	
  const std::vector<float> & GetProjErrorList() const;  

  std::vector<VM3Vector> & GetObjectLocationList();  	
  const std::vector<VM3Vector> & GetObjectLocationList() const;  

  std::vector<FeaturePair*> & GetAnchorList();  	
  const std::vector<FeaturePair*> & GetAnchorList() const;  

  CameraParameters & GetCameraParameters();  	
  const CameraParameters & GetCameraParameters() const;  

  void WriteViewpointOrientation( VM4Matrix Orientation, int frameID );
  void WriteViewpointLocation( VM3Vector Location, int frameID );
  
  bool HasObjectLocations() const;  

  void ReconstructAnchors( float featurePairLifeWeight, 
		  float featurePairDistanceWeight, float continuationThreshold, 
		  int anchorImageNumber, float estimatedTrackingError = 0.0f, 
		  float cameraPositionNoise = 0.0f, float cameraOrientationNoise = 0.0f );
  void ReconstructNonanchors( int nonanchorImageNumber, 
		  float estimatedTrackingError = 0.0f, float cameraPositionNoise = 0.0f, 
		  float cameraOrientationNoise = 0.0f );

  void OutputBundleFile( );
  float InputBundleResult( char * file );
  
  void AddPositionNoise( float positionNoise );
  void Triangulate( );
  float ReconstructBySubsets( int imageNumber );
  float ReconstructAllPoints( );

  void AnchorProjError();
  void NonanchorProjError();

  void LoadFeatureData( std::istream & input, float trackingNoise = 0.0f, float cameraPositionNoise = 0.0f, float cameraOrientationNoise = 0.0f);
  void SaveFeatureData( std::ostream & output ) const;  

  //void LoadReconData( std::istream & input );  	
  //void SaveReconData( std::ostream & output ) const;

private:  

  void EstimateViewpointOrientations();
  void RemoveBadFeatures();
  bool IsBadFeature( const Feature & feature ) const;

};

#include "SceneData.ipp"

#endif // __SceneData_hpp__
