#include "SynReader.hpp"
#include <algorithm>
#include <vector>

using namespace std;

namespace {

  bool CompareProjections( const OMSceneProjEntry & a, const OMSceneProjEntry & b )
  {
    if( a.objectId == b.objectId )
      return a.imageNum < b.imageNum;
    else
      return a.objectId < b.objectId;
  }

}

bool SynReader::ReadSynFile( OMSceneData & sceneData )
{
  //
  // Clear the features list.
  //
  features_.clear();

  //
  // Analyze the projections to build the feature list.
  //
  for( size_t i=0; i<sceneData.projList.size(); ++i )
    sceneData.projList[i].projId = i;
  vector<OMSceneProjEntry> projections( &sceneData.projList[0], &sceneData.projList[0] + sceneData.projList.size() );
  sort( projections.begin(), projections.end(), CompareProjections );
  unsigned tooShort = 0;
  if( projections.size() != 0 ) {
    for( size_t i=0; i<projections.size(); ++i ) {
      unsigned firstImageId = projections[i].imageNum;
      vector<Feature::Location> locations( 1, Feature::Location( projections[i].projId, projections[i].coord.x(), projections[i].coord.y() ) );
      while( ( i+1 < projections.size() ) && ( projections[i+1].imageNum == projections[i].imageNum + 1 ) && 
             ( projections[i+1].objectId == projections[i].objectId ) ) {
        ++i;
        locations.push_back( Feature::Location( projections[i].projId, projections[i].coord.x(), projections[i].coord.y() ) );
      }
      if( locations.size() >= 6 )
        features_.push_back( Feature( features_.size(), projections[i].objectId, firstImageId, locations ) );
      else
        ++tooShort;
    }
  }
  //cout << "Extracted " << features_.size() << " good features and dropped " << tooShort << " short features." << endl;

  return true;
}

const vector<Vector3> & SynReader::GetObjects() const
{
  return objects_;
}

const vector<Feature> & SynReader::GetFeatures() const
{
  return features_;
}

const vector<Vector3> & SynReader::GetCentersOfProjection() const
{
  return centersOfProjection_;
}
