#ifndef __SynReader_hpp__
#define __SynReader_hpp__

#include "Feature.hpp"
#include "Vector3.hpp"
#include "omscenedata.h"
#include <string>
#include <vector>

///
/// @brief A reader for the .syn file format.
///
class SynReader {

  std::vector<Vector3> objects_;
  std::vector<Feature> features_;
  std::vector<Vector3> centersOfProjection_;

public:

  ///
  /// @brief Read the features from a .syn file.
  ///
  bool SynReader::ReadSynFile( OMSceneData & sceneData );

  ///
  /// @brief Get the object locations.
  ///
  const std::vector<Vector3> & GetObjects() const;

  ///
  /// @brief Get the set of features read in the last call to
  /// ReadSynFile.
  ///
  const std::vector<Feature> & GetFeatures() const;

  ///
  /// @brief Get the centers of projection.
  ///
  const std::vector<Vector3> & GetCentersOfProjection() const;

};

#endif // __SynReader_hpp__
