#ifndef __Vector3_inl__
#define __Vector3_inl__

#include <cmath>
#include <limits>

inline Vector3::Vector3() {}

inline Vector3::Vector3( float x, float y, float z )
{
  data_[0] = x;
  data_[1] = y;
  data_[2] = z;
}

inline float & Vector3::operator[]( size_t i )
{
  return data_[i];
}

inline float Vector3::operator[]( size_t i ) const
{
  return data_[i];
}

inline const Vector3 Vector3::operator-() const
{
  return Vector3( -data_[0], -data_[1], -data_[2] );
}

inline const Vector3 & Vector3::operator*=( float s )
{
  data_[0] *= s;
  data_[1] *= s;
  data_[2] *= s;
  return *this;
}

inline const Vector3 & Vector3::operator/=( float s )
{
  return operator*=( 1.0f / s );
}

inline const Vector3 & Vector3::operator+=( const Vector3 & v )
{
  data_[0] += v.data_[0];
  data_[1] += v.data_[1];
  data_[2] += v.data_[2];
  return *this;
}

inline const Vector3 & Vector3::operator-=( const Vector3 & v )
{
  data_[0] -= v.data_[0];
  data_[1] -= v.data_[1];
  data_[2] -= v.data_[2];
  return *this;
}

inline float Vector3::GetSquaredLength() const
{
  return data_[0]*data_[0] + data_[1]*data_[1] + data_[2]*data_[2];
}

inline float Vector3::GetLength() const
{
  return sqrt( GetSquaredLength() );
}

inline const Vector3 operator*( const Vector3 & v, float s )
{
  Vector3 result( v );
  return result *= s;
}

inline const Vector3 operator*( float s, const Vector3 & v )
{
  Vector3 result( v );
  return result *= s;
}

inline const Vector3 operator/( const Vector3 & v, float s )
{
  Vector3 result( v );
  return result /= s;
}

inline const Vector3 operator+( const Vector3 & v1, const Vector3 & v2 )
{
  Vector3 result( v1 );
  return result += v2;
}

inline const Vector3 operator-( const Vector3 & v1, const Vector3 & v2 )
{
  Vector3 result( v1 );
  return result -= v2;
}

inline void Normalize( Vector3 & v )
{
  float length = v.GetLength();
  if( length > std::numeric_limits<float>::epsilon() )
    v /= length;
}

inline float Dot( const Vector3 & v1, const Vector3 & v2 )
{
  return v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2];
}

inline const Vector3 Cross( const Vector3 & v1, const Vector3 & v2 )
{
  return Vector3( v1[1]*v2[2] - v1[2]*v2[1], v1[2]*v2[0] - v1[0]*v2[2], v1[0]*v2[1] - v1[1]*v2[0] );
}

#endif // __Vector3_inl__
