#include "Disable4786.hpp"
#include <vmarray.h>
#include <vm2point.h>
#include <vm2vector.h>
#include <vm3point.h>
#include <vm3vector.h>
#include <vm4matrix.h>
#include <windows.h>
#include <sys/stat.h>
#include "aibar.h"

#include <vector>
#include <iterator>
#include <iostream>
#include <fstream>
#include <time.h>

#define CLOCKS_PER_MSEC (CLOCKS_PER_SEC/1000.0)

using namespace std;

#define DFLT_WIN_WIDTH            512
#define DFLT_WIN_HEIGHT           512

#define NOID                      -1
#define SCENEDATALOADID           1
#define SCENEDATASAVEID           2
#define OPTIMIZATION              3
#define COMPUTEANCHORPOINTSID     4
#define COMPUTEALLPOINTSID        5
#define PRINTERRORSTATSID	      6
#define CONDITIONTHRESHOLDID      7
#define VARIANCETHRESHOLDID	      8
#define COMPUTEALLID	          9
#define OUTPUTRECONPOINTSID       10
#define FEATUREPAIRLIFEWEIGHTID   11
#define FEATUREPAIRDISTANCEWEIGHTID 12
#define COMPAREERRORS             13
#define GATHERDATAID              14
#define LOADBOUNDINGBOXID         15

#define POSEOPTID                 17
#define MESHDATAID                16
#define ADJACENT_SUBSET_OPTIMIZATION					  18//BOBEDIT
#define DISJOINT_SUBSET_OPTIMIZATION		  19//BOBEDIT
#define DIFF_FILES				  20
#define	INDUCE_ERROR			  21

#define QUIT                      99

#define FTOL 1.0e-4

#define OUR       0
#define TRIANGULATE    1
#define SBA 2

#define CONTIGUOUS					0
#define INTERLEAVED					1
#define TOO_MANY_FEATURES			600//somewhat arbitrary point where the j^2 of O(n*j^2) is just decided to be too big


//#define COMPUTEROT

// user interface
static char             sceneDataFile[1024];
static char             sceneDataSaveFile[1024];

static char				diffInputFileOne[1024];
static char				diffInputFileTwo[1024];
static char				diffOutputFile[1024];
static bool				diffFlag = false;

static char             reconOutputFile[1024];
static char             errorCompareFile[1024];
static int              firstImageIndex = 0;
static int              lastImageIndex = 0;


static int              currentCamera = 0;
static float            percentileThreshold = 0;
static int              useProjError = 0;
static int              reconMethod  = OUR;
static float            pointsDistance = 7.0;
static float            trackingNoise = 0.0f;
static float            cameraPositionNoise = 0.0f;
static float            cameraOrientationNoise = 0.0f;
static float            positionNoise = 0.0f;
static float            featurePairLifeWeight = 0.5;
static float            featurePairDistanceWeight = 0.5;
static float            continuationThreshold = 0.5;
static int              anchorImageNumber = 15;
static int              nonanchorImageNumber = 15;
static float            timecost = 0;


static int				adjacentGroupSize = -1;//BOBEDIT
static int				adjacentMaxFeaturesPerImageGroup = -1;
static int				disjointMinFeatureLife = -1;//BOBEDIT	when images are subdivided, refers to how many of those images each feature sees
						//so if there are 370 images but only 5 being withdrawn, a disjointMinFeatureLife of 4 won't refer to images 1,2,3 out of 370
						//but rather 1, 2, 3 out of the 5 (and maybe 1, 70, 140, 210 out of 370)
static int				adjacentMinFeatureLife = -1;
static int				withdrawImageCount = -1;//BOBEDIT
static int				disjointMaxFeaturesPerImageGroup = -1;//BOBEDIT
static int				subsetIterationCount = -1;//BOBEDIT
static int				subsetType = -1;
static int				levelsToMerge = 1;
static bool				errorFlag = false;//BOBEDIT

// error statistics
static float            anchorMeanError = 0.0f;
static float            anchorStddevError = 0.0f;
static float            anchorMinError = 0.0f;
static float            anchorMaxError = 0.0f;
static float            nonanchorMeanError = 0.0f;
static float            nonanchorStddevError = 0.0f;
static float            nonanchorMinError = 0.0f;
static float            nonanchorMaxError = 0.0f;

// bounding box
static int              useBoundingBox = false;
static char             boundingBoxFile[1024];
static VM3Vector        boundingBoxMin;
static VM3Vector        boundingBoxMax;

// highlighed triangle
static int tri_in_select = -1;

// misc variables
typedef struct
{
   int v0,v1,v2;
   int ov0,ov1,ov2;
   int valid;
}SimpleTri;

std::vector <SimpleTri> trilist;
std::vector <VM3Vector> pntlist;

static SceneData        sceneData;
static SceneData		diffData;
static std::vector<float> conditionPercentile;
static std::vector<float> variancePercentile;

static void guiCB(int id);



/*****************************************************************************
*****************************************************************************/
static void parameterPrint()
{
  cout << "***************Parameter Check********************" << endl;
  cout << "adjacentGroupSize: " << adjacentGroupSize << endl;
  cout << "adjacentMaxFeaturesPerImageGroup: " <<  adjacentMaxFeaturesPerImageGroup << endl;
  cout << "disjointMinFeatureLife: " << disjointMinFeatureLife << endl;
  cout << "adjacentMinFeatureLife: " << adjacentMinFeatureLife << endl;
  cout << "withdrawImageCount: " << withdrawImageCount << endl;
  cout << "disjointMaxFeaturesPerImageGroup: " << disjointMaxFeaturesPerImageGroup << endl;
  cout << "subsetIterationCount: " << subsetIterationCount << endl;
  cout << "cameraPositionNoise: " << cameraPositionNoise << endl;
  cout << "cameraOrientationNoise: " << cameraOrientationNoise << endl;
  cout << "levelsToMerge: " << levelsToMerge << endl;
  cout << "***************************************************" << endl;
}
/*****************************************************************************
*****************************************************************************/
static void parameterAdjustment()
{
	int numf = sceneData.GetFeatureList().size();
	int numi = sceneData.GetAbsoluteImageIDList().size();

	if (numf == 0 || numi == 0)
	{
		cerr << "Either no features or no images in file." << endl;
		exit(0);
	}
	if (adjacentMaxFeaturesPerImageGroup > numf || disjointMaxFeaturesPerImageGroup > numf)
	{
		cerr << "Error: -f flag. Value entered after -f > number of features in file." << endl;
		exit(0);
	}
	if (adjacentGroupSize > numi || withdrawImageCount > numi)
	{
		cerr << "Error: -s flag. Value entered after -s > number of images in file." << endl;
		exit(0);
	}


	
	if (subsetType == CONTIGUOUS)
	{
		if (adjacentMinFeatureLife == -1)
			adjacentMinFeatureLife = 3;

		//works 4-20-2006
		//if -f is set, but not -s
		if (adjacentMaxFeaturesPerImageGroup != -1 && adjacentGroupSize == -1)
		{
			//DO NOT correct -f if it is over TOO_MANY_FEATURES's value
			int num_groups = numf / adjacentMaxFeaturesPerImageGroup;
		
			//to catch the extras
			if (numf - num_groups * adjacentMaxFeaturesPerImageGroup > adjacentMinFeatureLife)
				num_groups++;

	
			adjacentGroupSize = numi / num_groups;
			if (adjacentGroupSize * num_groups != numi)
				adjacentGroupSize++;

			if (adjacentGroupSize < adjacentMinFeatureLife)
				adjacentGroupSize = adjacentMinFeatureLife;
		}
		//works 4-20-2006
		//if -s is set, but not -f
		else if (adjacentMaxFeaturesPerImageGroup == -1 && adjacentGroupSize != -1)
		{
			int num_groups = numi / adjacentGroupSize;
			//to catch the extras: if there are more than 3 leftovers, make a group of them
			if (numi - num_groups * adjacentGroupSize > 3)
				num_groups++;

			adjacentMaxFeaturesPerImageGroup = numf / num_groups;
			if (adjacentMaxFeaturesPerImageGroup * num_groups != numf)
				adjacentMaxFeaturesPerImageGroup++;

			if (adjacentMaxFeaturesPerImageGroup >= TOO_MANY_FEATURES)
				adjacentMaxFeaturesPerImageGroup = TOO_MANY_FEATURES;
		}
		//works 4-20-2006
		//if neither are set
		else if (adjacentMaxFeaturesPerImageGroup == -1 && adjacentGroupSize == -1)
		{
			int tempf = numf;
			adjacentGroupSize = numi;

			while (tempf >= TOO_MANY_FEATURES)
			{
				tempf /= 2;
				adjacentGroupSize /= 2;
			}
			adjacentMaxFeaturesPerImageGroup = tempf;
		}


		/*
		if (numf > TOO_MANY_FEATURES)
			adjacentMaxFeaturesPerImageGroup = TOO_MANY_FEATURES;
		else if (adjacentMaxFeaturesPerImageGroup == -1)
		{
			if ((numf / 8) > 0)
				adjacentMaxFeaturesPerImageGroup = numf / 8;
			else if ((numf / 4) > 0)
				adjacentMaxFeaturesPerImageGroup = numf / 4;
			else if ((numf / 2) > 0)
				adjacentMaxFeaturesPerImageGroup = numf / 2;
			else
				adjacentMaxFeaturesPerImageGroup = numf;
		}

		if(adjacentGroupSize == -1)
		{
			if ((numi / 8) > 0)
				adjacentGroupSize = numi / 8;
			else if ((numi / 4) > 0)
				adjacentGroupSize = numi / 4;
			else if ((numi / 2) > 0)
				adjacentGroupSize = numi / 2;
			else
				adjacentGroupSize = numi;
		}
		*/
	}
	else if (subsetType == INTERLEAVED)
	{
		//NOTE: withdrawImageCount = groupSize, subsetIterationCount = number of groups, disjointMaxFeaturesPerImageGroup = # of features per group 
		//(no more, no less)
		if (disjointMinFeatureLife == -1)
			disjointMinFeatureLife = 3;

		//if neither -s or -f is set
		if (withdrawImageCount == -1 && disjointMaxFeaturesPerImageGroup == -1)
		{
			int tempf = numf;
			subsetIterationCount = 1;
			withdrawImageCount = numi;

			while (tempf >= TOO_MANY_FEATURES)
			{
				tempf /= 2;
				subsetIterationCount *= 2;
				withdrawImageCount /= 2;
			}

			disjointMaxFeaturesPerImageGroup = tempf;
		}
		//if -s is set but not -f
		else if (disjointMaxFeaturesPerImageGroup == -1)
		{
			subsetIterationCount = numi / withdrawImageCount;
			if (subsetIterationCount * withdrawImageCount != numi)
				subsetIterationCount++;

			disjointMaxFeaturesPerImageGroup = numf / subsetIterationCount;
			if (disjointMaxFeaturesPerImageGroup * subsetIterationCount != numf)
				disjointMaxFeaturesPerImageGroup++;

			if (disjointMaxFeaturesPerImageGroup >= TOO_MANY_FEATURES)
				disjointMaxFeaturesPerImageGroup = TOO_MANY_FEATURES - 1;
		}
		//works 4-20-2006
		//if -f is set but not -s
		else if (withdrawImageCount == -1)
		{
			//Note: don't crop down -f if it's larger than TOO_MANY_FEATURES

			subsetIterationCount = numf / disjointMaxFeaturesPerImageGroup;
			if (subsetIterationCount * disjointMaxFeaturesPerImageGroup != numf)
				subsetIterationCount++;

			withdrawImageCount = numi / subsetIterationCount;
			//if (withdrawImageCount * subsetIterationCount != numi)
			//	withdrawImageCount++;
		}
		else
			subsetIterationCount = numi / withdrawImageCount;


		/*
		int avgFeatLife = 0;
		for (int i=0; i < sceneData.GetFeatureList().size(); i++)
			avgFeatLife += sceneData.GetFeatureList()[i].GetRange().GetSize();
		
		if (i != 0)
			avgFeatLife /= i;

		cout << "avgFeatLife: " << avgFeatLife << endl;//debug

		if (disjointMinFeatureLife == -1)
			disjointMinFeatureLife = 3;
		
		if (withdrawImageCount == -1)
		{
			withdrawImageCount = numi / avgFeatLife * disjointMinFeatureLife;
			if (withdrawImageCount > numi)
				withdrawImageCount = numi;
		}

		subsetIterationCount = numi / withdrawImageCount;

		if (disjointMaxFeaturesPerImageGroup == -1)
		{
			if (numf / subsetIterationCount < TOO_MANY_FEATURES)
				disjointMaxFeaturesPerImageGroup = numf / subsetIterationCount;
			else
				disjointMaxFeaturesPerImageGroup = TOO_MANY_FEATURES - 1;
		}
		*/
	}
	

}
/*****************************************************************************
*****************************************************************************/
bool isReconstructedWell( int objectID )
{
	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();

	if( reconObjectList[objectID].GetReconFlag() == ReconObject::RECON_NONE )
		return false;


	if( reconObjectList[objectID].GetReconFlag() == ReconObject::RECON_ANCHOR )
		return true;

	if( conditionPercentile[objectID] < percentileThreshold )
		return false;

	if( variancePercentile[objectID] < percentileThreshold )
		return false;

	if( useBoundingBox ) 
	{
		const VM3Vector & reconLocation = reconObjectList[objectID].GetReconLocation();
		if( reconLocation.x() < boundingBoxMin.x() || 
			reconLocation.x() > boundingBoxMax.x() ||
			reconLocation.y() < boundingBoxMin.y() || 
			reconLocation.y() > boundingBoxMax.y() ||
			reconLocation.z() < boundingBoxMin.z() || 
			reconLocation.z() > boundingBoxMax.z() )
		return false;
	}

	return true;
}

/*****************************************************************************
*****************************************************************************/
static void
LoadMeshData()
{
   trilist.resize( 0 );
   
   FILE * fp = fopen("mesh.off","r");
   
   char buf[256];

   const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();

   fgets(buf,256,fp);
   int np, nt, foo;
   sscanf(buf,"OFF %d %d %d",&np,&nt,&foo);

   for (int i=0;i<np;i++)
   {
      fgets(buf,256,fp);
	  float x, y, z;
	  sscanf( buf,"%f %f %f",&x,&y,&z );
	  VM3Vector object( x, y, z );
	  pntlist.push_back( object );
   }

   for (i=0;i<nt;i++)
   {
      fgets(buf,256,fp);
      int tmp;
      int id[3];
      sscanf(buf, "%d %d %d %d", &tmp, &id[0] ,&id[1], &id[2]);	
	  SimpleTri tri;
	  tri.ov0 = id[0];
	  tri.ov1 = id[1];
	  tri.ov2 = id[2];
      
	  if ( tmp == 1) 
		  break;

      for (int k=0;k<3;k++)
	  {
		  for (int j=0;j<reconObjectList.size();j++)
		  {         
            //VM3Vector reconLocation = ;
			//VM3Vector meshObject = ;
			if ( ( pntlist[ id[k] ] - reconObjectList[j].GetReconLocation() ).length() < 0.001 )
			{
				id[k] = j;
				break;
			}
         }
      }

      // add the triangle into geometry
      tri.v0 = id[0];
      tri.v1 = id[1];
      tri.v2 = id[2];
	  tri.valid = 1;
      trilist.push_back(tri);
   }

   fclose(fp);

}

/*****************************************************************************
*****************************************************************************/
static void UpdateAnchorError();
static void UpdateNonanchorError();
static void computePercentiles();
static void extractAdjacentSubsets(std::vector<SceneData> & sceneDataSubsets);
static void extractDisjointSubsets(std::vector<SceneData> & sceneDataSubsets);
static void restoreAdjacentSubsets(SceneData & origSceneData, std::vector<SceneData> & sceneDataSubsets);
static void restoreDisjointSubsets(SceneData & origSceneData, std::vector<SceneData> & sceneDataSubsets);


static void GatherData()
{
	cameraPositionNoise = 0;//100.0;
	cameraOrientationNoise = 0;//0.2;
	positionNoise = 0;

	guiCB( SCENEDATALOADID );
	parameterAdjustment();
	//parameterPrint();
	sceneData.AddPositionNoise( positionNoise );
	SceneData sceneDataBackup(sceneData);



	bool loopFlag = true;
	//printf("You will see this message only once. A number of groups will now be optimized. Then group sizes will be increased by a factor of two and the new groups will be optimized. It may take quite a while to optimize log(n) groups of groups.\n\n");
	for (int k=0; loopFlag; k++)
	{
		sceneData = SceneData(sceneDataBackup);


		if (subsetType == INTERLEAVED)
		{
			if ( //subsetIterationCount * withdrawImageCount > sceneData.GetViewpointLocationList().size() ||
				disjointMaxFeaturesPerImageGroup > sceneData.GetFeatureList().size() ||
				 disjointMinFeatureLife > sceneData.GetViewpointLocationList().size() ||
				 disjointMinFeatureLife > withdrawImageCount )
			{
				cout << "Error: Invalid parameters for subset optimization." << endl;
				cout << "either " << subsetIterationCount << "*" << withdrawImageCount << " > " << sceneData.GetViewpointLocationList().size()
					<< "\t(# of Groups * # of Images per Group > Total Number of Images)" << endl;
				cout << "or " << disjointMaxFeaturesPerImageGroup << " > " << sceneData.GetFeatureList().size() << "\t(Min Feature Count > # of Features)" << endl;
				cout << "or " << disjointMinFeatureLife << " > " << sceneData.GetViewpointLocationList().size() << 
					"\t(Min # of Images that must see a Feature > # of Features)" << endl;
				cout << "or " << disjointMinFeatureLife << " > " << withdrawImageCount << 
					"\t(Min # of Images that must see a Feature > # of Images per Group)" << endl;
				//TODO: output error
				exit(0);
			}
		}
		else
		{
			if (adjacentMinFeatureLife > adjacentGroupSize)
			{
				cout << "Error: Invalid parameters for subset optimization." << endl;
				cout <<	"Min # of Images that see a Feature( " << adjacentMinFeatureLife << " )  >  # of Images in a Group( " << adjacentGroupSize
			         << " )" << endl;
				exit(0);
			}
			else if (adjacentGroupSize > sceneData.GetAbsoluteImageIDList().size() || adjacentGroupSize < 2)
			{
				cout << "Error: groupSize > # of images or groupSize < 2" << endl;
				exit(0);
			}
		}
		
		SceneData origSceneData(sceneData);
		std::vector<SceneData> sceneDataArray;



		if (subsetType == CONTIGUOUS)
			extractAdjacentSubsets(sceneDataArray);
		else
			extractDisjointSubsets(sceneDataArray);
			
				

		printf( "Num of groups: %d\n", sceneDataArray.size() );
		for ( int n=0; n<sceneDataArray.size(); n++ )
		{
			printf("\n================================================\n");
			printf("\nGroup Num: %d\n", n);
			printf("Num of points: %d\n",sceneDataArray[n].GetReconObjectList().size());
			printf("Num of cameras: %d\n",sceneDataArray[n].GetViewpointLocationList().size());
            			
					
			sceneDataArray[n].ReconstructAllPoints( );

		}
				
		if (subsetType == CONTIGUOUS)
			restoreAdjacentSubsets(origSceneData, sceneDataArray);
		else
			restoreDisjointSubsets(origSceneData, sceneDataArray);


		//TODO: double maxFeaturesPerGroup?
		adjacentGroupSize *= 2;
		
		if (adjacentMaxFeaturesPerImageGroup * 2 < TOO_MANY_FEATURES)
			adjacentMaxFeaturesPerImageGroup *= 2;
		else
			adjacentMaxFeaturesPerImageGroup = TOO_MANY_FEATURES - 1;

		withdrawImageCount *= 2;
		subsetIterationCount /= 2;
		disjointMaxFeaturesPerImageGroup *= 2;


		if (subsetType == CONTIGUOUS)
		{
			loopFlag = ( adjacentGroupSize < sceneData.GetAbsoluteImageIDList().size() );
		}
		else
		{				
			loopFlag = ( withdrawImageCount < sceneData.GetAbsoluteImageIDList().size() &&
						 subsetIterationCount > 0 );
		}
		levelsToMerge--;
		if (levelsToMerge < 1)
			loopFlag = false;
	}


}

/*****************************************************************************
*****************************************************************************/
static void updateAnchorProjErrorStatistics();
static void updateAnchorErrorStatistics();
static void updateNonanchorProjErrorStatistics();
static void updateNonanchorErrorStatistics();

static void UpdateAnchorError()
{
	if( useProjError )
		updateAnchorProjErrorStatistics();
	else
		updateAnchorErrorStatistics();
}

static void UpdateNonanchorError()
{
	if( useProjError )
		updateNonanchorProjErrorStatistics();
	else
		updateNonanchorErrorStatistics();
}

static void LoadBoundingBox()
{
	ifstream bbFile( boundingBoxFile );
	if( !bbFile ) {
		cerr << "Unable to open bounding box file: " << boundingBoxFile << endl;
		return;
	}

	float minX, minY, minZ;
	float maxX, maxY, maxZ;
	bbFile >> minX >> minY >> minZ >> maxX >> maxY >> maxZ;

	boundingBoxMin.set( minX, minY, minZ );
	boundingBoxMax.set( maxX, maxY, maxZ );
}

/*****************************************************************************
*****************************************************************************/
static void Usage()
{
	printf("usage: [<scenedata file> [<anchors file>]]\n");
}


/*****************************************************************************
*****************************************************************************/
static void
ParseArgs(int argc, char *argv[])
{
   if (argc < 3)
   {
	   cerr << "Invalid command line (number of parameters)." << endl;
	   cerr << "Minimum command line: 'aibar inputFile outputFile'" << endl;
	   exit(0);
   }
   
   for(int i=1; i < argc-2; i++)
   {
		if (strcmp(argv[i], "-c") == 0)
		{
			if (subsetType == -1)
				subsetType = CONTIGUOUS;
			else
			{
				cerr << "Invalid command line (-c)" << endl;
				exit(0);
			}
		}
		else if (strcmp(argv[i], "-i") == 0)
		{
			if (subsetType == -1)
				subsetType = INTERLEAVED;
			else
			{
				cerr << "Invalid command line (-i)" << endl;
				exit(0);
			}
		}
		else if (strcmp(argv[i], "-e") == 0)
		{
			if (i+1 >= argc || (atoi(argv[i+1]) == 0 && argv[i+1][0] != '0')) //|| (atoi(argv[i+2]) == 0 && argv[i+2][0] != '0') )
			{
				cerr << "Invalid command line (-e)" << endl;
				exit(0);
			}
			positionNoise = atof(argv[i+1]);
			//cameraPositionNoise = atof(argv[i+1]);
			//cameraOrientationNoise = atof(argv[i+2]);

			//if (cameraPositionNoise < 0 || cameraOrientationNoise < 0)
			if (positionNoise < 0)
			{
				cerr << "Invalid command line (-e)" << endl;
				exit(0);
			}

			errorFlag = true;
			i += 1;
		}
		else if (strcmp(argv[i], "-s") == 0)	//for image group Size
		{
			if (i+1 >= argc || atoi(argv[i+1]) == 0 || atoi(argv[i+1]) < 3)
			{
				cerr << "Invalid command line (-s)" << endl;
				exit(0);
			}
			if (subsetType == CONTIGUOUS || subsetType == -1)
				adjacentGroupSize = atoi(argv[i+1]);
			else //if (subsetType == INTERLEAVED)
				withdrawImageCount = atoi(argv[i+1]);

			
			i++;
		}
		else if (strcmp(argv[i], "-l") == 0)	//for min feature Life
		{	
			if (i+1 >= argc || atoi(argv[i+1]) == 0 || atoi(argv[i+1]) < 3)
			{
				cerr << "Invalid command line (-l)" << endl;
				exit(0);
			}
			if (subsetType == CONTIGUOUS || subsetType == -1)
				adjacentMinFeatureLife = atoi(argv[i+1]);
			else //if (subsetType == INTERLEAVED)
				disjointMinFeatureLife = atoi(argv[i+1]);
			
			i++;
		}
		else if (strcmp(argv[i], "-f") == 0)	//for number of Features per group	(contiguous only)
		{
			if (i+1 >= argc || atoi(argv[i+1]) == 0 || atoi(argv[i+1]) < 3)
			{
				cerr << "Invalid command line (-f)" << endl;
				exit(0);
			}
			if (subsetType == CONTIGUOUS || subsetType == -1)
				adjacentMaxFeaturesPerImageGroup = atoi(argv[i+1]);
			else
				disjointMaxFeaturesPerImageGroup = atoi(argv[i+1]);

			i++;
		}
		else if (strcmp(argv[i], "-m") == 0)	//for number of levels to Merge
		{
			if (i+1 >= argc || atoi(argv[i+1]) == 0 || atoi(argv[i+1]) < 1)
			{
				cerr << "Invalid command line (-m)" << endl;
				exit(0);
			}
			levelsToMerge = atoi(argv[i+1]);


			i++;
		}
		else if (strcmp(argv[i], "-d") == 0)	//for file 'diff'ing
		{
			if (i+3 >= argc)
			{
				cerr << "Invalid command line (-d)" << endl;
				exit(0);
			}
			strcpy(diffInputFileOne, argv[i+1]);
			strcpy(diffInputFileTwo, argv[i+2]);
			strcpy(diffOutputFile, argv[i+3]);
			diffFlag = true;
	
			i += 3;
		}
		else
		{
			cerr << "Invalid command line: bad flag" << endl;
			exit(0);
		}
   }
   if (!diffFlag)
   {
	strcpy(sceneDataFile, argv[argc-2]);
	strcpy(sceneDataSaveFile, argv[argc-1]);
   }

   if (subsetType == -1)
	subsetType = CONTIGUOUS;
}



/*****************************************************************************
*****************************************************************************/
static bool compareConditionNum( int object1, int object2 )
{
	return sceneData.GetReconObjectList()[object1].GetConditionNumber() > sceneData.GetReconObjectList()[object2].GetConditionNumber();
}

static bool compareVariance( int object1, int object2 )
{
	return sceneData.GetReconObjectList()[object1].GetViewpointVariance() < sceneData.GetReconObjectList()[object2].GetViewpointVariance();
}

static void computePercentiles()
{
	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
	conditionPercentile.resize( reconObjectList.size() );
	variancePercentile.resize( reconObjectList.size() );

	std::vector<int> conditionSortedFeatureIDs;
	std::vector<int> varianceSortedFeatureIDs;

	for( int i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR ) 
		{
			conditionSortedFeatureIDs.push_back( i );
			varianceSortedFeatureIDs.push_back( i );
		}
	}

	std::stable_sort( conditionSortedFeatureIDs.begin(), conditionSortedFeatureIDs.end(), &compareConditionNum );
	std::stable_sort( varianceSortedFeatureIDs.begin(), varianceSortedFeatureIDs.end(), &compareVariance );

	for( int j=0; j<conditionSortedFeatureIDs.size(); ++j ) 
	{
		conditionPercentile[conditionSortedFeatureIDs[j]] = float( j ) / float( conditionSortedFeatureIDs.size() );
		variancePercentile[varianceSortedFeatureIDs[j]] = float( j ) / float( varianceSortedFeatureIDs.size() );
	}
}

/*****************************************************************************
*****************************************************************************/
static void updateAnchorErrorStatistics()
{
	if( sceneData.HasObjectLocations() ) 
	{
		const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
		const std::vector<VM3Vector> & objectLocationList = sceneData.GetObjectLocationList();

		//
		// Compute the mean error
		//
		anchorMeanError = 0.0f;
		int totalFlagged = 0;
		int i;

		for( i=0; i<reconObjectList.size(); ++i ) 
		{
			if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
			{
				VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
				anchorMeanError += errVec.length();
				++totalFlagged;
			}
		}

		anchorMeanError /= float( totalFlagged );

		//
		// Compute the std dev
		//
		anchorStddevError = 0.0f;

		for( i=0; i<reconObjectList.size(); ++i ) 
		{
			if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
			{
				VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
				float diff = errVec.length() - anchorMeanError;
				anchorStddevError += diff * diff;
			}
		}
		
		anchorStddevError /= float( totalFlagged - 1 );
		anchorStddevError = sqrt( anchorStddevError );

		//
		// Compute the min/max
		//
		anchorMinError = -1.0f;
		anchorMaxError = -1.0f;
		for( i=0; i<reconObjectList.size(); ++i ) 
		{
			if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
			{
				VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
				float err = errVec.length();
				if( anchorMinError == -1.0f || err < anchorMinError )
					anchorMinError = err;
				if( anchorMaxError == -1.0f || err > anchorMaxError )
					anchorMaxError = err;
			}
		}
	}
	else 
	{
		anchorMeanError = 0.0f;
		anchorStddevError = 0.0f;
		anchorMinError = 0.0f;
		anchorMaxError = 0.0f;
	}
}


/*****************************************************************************
*****************************************************************************/
static void updateAnchorProjErrorStatistics()
{
	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
	const std::vector<float> & projErrorList = sceneData.GetProjErrorList();

    //
    // Compute the mean error
    //
    anchorMeanError = 0.0f;
    int totalFlagged = 0;
    int i;

    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
		{
			anchorMeanError += projErrorList[i];
			++totalFlagged;
		}
	}

    anchorMeanError /= float( totalFlagged );

    //
    // Compute the std dev
    //
    anchorStddevError = 0.0f;

    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
		{
			float diff = projErrorList[i] - anchorMeanError;
			anchorStddevError += diff * diff;
		}
	}

    anchorStddevError /= float( totalFlagged - 1 );
    anchorStddevError = sqrt( anchorStddevError );

    //
    // Compute the min/max
    //
    anchorMinError = -1.0f;
    anchorMaxError = -1.0f;
    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR ) 
		{
			if( anchorMinError == -1.0f || projErrorList[i] < anchorMinError )
				anchorMinError = projErrorList[i];
			if( anchorMaxError == -1.0f || projErrorList[i] > anchorMaxError )
				anchorMaxError = projErrorList[i];
		}
	}
}

/*****************************************************************************
 *****************************************************************************/
static void updateNonanchorErrorStatistics()
{
  if( sceneData.HasObjectLocations() ) 
  {
	  const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
	  const std::vector<VM3Vector> & objectLocationList = sceneData.GetObjectLocationList();

	  //
	  // Compute the mean error
	  //
	  nonanchorMeanError = 0.0f;
	  int totalFlagged = 0;
	  int i;
	  
	  for( i=0; i<reconObjectList.size(); ++i ) 
	  {
		  if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		  {
			  VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
			  nonanchorMeanError += errVec.length();
			  ++totalFlagged;
		  }
	  }
	  
	  nonanchorMeanError /= float( totalFlagged );

	  //
	  // Compute the std dev
	  //
	  nonanchorStddevError = 0.0f;
	  
	  for( i=0; i<reconObjectList.size(); ++i ) 
	  {
		  if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		  {
			  VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
			  float diff = errVec.length() - nonanchorMeanError;
			  nonanchorStddevError += diff * diff;
		  }
	  }
	  
	  nonanchorStddevError /= float( totalFlagged - 1 );
	  nonanchorStddevError = sqrt( nonanchorStddevError );

	  //
	  // Compute the min/max
	  //
	  nonanchorMinError = -1.0f;
	  nonanchorMaxError = -1.0f;
	  
	  for( i=0; i<reconObjectList.size(); ++i ) 
	  {
		  if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		  {
			  VM3Vector errVec = reconObjectList[i].GetReconLocation() - objectLocationList[i];
			  float err = errVec.length();
			  
			  if( nonanchorMinError == -1.0f || err < nonanchorMinError )
				  nonanchorMinError = err;
			  if( nonanchorMaxError == -1.0f || err > nonanchorMaxError )
				  nonanchorMaxError = err;
		  }
	  }
  }
  else 
  {
	  nonanchorMeanError = 0.0f;
	  nonanchorStddevError = 0.0f;
	  nonanchorMaxError = 0.0f;
	  nonanchorMinError = 0.0f;
  }
}

/*****************************************************************************
 *****************************************************************************/
static void updateNonanchorProjErrorStatistics()
{
	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
    const std::vector<float> & projErrorList = sceneData.GetProjErrorList();

    //
    // Compute the mean error
    //
    nonanchorMeanError = 0.0f;
    int totalFlagged = 0;
    int i;

    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		{
			nonanchorMeanError += projErrorList[i];
			++totalFlagged;
		}
	}
	
	nonanchorMeanError /= float( totalFlagged );

    //
    // Compute the std dev
    //
    nonanchorStddevError = 0.0f;

    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		{
			float diff = projErrorList[i] - nonanchorMeanError;
			nonanchorStddevError += diff * diff;
		}
    }

    nonanchorStddevError /= float( totalFlagged - 1 );
    nonanchorStddevError = sqrt( nonanchorStddevError );

    //
    // Compute the min/max
    //
    nonanchorMinError = -1.0f;
    nonanchorMaxError = -1.0f;

    for( i=0; i<reconObjectList.size(); ++i ) 
	{
		if( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR && isReconstructedWell(i) ) 
		{
			if( nonanchorMinError == -1.0f || projErrorList[i] < nonanchorMinError )
				nonanchorMinError = projErrorList[i];
			if( nonanchorMaxError == -1.0f || projErrorList[i] > nonanchorMaxError )
				nonanchorMaxError = projErrorList[i];
		}
	}
}

/*****************************************************************************
 *****************************************************************************/
static void outputProjError( const char * filename )
{
	FILE * fp = fopen( filename, "w" );
    if( fp == 0 )
		fprintf( stderr, "Unable to open error report file: %s\n", filename );

    //
    // Print out the errors.
    //
    const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
    const std::vector<float> & projErrorList = sceneData.GetProjErrorList();
    fprintf( fp, "featureID projError conditionNumber(cosAngle) viewpointVariance\n");
    for( int i=0; i<reconObjectList.size(); ++i ) 
		if( reconObjectList[i].GetReconFlag() != ReconObject::RECON_NONE ) 
			fprintf( fp, "%d %f %f %f\n", i, projErrorList[i], reconObjectList[i].GetConditionNumber(), reconObjectList[i].GetViewpointVariance() );

	fclose( fp );  
}

/*****************************************************************************
 *****************************************************************************/
static void outputReconPoints( const char * filename )
{
	int i, j, k=0;
  
	ofstream output( filename );
	if( !output )
		cerr << "Unable to open reconstruction output file: " << filename << endl;

	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
	std::vector<int> reconValidObject;
	reconValidObject.resize( reconObjectList.size() );

	for ( i=0; i<reconValidObject.size(); i++ )
		reconValidObject[i] = 0;

	for( i=0; i<reconObjectList.size(); ++i ) 
	{
		/*if( isReconstructedWell( i ) && reconObjectList[i].GetReconFlag() != ReconObject::RECON_NONE ) 
		{
		VM3Vector reconLocation = reconObjectList[i].GetReconLocation();

	  for ( j=0; j<reconValidObject.size(); j++ )
		  if ( reconValidObject[j] && i!=j && ( reconLocation - reconObjectList[j].GetReconLocation() ).length() < pointsDistance )
			  break;

	  if ( j == reconValidObject.size() )
	  {
		  reconValidObject[i] = 1;
		  output << reconLocation.x() << " " << reconLocation.y() << " " << reconLocation.z() << endl;
		  k++;
	  }
    }*/
		if( isReconstructedWell( i ) ) 
		{
			VM3Vector reconLocation = reconObjectList[i].GetReconLocation();
			output << reconLocation.x() << " " << reconLocation.y() << " " << reconLocation.z() << endl;
		}
		else
			output << "*" << endl;
	}

	output << k << endl;
}


/*****************************************************************************
 *****************************************************************************/
static void CompareErrors( const char * filename )
{ 
	FILE * fp = fopen( filename, "w" );
	if( fp == 0 ) 
		fprintf( stderr, "UNable to open reconstruction output file: %s\n", filename );

	int i;

	const std::vector<ReconObject> & reconObjectList = sceneData.GetReconObjectList();
	const std::vector<VM3Vector> & objectLocationList = sceneData.GetObjectLocationList();
	const std::vector<float> & projErrorList = sceneData.GetProjErrorList();

	fprintf( fp, "%f %f %f %f\n%f %f %f %f\n\n", anchorMeanError, anchorStddevError, anchorMinError, anchorMaxError,
		nonanchorMeanError, nonanchorStddevError, nonanchorMinError, nonanchorMaxError );

	fprintf( fp, "Anchor Error\n" );
	for ( i=0; i<reconObjectList.size(); i++ )
	{ 
		if ( reconObjectList[i].GetReconFlag() == ReconObject::RECON_ANCHOR )
		{ 
			VM3Vector Q1 = reconObjectList[i].GetReconLocation();
			//VM3Vector Q2 = objectLocationList[i];
	  
			//float actualError = ( Q1 - Q2 ).length();

			fprintf( fp, "%f\n", projErrorList[i] );
		}
	}

	fprintf( fp, "\nNonanchor Error\n" );
	for ( i=0; i<reconObjectList.size(); i++ )
	{ 
		if ( reconObjectList[i].GetReconFlag() == ReconObject::RECON_NONANCHOR )
		{ 
			VM3Vector Q1 = reconObjectList[i].GetReconLocation();
			//VM3Vector Q2 = objectLocationList[i];
	  
			//float actualError = ( Q1 - Q2 ).length();

			fprintf( fp, "%f\n", projErrorList[i] );
		}
	}

	fclose( fp );
}

/*****************************************************************************
	Merge Sort for std::vector<Feature> on .IntegerRange.GetLowerBound(), increasing
*****************************************************************************/
static void merge(int **numbers, int **temp, int left, int mid, int right)
{
  int i, left_end, num_elements, tmp_pos;

  left_end = mid - 1;
  tmp_pos = left;
  num_elements = right - left + 1;


  while ((left <= left_end) && (mid <= right))
  {

    if (numbers[left][0] <= numbers[mid][0])
    {

      temp[tmp_pos][0] = numbers[left][0];
	  temp[tmp_pos][1] = numbers[left][1];
      tmp_pos = tmp_pos + 1;
      left = left +1;
    }
    else
    {

      temp[tmp_pos][0] = numbers[mid][0];
	  temp[tmp_pos][1] = numbers[mid][1];
      tmp_pos = tmp_pos + 1;
      mid = mid + 1;
    }

  }

  while (left <= left_end)
  {
    temp[tmp_pos][0] = numbers[left][0];
	temp[tmp_pos][1] = numbers[left][1];
    left = left + 1;
    tmp_pos = tmp_pos + 1;
  }

  while (mid <= right)
  {
    temp[tmp_pos][0] = numbers[mid][0];
	temp[tmp_pos][1] = numbers[mid][1];
    mid = mid + 1;
    tmp_pos = tmp_pos + 1;
  }

  for (i=0; i < num_elements; i++)
  {
    numbers[right][0] = temp[right][0];
	numbers[right][1] = temp[right][1];
    right = right - 1;
  }
}

//should sort numbers on [][0] index, and maintain association between [][0] and [][1]
static void m_sort(int **numbers, int **temp, int left, int right)
{
  int mid;
  if (right > left)
  {
    mid = (right + left) / 2;

    m_sort(numbers, temp, left, mid);

	m_sort(numbers, temp, mid+1, right);

    merge(numbers, temp, left, mid+1, right);

  }
}



static std::vector<Feature> MergeSortOnLowerBound(std::vector<Feature> featureList_)//BOBEDIT
{
	//TODO: use a better algorithm and a copy of the list please. Free things as well
	//std::vector<Feature> myfeatures(featureList_);
	std::vector<Feature> myfeatures;
	int i=0, j=0;
	
	int **lowbounds = new int *[featureList_.size()];
	int **temp = new int *[featureList_.size()];


	if (lowbounds == NULL || temp == NULL)
		return NULL;
	for (i=0; i < featureList_.size(); i++)
	{
		lowbounds[i] = new int[2];
		temp[i] = new int[2];
		if (lowbounds[i] == NULL || temp[i] == NULL)
		{
			cout << "MergeSortOnLowerBound: could not allocate memory" << endl;
			exit(5);
		}

		lowbounds[i][0] = -1;	//the lower bound
		lowbounds[i][1] = -1;  //the original index in featureList_
		temp[i][0] = -1;	//the lower bound
		temp[i][1] = -1;  //the original index in featureList_
	}
	
	for (i=0; i < featureList_.size(); i++)
	{
		lowbounds[i][0] = featureList_[i].GetRange().GetLowerBound();
		lowbounds[i][1] = i;
	}	


	m_sort(lowbounds, temp, 0, featureList_.size()-1);


	for (i=0; i < featureList_.size(); i++)
		myfeatures.push_back(featureList_[lowbounds[i][1]]);

	for (i=0; i < featureList_.size(); i++)
	{
		delete [] lowbounds[i];
		delete [] temp[i];
	}
	delete [] lowbounds;
	delete [] temp;
	
	return myfeatures;
}


/********************************************************************
********************************************************************/
static void restoreDisjointSubsets(SceneData & origSceneData, std::vector<SceneData> & sceneDataSubsets)
{
	int n=0, i=0, j=0;

	//ofstream bobout ("restore_disjoint_subsets.txt");

	for(n=0; n < sceneDataSubsets.size(); n++)
	{
		//Note: image and feature lists are sorted by ascending value of absolute image and feature IDs, respectively
		j=0;

		//First, replace image dependent lists
		std::vector<int>::iterator itImg =			origSceneData.GetAbsoluteImageIDList().begin();			//absoluteImageIDList_; 
		std::vector<VM3Vector>::iterator itLoc =	origSceneData.GetViewpointLocationList().begin();			//viewpointLocationList_; 
		std::vector<VM3Vector>::iterator itReconLoc =	origSceneData.GetReconViewpointLocationList().begin();			//viewpointLocationList_;  
		std::vector<bool>::iterator	itValidOrient =	origSceneData.GetViewpointValidOrientationList().begin();	//viewpointValidOrientationList_;  
		std::vector<VM4Matrix>::iterator itOrient =	origSceneData.GetViewpointOrientationList().begin();			//viewpointOrientationList_; 
		
		//bobout << "absImageIDList.size(): " << sceneDataSubsets[n].GetAbsoluteImageIDList().size() << endl;
		for(i=0; i < sceneDataSubsets[n].GetAbsoluteImageIDList().size(); i++)
		{
	//		bobout << "i: " << i << endl;
			//Note: we should never reach the end of the origSceneData lists before the end of the new SceneData lists
			while( (*itImg) != sceneDataSubsets[n].GetAbsoluteImageIDList()[i])
			{
		//		bobout << "fly" << endl;
				itImg++;
				itLoc++;
				itReconLoc++;
				itValidOrient++;
				itOrient++;
			}

			origSceneData.GetAbsoluteImageIDList().erase(itImg);

			origSceneData.GetViewpointLocationList().erase(itLoc);
			origSceneData.GetReconViewpointLocationList().erase(itReconLoc);
			origSceneData.GetViewpointValidOrientationList().erase(itValidOrient);
			origSceneData.GetViewpointOrientationList().erase(itOrient);
	

			//insert, increment, delete
			origSceneData.GetAbsoluteImageIDList().insert( itImg, sceneDataSubsets[n].GetAbsoluteImageIDList()[i] );	//yes, this is almost pointless for GetAbsoluteImageIDList
			origSceneData.GetViewpointLocationList().insert( itLoc, sceneDataSubsets[n].GetViewpointLocationList()[i] );
			origSceneData.GetReconViewpointLocationList().insert( itReconLoc, sceneDataSubsets[n].GetReconViewpointLocationList()[i] );
			origSceneData.GetViewpointValidOrientationList().insert( itValidOrient, sceneDataSubsets[n].GetViewpointValidOrientationList()[i] );
			origSceneData.GetViewpointOrientationList().insert( itOrient, sceneDataSubsets[n].GetViewpointOrientationList()[i] );

		}
		
		
		//Second, replace feature dependent lists
		std::vector<Feature>::iterator itFeat =			origSceneData.GetFeatureList().begin();
		std::vector<ReconObject>::iterator itReconObj = origSceneData.GetReconObjectList().begin();//			reconObjectList_;
		std::vector<VM3Vector>::iterator itSBAReconLoc = origSceneData.GetSBAReconLocationList().begin();
		//std::vector<VM3Vector>::iterator itTriangulateReconLoc = origSceneData.GetTriangulateReconLocationList().begin();
		std::vector<float>::iterator itError =			origSceneData.GetProjErrorList().begin();			//		projErrorList_;	
		std::vector<VM3Vector>::iterator itObj =		origSceneData.GetObjectLocationList().begin();			//	objectLocationList_;
		//TODO: why is this -2?
		for (i=0; i < sceneDataSubsets[n].GetFeatureList().size(); i++)
		{
			while( (*itFeat).GetID() != sceneDataSubsets[n].GetFeatureList()[i].GetID() )
			{
				itFeat++;
				itReconObj++;
				itSBAReconLoc++;
				//itTriangulateReconLoc++;
				itError++;
				itObj++;
			}
			//insert, increment, delete
			//Since the FeatureList is unmodified, we don't need to update it
			origSceneData.GetReconObjectList().erase(itReconObj);
			origSceneData.GetSBAReconLocationList().erase(itSBAReconLoc);
			//origSceneData.GetTriangulateReconLocationList().erase(itTriangulateReconLoc);
			origSceneData.GetProjErrorList().erase(itError);
			origSceneData.GetObjectLocationList().erase(itObj);


			origSceneData.GetReconObjectList().insert( itReconObj, sceneDataSubsets[n].GetReconObjectList()[i] );
			origSceneData.GetSBAReconLocationList().insert( itSBAReconLoc, sceneDataSubsets[n].GetSBAReconLocationList()[i] );
			//origSceneData.GetTriangulateReconLocationList().insert( itTriangulateReconLoc, sceneDataSubsets[n].GetTriangulateReconLocationList()[i] );
			origSceneData.GetProjErrorList().insert( itError, sceneDataSubsets[n].GetProjErrorList()[i] );
			origSceneData.GetObjectLocationList().insert( itObj, sceneDataSubsets[n].GetObjectLocationList()[i] );

			itFeat++;
			itReconObj++;
			itSBAReconLoc++;
			//itTriangulateReconLoc++;
			itError++;
			itObj++;
		}

	}
	sceneData = SceneData(origSceneData);
//	bobout.close();
}


/********************************************************************
********************************************************************/
static void restoreAdjacentSubsets(SceneData & origSceneData, std::vector<SceneData> & sceneDataSubsets)
{
	int n=0, i=0, j=0;

	//ofstream bobout ("restore_adjacent_subsets.txt");

	//bobout << "sceneDataSubsets.size(): " << sceneDataSubsets.size() << endl;
	for(n=0; n < sceneDataSubsets.size(); n++)
	{

		//printf("restore scenedatasubset[%.3d]\r", n);
		//cout << "scenedatasubset[n]: " << n << endl;

		//bobout << endl << endl << "n: " << n << endl;
		//Note: image and feature lists are sorted by ascending value of absolute image and feature IDs, respectively
		j=0;

		//First, replace image dependent lists
		std::vector<int>::iterator itImg =			origSceneData.GetAbsoluteImageIDList().begin();			//absoluteImageIDList_; 
		std::vector<VM3Vector>::iterator itLoc =	origSceneData.GetViewpointLocationList().begin();			//viewpointLocationList_;  
		std::vector<bool>::iterator	itValidOrient =	origSceneData.GetViewpointValidOrientationList().begin();	//viewpointValidOrientationList_;  
		std::vector<VM4Matrix>::iterator itOrient =	origSceneData.GetViewpointOrientationList().begin();			//viewpointOrientationList_; 

		//cout << sceneDataSubsets[n].GetAbsoluteImageIDList().size() << " " << sceneDataSubsets[n].GetAbsoluteImageIDList()[0] << " " 
		//	<< sceneDataSubsets[n].GetFeatureList().size() << endl;
	//	bobout << "absImageIDList.size(): " << sceneDataSubsets[n].GetAbsoluteImageIDList().size() << endl;

		for(i=0; i < sceneDataSubsets[n].GetAbsoluteImageIDList().size(); i++)
		{
	//		bobout << "i1: " << i << " "<< sceneDataSubsets[n].GetAbsoluteImageIDList()[i] << endl;
			//Note: we should never reach the end of the origSceneData lists before the end of the new SceneData lists
			while( (*itImg) != sceneDataSubsets[n].GetAbsoluteImageIDList()[i])
			{

			//	bobout << "fly: " << (*itImg) << endl;

				itImg++;
				itLoc++;
				itValidOrient++;
				itOrient++;
			}

			origSceneData.GetAbsoluteImageIDList().erase(itImg);

			origSceneData.GetViewpointLocationList().erase(itLoc);
			origSceneData.GetViewpointValidOrientationList().erase(itValidOrient);
			origSceneData.GetViewpointOrientationList().erase(itOrient);
	

			//insert, increment, delete
			origSceneData.GetAbsoluteImageIDList().insert( itImg, sceneDataSubsets[n].GetAbsoluteImageIDList()[i] );	//yes, this is almost pointless for GetAbsoluteImageIDList
			origSceneData.GetViewpointLocationList().insert( itLoc, sceneDataSubsets[n].GetViewpointLocationList()[i] );
			origSceneData.GetViewpointValidOrientationList().insert( itValidOrient, sceneDataSubsets[n].GetViewpointValidOrientationList()[i] );
			origSceneData.GetViewpointOrientationList().insert( itOrient, sceneDataSubsets[n].GetViewpointOrientationList()[i] );

		}
	
		
		//Second, replace feature dependent lists
		std::vector<Feature>::iterator itFeat =			origSceneData.GetFeatureList().begin();
		std::vector<ReconObject>::iterator itReconObj = origSceneData.GetReconObjectList().begin();//			reconObjectList_;
		std::vector<VM3Vector>::iterator itSBAReconLoc = origSceneData.GetSBAReconLocationList().begin();
		//std::vector<VM3Vector>::iterator itTriangulateReconLoc = origSceneData.GetTriangulateReconLocationList().begin();
		std::vector<float>::iterator itError =			origSceneData.GetProjErrorList().begin();			//		projErrorList_;	
		std::vector<VM3Vector>::iterator itObj =		origSceneData.GetObjectLocationList().begin();			//	objectLocationList_;

	//	bobout << "size below: " << sceneDataSubsets[n].GetFeatureList().size() << endl;

		for (i=0; i < sceneDataSubsets[n].GetFeatureList().size(); i++)
		{
	//		bobout << "i2: " << i << " " << (*itFeat).GetID() << " " << sceneDataSubsets[n].GetFeatureList()[i].GetID() << endl;
			while( (*itFeat).GetID() != sceneDataSubsets[n].GetFeatureList()[i].GetID() )
			{
	//			bobout << "looping" << endl;
	//			bobout << (*itFeat).GetID() << " " << sceneDataSubsets[n].GetFeatureList()[i].GetID() << endl; 
				itFeat++;
				itReconObj++;
				itSBAReconLoc++;
				//itTriangulateReconLoc++;
				itError++;
				itObj++;
			}
		

			//insert, increment, delete
			//Since the FeatureList is unmodified, we don't need to update it
			origSceneData.GetReconObjectList().erase(itReconObj);
			origSceneData.GetSBAReconLocationList().erase(itSBAReconLoc);
			//origSceneData.GetTriangulateReconLocationList().erase(itTriangulateReconLoc);
			origSceneData.GetProjErrorList().erase(itError);
			origSceneData.GetObjectLocationList().erase(itObj);


			origSceneData.GetReconObjectList().insert( itReconObj, sceneDataSubsets[n].GetReconObjectList()[i] );
			origSceneData.GetSBAReconLocationList().insert( itSBAReconLoc, sceneDataSubsets[n].GetSBAReconLocationList()[i] );
			//origSceneData.GetTriangulateReconLocationList().insert( itTriangulateReconLoc, sceneDataSubsets[n].GetTriangulateReconLocationList()[i] );
			origSceneData.GetProjErrorList().insert( itError, sceneDataSubsets[n].GetProjErrorList()[i] );
			origSceneData.GetObjectLocationList().insert( itObj, sceneDataSubsets[n].GetObjectLocationList()[i] );

			
			itFeat++;
			itReconObj++;
			itSBAReconLoc++;
			//itTriangulateReconLoc++;
			itError++;
			itObj++;
		}
	}

	printf("\n");

	sceneData = SceneData(origSceneData);


	//bobout.close();

}
/*****************************************************************************
*****************************************************************************/

static void extractDisjointSubsets(std::vector<SceneData> & sceneDataSubsets)
{
		//make copy of sceneData
		SceneData origSceneData = sceneData;
		
		//ofstream bobout("bob_extract_subsets.txt");
		int withdrawImageCount_backup = withdrawImageCount;
		std::vector<int> absRefinedImages;
		std::vector<int> absRefinedFeatures;
		int i = 0, j = 0, n=0;
		int *relImagePosition = new int[withdrawImageCount];									//keeps track of the images set to be removed
		int **goodFeatureList = new int *[withdrawImageCount - disjointMinFeatureLife+1];			//keeps track of the features that might be removed, relative
		int *absImageIDsToRemove = new int[withdrawImageCount];
		int *finalFeaturesToRemove = new int[disjointMaxFeaturesPerImageGroup];						//keeps track of the features that WILL be removed, relative
		int firstImageSeen = 0, lastImageSeen = 0;
		int numberImagesSeen = 0;
		int imageSpacing = 0;												
		bool spotFilled = false;
		int totalNumberOfImages;

	//	cout << "extractDisjointSubsets. # pts: " << sceneData.GetFeatureList().size() << "\t#cameras: " << sceneData.GetViewpointLocationList().size() << endl;

		//If dynamic allocation fails
		if (relImagePosition == NULL || goodFeatureList == NULL || finalFeaturesToRemove == NULL)
		{
			cout << "guiCB(SUBSETREMOVAL): could not allocate memory: " << relImagePosition << " " << goodFeatureList << " " << finalFeaturesToRemove << endl;
			exit(3);
		}

	//	bobout << "withdrawimagecount: " << withdrawImageCount << "\tdisjointMinFeatureLife: " <<  disjointMinFeatureLife <<"\twd-mfl+1: " << withdrawImageCount - disjointMinFeatureLife+1 << endl;
		for (i=0; i < withdrawImageCount - disjointMinFeatureLife+1; i++)
		{
			goodFeatureList[i] = new int[disjointMaxFeaturesPerImageGroup];
			if (goodFeatureList[i] == NULL)
			{
				cout << "guiCB(SUBSETREMOVAL)2: could not allocate memory:" << endl;
				exit(4);
			}
			for (j=0; j < disjointMaxFeaturesPerImageGroup; j++)
			{
				goodFeatureList[i][j] = -1;
			//	bobout << "GoodFeatureList[" << i << "][" << j  << "] = " << goodFeatureList[i][j] << endl;
			}
		}

		//DEBUG
	//	for (i=0; i < sceneData.GetFeatureList().size(); i++)
	//		bobout << "i: " << i << "\tfeatID: " << sceneData.GetFeatureList()[i].GetID() << "\tlowBound: " << sceneData.GetFeatureList()[i].GetRange().GetLowerBound() << endl;
	//	bobout << endl << "Viewpoints: " << endl;
	//	for (i=0; i < sceneData.GetViewpointLocationList().size(); i++)
	//		bobout << "i: " << i << " " << sceneData.GetViewpointLocationList()[i].x() << " "
	//				<< sceneData.GetViewpointLocationList()[i].y() << " " << sceneData.GetViewpointLocationList()[i].z() << endl;

	/*
		//gflcheck
		for (i=0; i < withdrawImageCount - disjointMinFeatureLife+1; i++)
		{
			bobout << "gflcheck prei: " << i << endl;
				bobout << "gflcheck: " << i << "\t: " << goodFeatureList[i][0] << endl;
		}
*/

		//BEGIN LARGE ITERATION
		for (n=0; n < subsetIterationCount; n++)
		{
			//bobout << sceneData.GetCameraParameters().realLength << " " << sceneData.GetCameraParameters().realWidth << endl;
			totalNumberOfImages = sceneData.GetViewpointLocationList().size();
		//	cout << "totalImagesNo: " << totalNumberOfImages << "\ttotalfeatures: " << sceneData.GetFeatureList().size() << endl;


			//INITIALIZE ARRAYS
			{
				for (i=0; i < withdrawImageCount - disjointMinFeatureLife+1; i++)
				{
					for (j=0; j < disjointMaxFeaturesPerImageGroup; j++)
						goodFeatureList[i][j] = -1;
				}
	
				for (i=0; i < withdrawImageCount; i++)
					relImagePosition[i] = -1;
	
				for (i=0; i < disjointMaxFeaturesPerImageGroup; i++)
					finalFeaturesToRemove[i] = -1;
			}
	
	
			//sorts features by ascending lowest bound
			std::vector<Feature> myfeatures = MergeSortOnLowerBound(sceneData.GetFeatureList());
					
			if (totalNumberOfImages < withdrawImageCount)
				withdrawImageCount = totalNumberOfImages;
			if (withdrawImageCount < disjointMinFeatureLife)
				break;


			//cout << "totalImages " << totalNumberOfImages << endl;
			imageSpacing = totalNumberOfImages / withdrawImageCount;
			
			
			//	divide feat0's range into pieces by withdrawImageCount
			//		feat0.range / withdrawImageCount... img0 = *1, img1 = *2, img2 = *3...
			for (i=0; i < withdrawImageCount; i++)
				relImagePosition[i] = imageSpacing * i;
	
	
			//this loop fills 2-d array goodFeatureList[][] with features that span at least disjointMinFeatureLife images.
			//essentially, for all features that could include disjointMinFeatureLife images
			//iterate through features: while each feature's lowerbound is low enough to include disjointMinFeatureLife images
			for (i=0; myfeatures[i].GetRange().GetLowerBound() <= sceneData.GetAbsoluteImageIDList()[relImagePosition[withdrawImageCount-disjointMinFeatureLife]] && i < myfeatures.size(); i++)
			{
				//bobout << "gfl loop i: " << i << endl;
				//if feature range encompasses at least disjointMinFeatureLife images
					//put it in the first image seen array slot
				//calculate firstImageSeen
				firstImageSeen = -1;
				lastImageSeen = -1;
				for (j=0; j < withdrawImageCount;j++)
				{	
					if (firstImageSeen == -1 && myfeatures[i].GetRange().GetLowerBound() <= sceneData.GetAbsoluteImageIDList()[relImagePosition[j]])
						firstImageSeen = j;
					if (sceneData.GetAbsoluteImageIDList()[relImagePosition[j]] <= (myfeatures[i].GetRange().GetLowerBound() + myfeatures[i].GetRange().GetSize()))
						lastImageSeen = j;
				}
				if (firstImageSeen == -1 || lastImageSeen == -1)
					continue;
	
			//	bobout << "A firstimageseen: " << firstImageSeen << " lastImageSeen: " << lastImageSeen << endl;
			
	
				//calculate number of images seen
				numberImagesSeen = lastImageSeen - firstImageSeen + 1; //+1 is for inclusiveness
	
				if (numberImagesSeen < disjointMinFeatureLife)
					continue;
	
			//	bobout << "B numimagesSeen: " << numberImagesSeen << "\tabsFID: " << myfeatures[i].GetID() << endl;
	
				spotFilled = false;
				//if empty slot in array, fill it
				for (j=0; j < disjointMaxFeaturesPerImageGroup; j++)
				{
				//	bobout << "j: " << j << endl;
					//if a spot is empty -> -1
					if (goodFeatureList[firstImageSeen][j] == -1)
					{
						//bobout << "assign " << i << " to position " << j << endl;
						goodFeatureList[firstImageSeen][j] = i;
						spotFilled = true;
						break;
					}
				}
			//	bobout << "C: " << spotFilled << "\n";
				//else, fill array with features that have longest range
				if (spotFilled == false)
				{
					//bobout << "spot unfilled" << endl;
					int minRangeIndex = 0;
					for (j=0; j < disjointMaxFeaturesPerImageGroup; j++)
					{
						if (myfeatures[goodFeatureList[firstImageSeen][j]].GetRange().GetSize() < myfeatures[goodFeatureList[firstImageSeen][minRangeIndex]].GetRange().GetSize())
							minRangeIndex = j;
					}
					if (myfeatures[goodFeatureList[firstImageSeen][minRangeIndex]].GetRange().GetSize() < myfeatures[i].GetRange().GetSize() )
					{
						//bobout << "[][]: [" << firstImageSeen << "][" << minRangeIndex << "] = " << i << endl;
						goodFeatureList[firstImageSeen][minRangeIndex] = i;
					}
				}
		//		bobout << "D\n";
			}
			//bobout << "0D" << endl;
			
			
			int avgRange = 0;
			for (i=0; i < myfeatures.size(); i++)
				avgRange += myfeatures[i].GetRange().GetSize();
			avgRange /= myfeatures.size();

			//should be the case that avg range = totalImages / withdrawImageCount * disjointMinFeatureLife
			if (goodFeatureList[0][0] == -1)
			{
				cout << "Group without a valid feature." << endl;
				//cout << "The parameters entered result in not a single valid feature being found. The next two lines suggest alternate parameters: " << endl;
				//cout << "   Parameters not good: avgrange: " << avgRange << "\t withdrawImageCount: " << withdrawImageCount << " " << totalNumberOfImages << endl;
				//cout << "   suggested better imageCount with locked featureLife: " << totalNumberOfImages * disjointMinFeatureLife / avgRange << endl << endl;
				continue;
			}

			//Choose the best features to be removed from the tentative list of features that can be removed
			int counter = 0;
			for (j=0; j < disjointMaxFeaturesPerImageGroup; j++)
			{
				for (i=0; i < (withdrawImageCount-disjointMinFeatureLife + 1) && counter < disjointMaxFeaturesPerImageGroup; i++)
				{
					if(goodFeatureList[i][j] != -1)
					{
						//bobout << "[i][j] = [" << i << "][j" << j << "], gfl[i][j] = " << goodFeatureList[i][j] << "\tcounter: " << counter << endl;
						finalFeaturesToRemove[counter] = goodFeatureList[i][j];
						counter++;
					}
				}
			}
			//BOB COME HERE
		//	if (counter < disjointMaxFeaturesPerImageGroup)
		//	{
		//		cout << "Not enough features found in group" << endl;
		//		continue;
		//	}
				//bobout << "0E" << endl;
				
			//remove images
			std::vector<int>::iterator itImg =	sceneData.GetAbsoluteImageIDList().begin();			//absoluteImageIDList_; 
			std::vector<VM3Vector>::iterator itLoc =	sceneData.GetViewpointLocationList().begin();			//viewpointLocationList_;  
			std::vector<VM3Vector>::iterator itReconLoc =	sceneData.GetReconViewpointLocationList().begin();			//viewpointLocationList_; 
			std::vector<bool>::iterator	itValidOrient =	sceneData.GetViewpointValidOrientationList().begin();	//viewpointValidOrientationList_;  
			std::vector<VM4Matrix>::iterator itOrient =	sceneData.GetViewpointOrientationList().begin();			//viewpointOrientationList_; 
	
			//DO NOT remove this, it will do nothing on the first pass, but on subsequent passes the values will differ
			for (i=0; i < withdrawImageCount; i++)
			{
				absImageIDsToRemove[i] = sceneData.GetAbsoluteImageIDList()[relImagePosition[i]];
			//	bobout << "absImageIDs to remove: " << absImageIDsToRemove[i] << endl;
			}
		//	bobout << endl;

			for (; itImg != sceneData.GetAbsoluteImageIDList().end(); itImg++, itLoc++, itReconLoc++, itValidOrient++, itOrient++)
			{
				for (j=0; j<withdrawImageCount; j++)
				{
					if ((*itImg) == absImageIDsToRemove[j])
					{
						sceneData.GetAbsoluteImageIDList().erase(itImg);
						sceneData.GetViewpointLocationList().erase(itLoc);
						sceneData.GetReconViewpointLocationList().erase(itReconLoc);
						sceneData.GetViewpointValidOrientationList().erase(itValidOrient);
						sceneData.GetViewpointOrientationList().erase(itOrient);
						itImg--;
						itLoc--;
						itReconLoc--;
						itValidOrient--;
						itOrient--;
						break;
					}
				}
			}



			//remove features
			std::vector<Feature>::iterator itFeat = sceneData.GetFeatureList().begin();
			std::vector<ReconObject>::iterator itReconObj = sceneData.GetReconObjectList().begin();//			reconObjectList_;  
			std::vector<VM3Vector>::iterator itSBAReconLoc = sceneData.GetSBAReconLocationList().begin();
			//std::vector<VM3Vector>::iterator itTriangulateReconLoc = sceneData.GetTriangulateReconLocationList().begin();
			std::vector<float>::iterator itError = sceneData.GetProjErrorList().begin();			//		projErrorList_;	
			std::vector<VM3Vector>::iterator itObj = sceneData.GetObjectLocationList().begin();			//	objectLocationList_;

		//	bobout << "Before: " << sceneData.GetFeatureList().size() << " " << sceneData.GetReconObjectList().size() << " " << sceneData.GetProjErrorList().size() << " " << 
		//		sceneData.GetObjectLocationList().size() << endl;
		//	cout << "the SIZE: " << sceneData.GetSBAReconLocationList().size() << " " << sceneData.GetTriangulateReconLocationList().size() << " " 
			//		<< sceneData.GetFeatureList().size() << endl;

			//note: all lists above MUST and SHOULD have the same length. This will not be checked here.
			//TODO: sort (finalFeaturesToRemove) using n log n function (it will save time if a HUGE number of features are being removed)
			for (; itFeat != sceneData.GetFeatureList().end(); itFeat++, itReconObj++, itSBAReconLoc++, itError++, itObj++)
			{
				//bobout << "ID: " << (*itFeat).GetID() << endl;
				for (j=0; j<disjointMaxFeaturesPerImageGroup; j++)
				{
					if ((*itFeat).GetID() == myfeatures[finalFeaturesToRemove[j]].GetID())
					{			
						//bobout << "j: " << j << "\tabsFID: " << myfeatures[finalFeaturesToRemove[j]].GetID() << endl;//DEBUG
							
						sceneData.GetFeatureList().erase(itFeat);
						sceneData.GetReconObjectList().erase(itReconObj);
						sceneData.GetSBAReconLocationList().erase(itSBAReconLoc);
						//sceneData.GetTriangulateReconLocationList().erase(itTriangulateReconLoc);
						
						sceneData.GetProjErrorList().erase(itError);
						sceneData.GetObjectLocationList().erase(itObj);
						itFeat--;
						itReconObj--;
						itSBAReconLoc--;
						//itTriangulateReconLoc--;
						itError--;
						itObj--;
						break;
					}
				}
			}

	
			//cout << "the SIZE: " << sceneData.GetSBAReconLocationList().size() << " " << sceneData.GetTriangulateReconLocationList().size() << " " 
			//		<< sceneData.GetFeatureList().size() << endl;
		

			for (i=0; i < withdrawImageCount; i++)
			{
				if (absImageIDsToRemove[i] != -1)
					absRefinedImages.push_back(absImageIDsToRemove[i]);
			}
			for(i=0; i < disjointMaxFeaturesPerImageGroup; i++)
			{
				if (finalFeaturesToRemove[i] != -1)
				{
					absRefinedFeatures.push_back(myfeatures[finalFeaturesToRemove[i]].GetID());
					//bobout << "i: " << i << "finalfeaturetoremove: " << finalFeaturesToRemove[i] << "\tFID: " << myfeatures[finalFeaturesToRemove[i]].GetID() << endl;
				}
			}

			//create a vector of mini scene_datum

			if (absRefinedImages.size() == 0 || absRefinedFeatures.size() == 0)
				continue;
			//TODO: if this break/continue occurs, undo deletion... or rather don't delete and only delete if we pass this point

			//sort images and features that will be saved
			sort(absRefinedImages.begin(), absRefinedImages.end());
			sort(absRefinedFeatures.begin(), absRefinedFeatures.end());

			//DEBUG
			/*
			bobout << "AbsRefinedImages" << endl;
			for(i=0; i < absRefinedImages.size(); i++)
				bobout << "i: " << i << "\timageId: " << absRefinedImages[i] << endl;
			bobout << endl;
			bobout << "AbsRefinedFeatures" << endl;
			bobout << "absrefinedfeatures.size(): " << absRefinedFeatures.size() << endl;
			for(i=0; i < absRefinedFeatures.size(); i++)
				bobout << "i: " << i << "\tfeatureId: " << absRefinedFeatures[i] << endl;//"\tLowerBound: " << origSceneData.GetFeatureList()[absRefinedFeatures[i]].GetRange().GetLowerBound() << endl;
			bobout << endl;
*/

	
			std::vector<VM3Vector> viewpointLocationList_temp;
			std::vector<VM3Vector> reconViewpointLocationList_temp;
			std::vector<int> absoluteImageIDList_temp; 
			std::vector<bool> viewpointValidOrientationList_temp;  
			std::vector<VM4Matrix> viewpointOrientationList_temp; 
			std::vector<Feature> featureList_temp;  	
			std::vector<ReconObject> reconObjectList_temp;  
			std::vector<VM3Vector> SBAReconLocationList_temp;
			//std::vector<VM3Vector> triangulateReconLocationList_temp;
			std::vector<float> projErrorList_temp;	
			std::vector<VM3Vector> objectLocationList_temp;  	

			//Save selected indices of image-dependent vectors
			for (i=0; i < absRefinedImages.size(); i++)
			{
				viewpointLocationList_temp.push_back(			origSceneData.GetViewpointLocationList()[absRefinedImages[i]] );
				reconViewpointLocationList_temp.push_back(			origSceneData.GetReconViewpointLocationList()[absRefinedImages[i]] );
				absoluteImageIDList_temp.push_back(				absRefinedImages[i] );
				viewpointValidOrientationList_temp.push_back(	origSceneData.GetViewpointValidOrientationList()[absRefinedImages[i]] );
				viewpointOrientationList_temp.push_back(		origSceneData.GetViewpointOrientationList()[absRefinedImages[i]] );
			}
	
			//Save selected indices of feature-dependent vectors

			for (i=0; i < absRefinedFeatures.size(); i++)
			{
				//cout << "i: " << i << "\tabsrefined[i]: " << absRefinedFeatures[i] << endl;
				featureList_temp.push_back(					origSceneData.GetFeatureList()[absRefinedFeatures[i]] );
				reconObjectList_temp.push_back(				origSceneData.GetReconObjectList()[absRefinedFeatures[i]] );
				
				SBAReconLocationList_temp.push_back(		origSceneData.GetSBAReconLocationList()[absRefinedFeatures[i]] );
				//triangulateReconLocationList_temp.push_back(origSceneData.GetTriangulateReconLocationList()[absRefinedFeatures[i]] );
				projErrorList_temp.push_back(				origSceneData.GetProjErrorList()[absRefinedFeatures[i]] );
				objectLocationList_temp.push_back(			origSceneData.GetObjectLocationList()[absRefinedFeatures[i]] );
			}

	
			
			//reduce the number of projections for each feature to correspond to just the number of images in its scene
			std::vector<Feature::Location> locList;
			int lowIndexIn; //for lowest index (inclusive): gives the index of the first image in 'absoluteImageIDList_temp' seen by the current feature
			int highIndexEx;//for high index (exclusive): gives one past the index of the last image in 'absoluteImageIDList_temp' seen by the current feature
			
			for (i=0; i < featureList_temp.size(); i++)
			{
				//calculate size
		//		bobout << featureList_temp[i].GetID() << " " << featureList_temp[i].GetRange().GetLowerBound() << "\t"
		//				<< origSceneData.GetFeatureList()[featureList_temp[i].GetID()].GetID() << endl;
				IntegerRange imageRange = origSceneData.GetFeatureList()[featureList_temp[i].GetID()].GetRange();
				lowIndexIn = -1;
				highIndexEx = absoluteImageIDList_temp.size();
				for (j=0; j < absoluteImageIDList_temp.size(); j++)
				{
			//		bobout << "j: " << j << "\tlb: " << imageRange.GetLowerBound() << "\tabsImgId: " << absoluteImageIDList_temp[j] << "\tlowindex: " << lowIndexIn << endl;
					if (lowIndexIn == -1 && imageRange.GetLowerBound() <= absoluteImageIDList_temp[j])
						lowIndexIn = j;
					if (imageRange.GetUpperBound() <= absoluteImageIDList_temp[j])
					{
						highIndexEx = j;
						break;
					}
				}
			
		//		bobout << "lowindexIN: " << lowIndexIn << "\thighindexex: " << highIndexEx << endl;

				int currentIndex = lowIndexIn;
				int firstImageFrame = -1;
				for( j=imageRange.GetLowerBound(); j<imageRange.GetUpperBound() && currentIndex < highIndexEx; j++ ) 
				{
					if (j == absoluteImageIDList_temp[currentIndex])
					{
						if (firstImageFrame == -1)
							firstImageFrame = currentIndex;
						currentIndex++;
						locList.push_back( origSceneData.GetFeatureList()[absRefinedFeatures[i]].GetLocation( j ) );
					}
					
				}
				featureList_temp[i].overwriteLocationList(firstImageFrame, locList);



				//DEBUG
/*		
				bobout << "Loclist size: " << locList.size() << "\tactualListSize: " << featureList_temp[i].GetRange().GetSize() << endl;
				bobout << "featureList_temp[" << i << "]'s ID: " << featureList_temp[i].GetID() << endl;
			//	for(j=0; j < featureList_temp[i].GetRange().GetSize(); j++)
			//		bobout << "  " << featureList_temp[i].GetLocation(j).x_ << " " << featureList_temp[i].GetLocation(j).y_ << endl;
				for(j=0; j < locList.size(); j++)
					bobout << "  " << locList[j].x_ << " " << locList[j].y_ << "\txaddress: " << &locList[j] << endl;
				bobout << "mid" << endl;
*/
		//		bobout << "Feature ID: " << featureList_temp[i].GetID() << endl;
		//		for(j=lowIndexIn; j < highIndexEx; j++)
		//		{
				//	bobout << "featureListsize: " << featureList_temp[i].GetRange().GetSize() << "\tj: " << j << endl;
					//bobout << "lowerbound: " << imageRange.GetLowerBound() << "\tcurrentIndex: " << currentIndex << "\tabsID: " << absoluteImageIDList_temp[firstImageFrame] 
					//<< "\tfirstImageFrame: " << firstImageFrame << endl;

		//			bobout << "x " << featureList_temp[i].GetLocation(j).x_;
		//			bobout << "\ty " << featureList_temp[i].GetLocation(j).y_ << endl;
		//		}



				while(locList.size() > 0)
					locList.pop_back();

				//bobout <<"end" << endl;
			}
			

			//does this need to be freed?
			SceneData tempSceneData ( viewpointLocationList_temp, absoluteImageIDList_temp, viewpointValidOrientationList_temp, viewpointOrientationList_temp,
					featureList_temp, reconObjectList_temp, reconViewpointLocationList_temp, 
					SBAReconLocationList_temp, //triangulateReconLocationList_temp,
					projErrorList_temp, objectLocationList_temp, 0, 0, 0, origSceneData.GetCameraParameters());
			sceneDataSubsets.push_back(tempSceneData);


			//empty vectors
			while (absRefinedImages.size() > 0)
			{
				absRefinedImages.pop_back();
				viewpointLocationList_temp.pop_back();
				absoluteImageIDList_temp.pop_back();
				viewpointValidOrientationList_temp.pop_back();
				viewpointOrientationList_temp.pop_back();
			}
			while (absRefinedFeatures.size() > 0)
			{
				absRefinedFeatures.pop_back();
				featureList_temp.pop_back();
				reconObjectList_temp.pop_back();
				SBAReconLocationList_temp.pop_back();
				//triangulateReconLocationList_temp.pop_back();
				projErrorList_temp.pop_back();
				objectLocationList_temp.pop_back();
			}
		}//END LARGE ITERATION
/************************************************************************/

		withdrawImageCount = withdrawImageCount_backup;

		//memory cleanup
		delete [] relImagePosition;
		delete [] absImageIDsToRemove;
		delete [] finalFeaturesToRemove;
		//DEBUG
		/*
		if (disjointMinFeatureLife == 352)
		{		
			for (i=0; i < withdrawImageCount - disjointMinFeatureLife+1; i++)
			{
				bobout << "rari: " << i << "\t: " << goodFeatureList[i][0] << endl;
			}
		}
		*/
		for (i=0; i < withdrawImageCount - disjointMinFeatureLife+1; i++)
			delete [] goodFeatureList[i];	
		delete [] goodFeatureList;
		//delete std::vector<Features> myfeatures;
		//TODO: free myfeatures

		//bobout.close();
}
/*****************************************************************************
*****************************************************************************/
static void extractAdjacentSubsets(std::vector<SceneData> & sceneDataSubsets)
{
	//ofstream bobout("extract_adjacent_subsets.txt");

	//bobout << "current grou[ size: " << adjacentGroupSize << endl;
	
	
	int n=0, i=0, j=0;
	
	std::vector<Feature> myfeatures = MergeSortOnLowerBound(sceneData.GetFeatureList());
	std::vector<SceneData> sceneDataArray;
	std::vector<int> absFeatureList;
	std::vector<bool> isFeatureUsedList;

	isFeatureUsedList.resize( myfeatures.size() );
	for (i=0; i < isFeatureUsedList.size(); i++)
		isFeatureUsedList[i] = false;


	int firstImageInGroup = 0;
	int firstImageOutOfGroup = 0;


	//bobout << "isfeatureusedlist.size(): " << isFeatureUsedList.size() << endl;
	//n is group number, while less than total number of groups
	//THE EDIT
	int numGroups = (sceneData.GetAbsoluteImageIDList().size() + adjacentGroupSize - 1) / adjacentGroupSize;
	//if (numGroups * adjacentGroupSize != sceneData.GetAbsoluteImageIDList().size() + adjacentGroupSize - 1)
	//	numGroups++;
	//cout << "numGroups: " << numGroups << endl;


	for (n=0; n < numGroups; n++)
	{
		/*firstImageInGroup = n * adjacentGroupSize;
		firstImageOutOfGroup = (n+1) * adjacentGroupSize;
		if( firstImageOutOfGroup > sceneData.GetAbsoluteImageIDList().size() )
			firstImageOutOfGroup = sceneData.GetAbsoluteImageIDList().size();*/

		//cout << firstImageInGroup << " " << firstImageOutOfGroup << endl;
		firstImageInGroup = firstImageOutOfGroup;
		while( firstImageInGroup < sceneData.GetAbsoluteImageIDList().size() && !sceneData.GetViewpointValidOrientationList()[firstImageInGroup] )
			firstImageInGroup ++;

		//cout << "hmm " << firstImageInGroup << endl;
		if ( firstImageInGroup == sceneData.GetAbsoluteImageIDList().size() )
			break;
		
		firstImageOutOfGroup = firstImageInGroup + 1;

		while( firstImageOutOfGroup < sceneData.GetAbsoluteImageIDList().size() &&
			   sceneData.GetViewpointValidOrientationList()[firstImageOutOfGroup] &&
			   firstImageOutOfGroup - firstImageInGroup < adjacentGroupSize )
			firstImageOutOfGroup ++;

		//cout << "entering loop. myfeatures.size(): " << myfeatures.size() << endl;
		//cout <<" firstImageInGroup: " << firstImageInGroup << " firstImageoutofgroup: " << firstImageOutOfGroup << endl;
		int featureCounter = 0;
		//iterate through each feature which index could see
		for ( i=0; myfeatures[i].GetRange().GetLowerBound() < firstImageOutOfGroup && i < myfeatures.size(); i++)
		{
			if ( ( (myfeatures[i].GetRange().GetUpperBound() - firstImageInGroup) >= adjacentMinFeatureLife		//if the range overlaps more than minFeatureLife
						|| (myfeatures[i].GetRange().GetLowerBound() < firstImageOutOfGroup						//or if
								&& myfeatures[i].GetRange().GetUpperBound() == sceneData.GetAbsoluteImageIDList().size() ) )
				//(myfeatures[i].GetRangeSize() >= adjacentMinFeatureLife)	//if the range of the feature is sufficient
				//	&& (myfeatures[i].GetRange().GetLowerBound() <= firstImageInGroup)
					&& (featureCounter < adjacentMaxFeaturesPerImageGroup)		//if we still have features to extract
					&& (isFeatureUsedList[i] == false) )						//if the feature isn't already in use
			{
					
				
					//bobout << "FeatureID: " << myfeatures[i].GetID() << "\tFirstImgSeen: " << myfeatures[i].GetRange().GetLowerBound() << 
					//	"\tUpperBound: " << myfeatures[i].GetRange().GetUpperBound() << endl;
					absFeatureList.push_back(myfeatures[i].GetID());
					isFeatureUsedList[i] = true;
					//bobout << "FeatureCounter: " << featureCounter << "\tadjacMaxFeat: " << adjacentMaxFeaturesPerImageGroup << endl;
					featureCounter++;
			}
		}
		//cout << "ok" << endl;
		
		if (absFeatureList.size() == 0)
		{			
			//cout << "uh-oh, skipping" << endl;
				//((myfeatures[n].GetRange().GetLowerBound() - firstImageInGroup)
				//		/adjacentGroupSize)*adjacentGroupSize;//myfeatures[n].GetRange().GetLowerBound();
				
			continue;
		}
	
		sort(absFeatureList.begin(), absFeatureList.end());
		//	cout << "start creation of sceneDataSubset" << endl;
		//Create sceneDataSubset
			
		std::vector<VM3Vector> viewpointLocationList_temp;
		std::vector<VM3Vector> reconViewpointLocationList_temp;
		std::vector<int> absoluteImageIDList_temp; 
		std::vector<bool> viewpointValidOrientationList_temp;  
		std::vector<VM4Matrix> viewpointOrientationList_temp; 
		std::vector<Feature> featureList_temp;  	
		std::vector<ReconObject> reconObjectList_temp;  
		std::vector<VM3Vector> SBAReconLocationList_temp;
		//std::vector<VM3Vector> triangulateReconLocationList_temp;
		std::vector<float> projErrorList_temp;	
		std::vector<VM3Vector> objectLocationList_temp;  	


		
		//Save selected indices of image-dependent vectors
		for (i=firstImageInGroup; i < firstImageOutOfGroup; i++)
		{
			viewpointLocationList_temp.push_back(			sceneData.GetViewpointLocationList()[i] );
			reconViewpointLocationList_temp.push_back(			sceneData.GetReconViewpointLocationList()[i] );
			absoluteImageIDList_temp.push_back(				i );
			viewpointValidOrientationList_temp.push_back(	sceneData.GetViewpointValidOrientationList()[i] );
			viewpointOrientationList_temp.push_back(		sceneData.GetViewpointOrientationList()[i] );
		}


		//bobout << sceneData.GetObjectLocationList().size() << " " << sceneData.GetSBAReconLocationList().size() << " " <<
		//	sceneData.GetReconObjectList().size() << " " << sceneData.GetTriangulateReconLocationList().size() << endl;
		//Save selected indices of feature-dependent vectors
		for (i=0; i < absFeatureList.size(); i++)
		{
		//	bobout << "i: " << i << "\tabsrefined[i]: " << absFeatureList[i] << endl;
			featureList_temp.push_back(					sceneData.GetFeatureList()[absFeatureList[i]] );
			reconObjectList_temp.push_back(				sceneData.GetReconObjectList()[absFeatureList[i]] );
		
			SBAReconLocationList_temp.push_back(		sceneData.GetSBAReconLocationList()[absFeatureList[i]] );
		//	triangulateReconLocationList_temp.push_back(sceneData.GetTriangulateReconLocationList()[absFeatureList[i]] );
			projErrorList_temp.push_back(				sceneData.GetProjErrorList()[absFeatureList[i]] );
			objectLocationList_temp.push_back(			sceneData.GetObjectLocationList()[absFeatureList[i]] );
		}
	
		//for (i=0; i < featureList_temp.size(); i++)
		//	bobout << "ay1 before. i: " << i << "\tFID: " << featureList_temp[i].GetID() << "\tlb: " << featureList_temp[i].GetRange().GetLowerBound() << endl;

		
		//reduce the number of projections for each feature to correspond to just the number of images in its scene
		std::vector<Feature::Location> locList;
		int lowIndexIn; //for lowest index (inclusive): gives the index of the first image in 'absoluteImageIDList_temp' seen by the current feature
		int highIndexEx;//for high index (exclusive): gives one past the index of the last image in 'absoluteImageIDList_temp' seen by the current feature
		
		//bobout << "FL size: " << featureList_temp.size() << endl;
		//bobout << "absImgID.size: " << absoluteImageIDList_temp.size() << endl;
		for (i=0; i < featureList_temp.size(); i++)
		{
		//calculate size
			//	bobout << featureList_temp[i].GetID() << " " << featureList_temp[i].GetRange().GetLowerBound() << "\t"
			//				<< sceneData.GetFeatureList()[featureList_temp[i].GetID()].GetID() << endl;
			IntegerRange imageRange = sceneData.GetFeatureList()[featureList_temp[i].GetID()].GetRange();
			lowIndexIn = -1;
			highIndexEx = absoluteImageIDList_temp.size();
			for (j=0; j < absoluteImageIDList_temp.size(); j++)
			{
		//		bobout << "j: " << j << "\tlb: " << imageRange.GetLowerBound() << "\tabsImgId: " << absoluteImageIDList_temp[j] << "\tlowindex: " << lowIndexIn << endl;
				if (lowIndexIn == -1 && imageRange.GetLowerBound() <= absoluteImageIDList_temp[j])
					lowIndexIn = j;
				if (imageRange.GetUpperBound() <= absoluteImageIDList_temp[j])
				{
					highIndexEx = j;
					break;
				}
			}
		
			//bobout << "lowindexIN: " << lowIndexIn << "\thighindexex: " << highIndexEx << endl;

			int currentIndex = lowIndexIn;
			int firstImageFrame = -1;
			for( j=imageRange.GetLowerBound(); j<imageRange.GetUpperBound() && currentIndex < highIndexEx; j++ ) 
			{
				if (j == absoluteImageIDList_temp[currentIndex])
				{
					if (firstImageFrame == -1)
						firstImageFrame = currentIndex;
					currentIndex++;
					locList.push_back( sceneData.GetFeatureList()[absFeatureList[i]].GetLocation( j ) );
				}
				
			}
			featureList_temp[i].overwriteLocationList(firstImageFrame, locList);
			
		
			while(locList.size() > 0)
				locList.pop_back();
	
		}
				
	
		//does this need to be freed?
		SceneData tempSceneData ( viewpointLocationList_temp, absoluteImageIDList_temp, viewpointValidOrientationList_temp, viewpointOrientationList_temp,
				featureList_temp, reconObjectList_temp, reconViewpointLocationList_temp, 
				SBAReconLocationList_temp, //triangulateReconLocationList_temp,
				projErrorList_temp, objectLocationList_temp, 0, 0, 0, sceneData.GetCameraParameters());
		sceneDataSubsets.push_back(tempSceneData);
			
	
		//empty vectors
		while (viewpointLocationList_temp.size() > 0)
		{
			viewpointLocationList_temp.pop_back();
			reconViewpointLocationList_temp.pop_back();
			absoluteImageIDList_temp.pop_back();
			viewpointValidOrientationList_temp.pop_back();
			viewpointOrientationList_temp.pop_back();
		}
		while (absFeatureList.size() > 0)
		{
			absFeatureList.pop_back();
			featureList_temp.pop_back();
			reconObjectList_temp.pop_back();
			SBAReconLocationList_temp.pop_back();
			//triangulateReconLocationList_temp.pop_back();
			projErrorList_temp.pop_back();
			objectLocationList_temp.pop_back();
		}
		
	
	}
	//bobout.close();

	
}
/*****************************************************************************
*****************************************************************************/
void ReadModels()
{
	FILE *fp1 = fopen( "house.features", "w" );
	FILE *fp2 = fopen( "house.nview-corners", "r" );

	for ( int i=0; i<672; i++ )
	{
		char buf1[1024];
		char buf2[1024];
		//int temp[10];
		fgets( buf1, 1024, fp2 );

		//sscanf( buf1, "%d %d %d %d %d %d %d %d %d %d", temp, temp+1, temp+2, temp+3, temp+4, temp+5, temp+6, temp+7, temp+8, temp+9 );
		//printf( "%d %d %d %d %d %d %d %d %d %d\n", temp[0], temp[1], temp[2], temp[3], temp[4], temp[5], temp[6], temp[7], temp[8], temp[9] );

		for ( int j=0; j<50; j+=5 )
			if ( buf1[j] != '*' )
				break;

		fprintf( fp1, "FirstImageID %d %d\n", j / 5, i );

		for ( int k=j; k<50; k+=5 )
			if ( buf1[k] =='*' )
				break;

		fprintf( fp1, "NumProjections %d\n", ( k - j ) / 5 );

		for ( int m=j; m<k; m+=5 )
		{
			int projectionID;
			char fileName[1024];

			sscanf( buf1+m, "%d", &projectionID );
			sprintf( fileName, "house.00%d.corners", m/5);

			FILE *fp3 = fopen( fileName , "r" );
			for ( int n=0; n<=projectionID; n++ )
				fgets( buf2, 1024, fp3 );

			float x, y;
			sscanf( buf2, "%f %f", &x, &y );
			fprintf( fp1, "%f %f\n", ( 384 - x ) / float ( 384 ), ( 288 - y ) / float ( 288 ) );
			fclose( fp3 );
		}
		fprintf( fp1, "\n" );
	}

	fclose( fp2 );

	FILE *fp4 = fopen( "house.p3d", "r" );
	fprintf( fp1, "NumObjects 672\n" );

	for ( i=0; i<672; i++ )
	{
		char buf[256];
		fgets( buf, 256, fp4 );		
		float x, y, z;
		sscanf( buf, "%f %f %f", &x, &y, &z );
		fprintf( fp1, "%f %f %f\n", x, y, z );
	}

	fclose( fp4 );
	fclose( fp1 );
}


/*****************************************************************************
 *****************************************************************************/
static void guiCB(int id)
{
  switch (id) {

	case DIFF_FILES :
	  {
		try
		{
			cout << "Loading first input file (" << diffInputFileOne << ")" << endl;
		  //Load both files
			std::ifstream diffOneInput( diffInputFileOne );
			sceneData.LoadFeatureData( diffOneInput, 0, 0, 0 );
			diffOneInput.close();


			cout << "Loading second input file (" << diffInputFileTwo << ")" << endl;
			std::ifstream diffTwoInput( diffInputFileTwo );
		    diffData.LoadFeatureData( diffTwoInput, 0, 0, 0 );
		    diffTwoInput.close();


			//Diff both files
			cout << "Outputting Differences (" << diffOutputFile << ")" << endl;
			std::ofstream diffOutput( diffOutputFile );
			diffData.OutputDistanceBetweenScenes( sceneData, diffOutput );
			diffOutput.close();
		
			cout << "Differences output." << endl;
		}
        catch( std::exception & e ) 
		{
          cerr << e.what() << endl;
        }
		
		break;
	  }
	case INDUCE_ERROR :
	  {
		 // cerr << "Debug: " << sceneDataFile << " " << sceneDataSaveFile << endl;
		 guiCB( SCENEDATALOADID );

		 sceneData.AddPositionNoise( positionNoise );
		 cerr << "Saving..." << endl;
		 guiCB( SCENEDATASAVEID );

		break;
	  }

    case SCENEDATALOADID :
      {
		try 
		{
          std::ifstream sceneDataInput( sceneDataFile );
		  sceneData.LoadFeatureData( sceneDataInput, trackingNoise, cameraPositionNoise, cameraOrientationNoise );
		  sceneDataInput.close();



        }
        catch( std::exception & e ) 
		{
          cerr << e.what() << endl;
        }

        break;
      }
    case SCENEDATASAVEID :
      {
		try
		{

			std::ofstream sceneDataOutput( sceneDataSaveFile );
			sceneData.SaveFeatureData( sceneDataOutput );
			sceneDataOutput.close();//BOBEDIT
		}
		catch( std::exception & e ) 
		{
          cerr << e.what() << endl;
        }

        break;
      }
    case COMPUTEANCHORPOINTSID :
      {
        // Reconstruct the anchor points
		sceneData.ReconstructAnchors( featurePairLifeWeight, featurePairDistanceWeight, continuationThreshold, anchorImageNumber,
            trackingNoise, cameraPositionNoise, cameraOrientationNoise );

        // Update the anchor error stats
        UpdateAnchorError();

        break;
      }
	//BOBEDIT
	  //TODO: circular locality (if you walk all the way around the picture)
	  //TODO: we assume no feature appears in multiple groups
	  //sceneData.GetFeatureList() is sorted by increasing upper bound, so we resort it
	  //prints out all the features that span groups of images of size specified in glui box "anchor group number"
	  //for example, if '3' were in 'anchor group number', all features that appear in images 0, 1-3, 4-6, 7-9, 10-12, 13-15 would be output together in their
	  //	corresponding groups. Note, the first image, 0, is in its own group
	case ADJACENT_SUBSET_OPTIMIZATION :
	{
		int i=0;

		//ofstream bobout("adjacent_subset_optimization.txt");

		if (adjacentMinFeatureLife > adjacentGroupSize)
		{
			cout << "Error: adjacentMinFeatureLife( " << adjacentMinFeatureLife << " )  >  adjacentGroupSize( " << adjacentGroupSize
				<< " )" << endl;
			break;
		}
		else if (adjacentGroupSize > sceneData.GetAbsoluteImageIDList().size() || adjacentGroupSize < 2)
		{
			cout << "Error: groupSize > # of images or groupSize < 2" << endl;
			break;
		}

		std::vector<SceneData> sceneDataArray;
		extractAdjacentSubsets(sceneDataArray);

		cout << "# of subsets: " << sceneDataArray.size() << endl;
		for ( i=0; i<sceneDataArray.size(); i++ )
		{
			printf("group %d\n", i);
			printf("Num of points: %d\n",sceneDataArray[i].GetReconObjectList().size());
			/*for ( int j=0; j<sceneDataArray[i].GetReconObjectList().size(); j++ )
			{
				VM3Vector temp = sceneDataArray[i].GetReconObjectList()[j].GetReconLocation();
				temp.print(stdout);
			}*/
			printf("Num of cameras: %d\n",sceneDataArray[i].GetViewpointLocationList().size());
			/*for ( j=0; j<sceneDataArray[i].GetViewpointLocationList().size(); j++ )
				sceneDataArray[i].GetViewpointLocationList()[j].print(stdout);*/
		}
		printf("\n");
		fflush(stdout);



		cout << "sceneDataArray.size(): " << sceneDataArray.size() << endl;

		restoreAdjacentSubsets(sceneData, sceneDataArray);


		break;
	}
	case DISJOINT_SUBSET_OPTIMIZATION : 
	{
		int i=0;
		//If the parameters above are invalid
		if ( subsetIterationCount * withdrawImageCount > sceneData.GetViewpointLocationList().size() ||
			 disjointMaxFeaturesPerImageGroup > sceneData.GetFeatureList().size() ||
			 disjointMinFeatureLife > sceneData.GetViewpointLocationList().size() ||
			 disjointMinFeatureLife > withdrawImageCount )
		{
			cout << "invalid parameters for subset optimization." << endl;
			cout << "either " << subsetIterationCount << "*" << withdrawImageCount << " > " << sceneData.GetViewpointLocationList().size() << endl;
			cout << "or " << disjointMaxFeaturesPerImageGroup << " > " << sceneData.GetFeatureList().size() << endl;
			cout << "or " << disjointMinFeatureLife << " > " << sceneData.GetViewpointLocationList().size() << endl;
			//TODO: output error
			break;
		}

		//Backup sceneData (as extractDisjointSubsets will change it)
		SceneData origSceneData = SceneData(sceneData);

			
		//Extract Subsets
		std::vector<SceneData> sceneDataArray;
		extractDisjointSubsets(sceneDataArray);

		for ( i=0; i<sceneDataArray.size(); i++ )
		{
			printf("group %d\n", i);
			printf("Num of points: %d\n",sceneDataArray[i].GetReconObjectList().size());
			/*for ( int j=0; j<sceneDataArray[i].GetReconObjectList().size(); j++ )
			{
				VM3Vector temp = sceneDataArray[i].GetReconObjectList()[j].GetReconLocation();
				temp.print(stdout);
			}*/

			printf("Num of cameras: %d\n",sceneDataArray[i].GetViewpointLocationList().size());
			/*for ( j=0; j<sceneDataArray[i].GetViewpointLocationList().size(); j++ )
				sceneDataArray[i].GetViewpointLocationList()[j].print(stdout);*/

		}
		printf("\n");
		fflush(stdout);

		//Put subsets back into sceneData
		restoreDisjointSubsets(origSceneData, sceneDataArray);


		break;
	}
	//END BOBEDIT
    case COMPUTEALLPOINTSID :
      {
        // Reconstruct the nonanchors
		sceneData.ReconstructNonanchors( nonanchorImageNumber, trackingNoise, cameraPositionNoise, cameraOrientationNoise );
		
		computePercentiles();

        // Update the error stats
        UpdateNonanchorError();

        break;
      }
    case COMPUTEALLID :
      {
		sceneData.AddPositionNoise( positionNoise );

		sceneData.ReconstructAllPoints( );

		
		computePercentiles();
		UpdateAnchorError();


        break;
      }

    case OUTPUTRECONPOINTSID :
      {
        outputReconPoints( reconOutputFile );
        break;
      }
    case CONDITIONTHRESHOLDID :
      {
        // Update the error stats
        UpdateNonanchorError();


        break;
      }
    case VARIANCETHRESHOLDID :
      {
        // Update the error stats
        UpdateNonanchorError();

        break;
      }
    case FEATUREPAIRLIFEWEIGHTID :
      {
        featurePairDistanceWeight = 1 - featurePairLifeWeight;
        break;
      }
    case FEATUREPAIRDISTANCEWEIGHTID :
      {
        featurePairLifeWeight = 1 - featurePairDistanceWeight;
        break;
      }
    case GATHERDATAID :
      {
        GatherData();
        break;
      }
    case MESHDATAID :
      {
         LoadMeshData();
         break;
      }
    case LOADBOUNDINGBOXID :
      {
        LoadBoundingBox();
        break;
      }
    case COMPAREERRORS :
      {
        CompareErrors( errorCompareFile );
        break;
      }
    case QUIT :
      {
        printf("Bye bye.\n");
        exit(0);
      }
    case NOID :
      {
        break;
      }
    default :
      {
        printf("GUI error!\n");
      }
  }
}





/*****************************************************************************
 *****************************************************************************/

int
main(int argc, char *argv[])
{
  // parse command line
  ParseArgs(argc, argv);

  if (diffFlag)
  {
	  guiCB( DIFF_FILES );
  }
  else if (errorFlag)
  {
	guiCB( INDUCE_ERROR );
  }
  else
  {
	GatherData();
	cout << "Computation done, outputting to file." << endl;
	guiCB(SCENEDATASAVEID);
	cout << "Done" << endl;
  }

  return (TRUE);
}
