#ifndef OMSCENEDATA_ALREADY_INCLUDED
#define OMSCENEDATA_ALREADY_INCLUDED

#include <common.h>
#include <vm2point.h>
#include <vm3point.h>
#include <vm3vector.h>
#include <vm4matrix.h>
#include <vmarray.h>


/*****************************************************************************
*****************************************************************************/

typedef struct {
   int projId;
   int imageNum;
   int objectId;
   VM2Point coord;
} OMSceneProjEntry;

class OMSceneData {
   public:

      VMArray<VM3Point> objectList;
      VMArray<VM4Matrix> imageMatList;
      VMArray<OMSceneProjEntry> projList;

      OMSceneData() {}
      ~OMSceneData() {}

      int findImage(int imageNum, int *first=NULL, int *last=NULL)
      {
         // returns the first entry for given image
         for (int p=0; p<projList.size(); p++) {
            if (projList[p].imageNum == imageNum) {
               if (first) 
                  *first = p;
               if (last) {
                  for (int q=p; q<projList.size(); q++) {
                     if (projList[q].imageNum != imageNum) {
                        break;
                     }
                  }
                  *last = q-1;
               }
               return (p);
            }
         }

         return (-1);
      }

      int findObject(int imageNum, int objectId, VM2Point *coord=NULL)
      {
         int first, last;
         if (findImage(imageNum, &first, &last) >= 0) {
            for (int p=first; p<=last; p++) {
               OMSceneProjEntry &entry = projList[p];
               if (entry.imageNum == imageNum && entry.objectId == objectId) {
                  if (coord)
                     *coord = entry.coord;
                  return (TRUE);
               }
            }
         }

         return (FALSE);
      }

      int findFirstImage(int objectId)
      {
         // assume projections are stored in sorted image order
         for (int p=0; p<projList.size(); p++) {
            if (projList[p].objectId == objectId) {
               return (projList[p].imageNum);
            }
         }

         return (-1);
      }

      int findLastImage(int objectId)
      {
         // assume projections are stored in sorted image order
         for (int p=projList.size()-1; p>=0; p--) {
            if (projList[p].objectId == objectId) {
               return (projList[p].imageNum);
            }
         }

         return (-1);
      }

      int write(char *fileName)
      {
         FILE *fp;
         int i;

         if (!(fp = fopen(fileName, "w"))) {
            printf("Could not create \"%s\"!\n", fileName);
            return (FALSE);
         }

         fprintf(fp, "Num-Objects %i\n", objectList.size());
         for (i=0; i<objectList.size(); i++) {
            fprintf(fp, "%f %f %f\n", 
                    objectList[i].x(),
                    objectList[i].y(),
                    objectList[i].z());
         }
         fprintf(fp, "\n");

         fprintf(fp, "Num-Image-Matrices %i\n", imageMatList.size());
         for (i=0; i<imageMatList.size(); i++) {
            imageMatList[i].print(fp);
         }

         fprintf(fp, "Num-Projections %i\n", projList.size());
         for (i=0; i<projList.size(); i++) {
            fprintf(fp, "%i %i    %f %f\n",
                    projList[i].imageNum,
                    projList[i].objectId,
                    projList[i].coord.x(),
                    projList[i].coord.y());
         }

         fclose (fp);
         return (TRUE);
      }

      int read(char *fileName)
      {
         FILE *fp;
         char buf[256];
         float x,y,z,w;
         int i,j,val,id;

         if (!(fp = fopen(fileName, "r"))) {
            printf("Could not open \"%s\"!\n", fileName);
            return (FALSE);
         }

         // empty lists
         objectList.empty();
         imageMatList.empty();
         projList.empty();

         // get object list
         fgets(buf, 256, fp);
         sscanf(buf," %*s %i ", &val);
         objectList.resize(val);
         for (i=0; i<objectList.size(); i++) {
            fgets(buf, 256, fp);
            sscanf(buf, " %f %f %f ", &x, &y, &z);
            objectList[i].set(x,y,z);
         }
         fgets(buf, 256, fp);

         // get list of matrices (per frame)
         fgets(buf, 256, fp);
         sscanf(buf," %*s %i ", &val);
         imageMatList.resize(val);
         for (i=0; i<imageMatList.size(); i++) {
            VM4Matrix mat;
            for (j=0; j<4; j++) {
               fgets(buf, 256, fp);
               sscanf(buf, " %f %f %f %f ", &x, &y, &z, &w);
               mat.set(j,0,x);
               mat.set(j,1,y);
               mat.set(j,2,z);
               mat.set(j,3,w);
            }
            imageMatList[i] = mat;
            fgets(buf, 256, fp);
         }

         // get projection list
         fgets(buf, 256, fp);
         sscanf(buf," %*s %i ", &val);
         projList.resize(val);
         for (i=0; i<projList.size(); i++) {
            OMSceneProjEntry entry;
            fgets(buf, 256, fp);
            sscanf(buf, " %i %i %f %f ", &val, &id, &x, &y);
            entry.imageNum = val;
            entry.objectId = id;
            entry.coord.set(x,y);
            projList[i] = entry;
         }

         fclose (fp);
         return (TRUE);
      }

};

#endif
