#include "Disable4786.hpp"
#include <stdlib.h>
#include <math.h>
#include <stdexcept>
#include <time.h>
#include "OMSceneData.h"
#include "Nrutil.h"
#include "Feature.hpp"
#include "FeaturePair.hpp"
#include "IntegerRange.hpp"
#include <vector>
#include <deque>

#define INUM 15
#define PI 3.14159265358979323846
#define LENGTH_P 512
#define WIDTH_P 512
#define FOCUS 0.1
#define LENGTH_R 2*0.1*tan(PI/6)
#define WIDTH_R 2*0.1*tan(PI/6)

void svbksb(float **u, float w[], float **v, int m, int n, float b[], float x[]);
void svdcmp(float **a, int m, int n, float w[], float **v);

//
//Given a m*n matrix A and a m*1 vector B, solve the linear equations A*x=B.
//
void LinearEquationsSolver( float **A, float *x, float *B, int m, int n, float & conditionNum )
{ float **u = matrix(1, m, 1, n);
  float **v = matrix(1, n, 1, n);
  float *w = vector(1, n);
  float *b = vector(1, m);
  int i,j;
  float wmax,wmin,Wmin;

  for (i = 1; i <= n; ++i) 
    for (j = 1; j <= m; ++j) 
      u[j][i] = A[j][i];

  for (j = 1; j <= m; ++j) 
    b[j] = B[j];
 
  for (i = 1; i <= n; ++i) 
    x[i] = 0.0;

  // do a singular value decomposition
  svdcmp(u, m, n, w, v);

  wmax=0.0; 
  
  for ( j=1; j<=n; j++ ) 
    if ( w[j] > wmax ) 
      wmax=w[j]; 

  for ( j=1; j<=n; j++ )
	conditionNum+=wmax/w[j];

  //for ( j=1; j<=n; j++ )
  //  if ( Wmin > w[j] )
	//  Wmin=w[j];

  wmin=wmax*1.0e-6; 

  for( j=1; j<=n; j++ ) 
    if ( w[j] < wmin ) 
      w[j]=0.0;  

  // do a backsubstitution
  svbksb(u, w, v, m, n, b, x);
}

//
//Given the object ID i, return the projection lists for some images of this point and the anchor points we will use.
//
void CheckForProjs( VMArray<OMSceneProjEntry> & projList, const Feature & feature, 
				    VMArray<OMSceneProjEntry> ProjList[], std::deque<FeaturePair*> & anchormap )
{ int i, j, n, LowerBound, UpperBound;

  LowerBound=feature.GetRange().GetLowerBound();
  UpperBound=feature.GetRange().GetUpperBound();
  n=UpperBound-LowerBound;

  if ( n<INUM )
  { ProjList[2].resize(n);
    n=INUM;
  }
  else
    ProjList[2].resize(INUM);

  for ( i=0, j=0; i<ProjList[2].size(); i++ )
  { if ( anchormap[LowerBound+i*n/INUM] )        
    { ProjList[0][j]=projList[anchormap[LowerBound+i*n/INUM]->GetFirstFeature().GetLocation(LowerBound+i*n/INUM).projectionId_];
      ProjList[1][j]=projList[anchormap[LowerBound+i*n/INUM]->GetSecondFeature().GetLocation(LowerBound+i*n/INUM).projectionId_];
	  ProjList[2][j]=projList[feature.GetLocation(LowerBound+i*n/INUM).projectionId_];
	  j++;
	}
  }

  ProjList[2].resize(j);
  ProjList[0].resize(ProjList[2].size());
  ProjList[1].resize(ProjList[2].size());
}

//
//Given the projection lists from CheckForPros() and the object ID, we can calculate the 3D coordinates of this point. Save the
//coordinates to the object list reconObjectList, and change the flag of this point in the list reconObjectFlag.
//
void SolveForOneObjectPoint( VMArray<VM4Matrix> & imageMatList, VMArray<OMSceneProjEntry> ProjList[], int objectId,
                             VMArray<VM3Point> & reconObjectList, VMArray<int> & reconObjectFlag, VMArray<float> & conditionNum, VMArray<float> & Var_C, 
							 VMArray<float> & reconObjectDistance )
{ VM3Vector C[INUM], Q[INUM][2], M[INUM][3], Coord[INUM][3], Ave_C( 0.0, 0.0, 0.0 ), Object;
  VM3Point CP;
  int i, j, m;
  int Inum=ProjList[2].size();
  float gamma[INUM][2], k[INUM][3];
  float **A = matrix(1, 3*INUM, 1, 3+INUM);
  float *x = vector(1, 3+INUM);
  float *B = vector(1, 3*INUM);

  conditionNum[objectId]=0.0;

  if ( Inum<10 )
  { reconObjectList[objectId].set( 0, 0, 0 );  
    return;
  }
  
  for ( i=0; i<Inum; i++ )
  { CP=imageMatList[ProjList[2][i].imageNum].cameraPosition();
    VM3Vector T(CP); 
    C[i]=T;
  }

  for ( i=0; i<Inum; i++ )
    Ave_C+=C[i];

  Ave_C=Ave_C/Inum;
  Var_C[objectId]=0;

  for ( i=0; i<Inum; i++ )
    Var_C[objectId]+=(C[i]-Ave_C).lengthSquared();

  Var_C[objectId]=Var_C[objectId]/(Inum-1);

  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<2; j++ )    
    { CP=reconObjectList[ProjList[j][i].objectId];
      VM3Vector T(CP);
      Q[i][j]=T;
    }
  }

  for ( i=0; i<Inum; i++ )
  { M[i][0]=Q[i][0]-C[i];
    M[i][1]=(Q[i][0]-C[i]).dot(Q[i][0]-C[i])*(Q[i][1]-C[i])-(Q[i][0]-C[i]).dot(Q[i][1]-C[i])*(Q[i][0]-C[i]);
    M[i][2]=(Q[i][0]-C[i]).cross(Q[i][1]-C[i]);
  }
  
  for ( i=0; i<Inum; i++ )
    for ( j=0; j<3; j++ )
      Coord[i][j].set((ProjList[j][i].coord.x()-LENGTH_P/2)/LENGTH_P*LENGTH_R,
                      (ProjList[j][i].coord.y()-WIDTH_P/2)/WIDTH_P*WIDTH_R,
                      -FOCUS);


  for ( i=0; i<Inum; i++ )
  {  for ( j=0; j<2; j++ )
     { gamma[i][j]=sqrt((Q[i][j]-C[i]).dot(Q[i][j]-C[i])/Coord[i][j].dot(Coord[i][j]));
     }
  }

  for ( i=0; i<Inum; i++ )
  { k[i][0]=Coord[i][0].dot(Coord[i][2]);
    k[i][1]=(Coord[i][0].cross(Coord[i][2])).dot(Coord[i][0].cross(Coord[i][1]));
    k[i][2]=Coord[i][2].dot(Coord[i][0].cross(Coord[i][1]));
  }

  for ( i=1; i<=3*Inum; i++ )
    for ( j=1; j<=3+Inum; j++ )
      A[i][j]=0;

  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<3; j++ )
    { A[1+j+3*i][1]=M[i][j].x();
      A[1+j+3*i][2]=M[i][j].y();
      A[1+j+3*i][3]=M[i][j].z();
    }
    A[1+3*i][4+i]=-gamma[i][0]*k[i][0];
    A[2+3*i][4+i]=-gamma[i][0]*gamma[i][0]*gamma[i][1]*k[i][1];
    A[3+3*i][4+i]=-gamma[i][0]*gamma[i][1]*k[i][2];
  }

  for ( i=0; i<Inum; i++ )
    for ( j=0; j<3; j++ )
      B[1+j+3*i]=C[i].dot(M[i][j]);

  LinearEquationsSolver( A, x, B, 3*Inum, 3+Inum, conditionNum[objectId] );

  reconObjectList[objectId].set( x[1], x[2], x[3] );
  Object.set( x[1], x[2], x[3] );
  reconObjectFlag[objectId]=1;
  reconObjectDistance[objectId]=0;

  for ( i=0; i<Inum; i++ )
    reconObjectDistance[objectId]+=(Object-C[i]).length();

  reconObjectDistance[objectId]/=Inum;
}

//
//Given the list of anchor points for each image anchorIdList, we can calculate the 3D coordinates of every other points.  Save the
//coordinates to the object list reconObjectList, and change the flag of solved points in reconObjectFlag.
//
void SolveForAllObjectPoints( VMArray<VM4Matrix> & imageMatList, VMArray<OMSceneProjEntry> & projList, 
    const std::vector<Feature> & features, std::deque<FeaturePair*> & anchormap, 
    VMArray<VM3Point> & reconObjectList, VMArray<int> & reconObjectFlag, VMArray<float> & conditionNum, VMArray<float> & Var_C, 
    VMArray<float> & reconObjectDistance )
{ int i,j,n;
  VMArray<OMSceneProjEntry> ProjList[3];

  for ( i=0; i<reconObjectList.size(); i++ )
    conditionNum[i]=-1.0;
  
  for ( i=0; i<features.size(); i++ )
  { if ( reconObjectFlag[features[i].GetObjectID()]!=0 ) 
      continue;
	
    for ( j=0; j<3; j++ )
      ProjList[j].resize(INUM);    
    CheckForProjs( projList, features[i], ProjList, anchormap );
    try {
      SolveForOneObjectPoint( imageMatList, ProjList, features[i].GetObjectID(), reconObjectList, reconObjectFlag, conditionNum, Var_C, reconObjectDistance );
    }
    catch( const exception & e ) {
      printf( e.what() );
    }
  }
}



#if 0
void SolveForOneError( float tol, VMArray<VM3Point> & objectList, VMArray<VM4Matrix> & imageMatList, VMArray<OMSceneProjEntry> & projList, VMArray<OMSceneProjEntry> ProjList[], int objectId,
                       VMArray<VM3Point> & reconObjectList, VMArray<int> & reconObjectFlag, VMArray<float> & Angle )
{ VM3Vector C[INUM], Q[INUM][3], M[INUM][3], Coord[INUM][3];
  VM3Point CP,CPP;
  int i, j, m;
  int Inum=ProjList[2].size();
  float gamma[INUM][3], k[INUM][3], t, ran[3];
  float **A = matrix(1, 3*INUM, 1, 3+INUM);
  float *x = vector(1, 3+INUM);
  float *B = vector(1, 3*INUM);
  float *V = vector(1, 3*INUM);

  Angle[objectId]=0.0;

  if ( Inum<2 )
  { reconObjectList[objectId].set( 0, 0, 0 );  
    return;
  }
  
  for ( i=0; i<Inum; i++ )
  { CP=imageMatList[ProjList[2][i].imageNum].cameraPosition();
    VM3Vector T(CP); 
    C[i]=T;
	//imageMatList[ProjList[2][i].imageNum].print(stdout);
	//printf("%f %f %f\n\n",C[i].x(),C[i].y(),C[i].z());
  }

  for ( i=0; i<Inum-1; i++ )
  { for ( j=1; j<Inum; j++ )
    { t=ComputeAngle( objectList[objectId], C[i], C[j] );
      if ( Angle[objectId]<t )
	  Angle[objectId]=t; 
	}
  }

  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<3; j++ )    
    { CP=objectList[ProjList[j][i].objectId];
      VM3Vector T(CP);
      Q[i][j]=T;
	  //printf("%f %f %f\n",Q[i][j].x(),Q[i][j].y(),Q[i][j].z());
    }
	//printf("\n");
  }

  //printf("\n");

  for ( i=0; i<Inum; i++ )
  { M[i][0]=Q[i][0]-C[i];
    M[i][1]=(Q[i][0]-C[i]).dot(Q[i][0]-C[i])*(Q[i][1]-C[i])-(Q[i][0]-C[i]).dot(Q[i][1]-C[i])*(Q[i][0]-C[i]);
    M[i][2]=(Q[i][0]-C[i]).cross(Q[i][1]-C[i]);
  }
  
  /*for ( i=0; i<Inum; i++ )
  { for ( j=0; j<2; j++ )
    { Coord[i][j].set((ProjList[j][i].coord.x()-LENGTH_P/2)/LENGTH_P*LENGTH_R,
                      (ProjList[j][i].coord.y()-WIDTH_P/2)/WIDTH_P*WIDTH_R,
                      -FOCUS);
	}
    CPP=imageMatList[ProjList[2][i].imageNum]*(Q[i][2]-C[i]);
    t=CPP.z()/(-FOCUS);
    Coord[i][2].set( CPP.x()/t, CPP.y()/t, -FOCUS);
  }*/

  /*for ( i=0; i<Inum; i++ )
  { for ( j=0; j<2; j++ )
    { CPP=imageMatList[ProjList[j][i].imageNum]*(Q[i][j]-C[i]);
      t=CPP.z()/(-FOCUS);
      Coord[i][j].set( CPP.x()/t, CPP.y()/t, -FOCUS);
	  
	}
    Coord[i][2].set((ProjList[2][i].coord.x()-LENGTH_P/2)/LENGTH_P*LENGTH_R,
                    (ProjList[2][i].coord.y()-WIDTH_P/2)/WIDTH_P*WIDTH_R,
                    -FOCUS);
  }*/

  srand( time(0) );
  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<2; j++ )
    { ran[0]= tol;
      ran[1]= 2 * PI * ( float( rand() ) / float( RAND_MAX ) );
	  CPP=imageMatList[ProjList[j][i].imageNum]*(Q[i][j]-C[i]);
      t=CPP.z()/(-FOCUS);
      Coord[i][j].set( CPP.x()/t+ran[0]*cos(ran[1])/LENGTH_P*LENGTH_R, CPP.y()/t+ran[0]*sin(ran[1])/WIDTH_P*WIDTH_R, -FOCUS);
    }
    //ran[0]= tol;
    //ran[1]= 2 * PI * ( float( rand() ) / float( RAND_MAX ) );
	CPP=imageMatList[ProjList[2][i].imageNum]*(Q[i][2]-C[i]);
    t=CPP.z()/(-FOCUS);
    Coord[i][2].set( CPP.x()/t, CPP.y()/t, -FOCUS);
  }

  /*srand( time(0) );
  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<2; j++ )
    { CPP=imageMatList[ProjList[j][i].imageNum]*(Q[i][j]-C[i]);
      t=CPP.z()/(-FOCUS);
      Coord[i][j].set( CPP.x()/t, CPP.y()/t, -FOCUS);
    }
    ran[0]= tol;
    ran[1]= 2 * PI * ( float( rand() ) / float( RAND_MAX ) );
	CPP=imageMatList[ProjList[2][i].imageNum]*(Q[i][2]-C[i]);
    t=CPP.z()/(-FOCUS);
    Coord[i][2].set( CPP.x()/t+ran[0]*cos(ran[1])/LENGTH_P*LENGTH_R, CPP.y()/t+ran[0]*sin(ran[1])/WIDTH_P*WIDTH_R, -FOCUS);
  }*/

  for ( i=0; i<Inum; i++ )
  {  for ( j=0; j<3; j++ )
     { gamma[i][j]=sqrt((Q[i][j]-C[i]).dot(Q[i][j]-C[i])/Coord[i][j].dot(Coord[i][j]));
     }
  }

  for ( i=0; i<Inum; i++ )
  { k[i][0]=Coord[i][0].dot(Coord[i][2]);
    k[i][1]=(Coord[i][0].cross(Coord[i][2])).dot(Coord[i][0].cross(Coord[i][1]));
    k[i][2]=Coord[i][2].dot(Coord[i][0].cross(Coord[i][1]));
  }

  for ( i=1; i<=3*Inum; i++ )
    for ( j=1; j<=3+Inum; j++ )
      A[i][j]=0;

  for ( i=0; i<Inum; i++ )
  { for ( j=0; j<3; j++ )
    { A[1+j+3*i][1]=M[i][j].x();
      A[1+j+3*i][2]=M[i][j].y();
      A[1+j+3*i][3]=M[i][j].z();
    }
    V[i*3+1]=A[1+3*i][4+i]=-gamma[i][0]*k[i][0];
    V[i*3+2]=A[2+3*i][4+i]=-gamma[i][0]*gamma[i][0]*gamma[i][1]*k[i][1];
    V[i*3+3]=A[3+3*i][4+i]=-gamma[i][0]*gamma[i][1]*k[i][2];
  }

  //for ( i=1; i<=3*INUM; i++ )
  //{ printf(" %f ",V[i]);
  //}
  //printf("\n");

  for ( i=0; i<Inum; i++ )
    for ( j=0; j<3; j++ )
      B[1+j+3*i]=C[i].dot(M[i][j]);


  /*for ( i=1; i<=3*Inum; i++ )
  { for ( j=1; j<=3+Inum; j++ )
    { printf("%f ",A[i][j]);
	}
	printf("\n");
  }

  printf("\n");*/

  LinearEquationsSolver( A, x, B, 3*Inum, 3+Inum );

  reconObjectList[objectId].set( x[1], x[2], x[3] );
  //if ( Angle[objectId]>=0.15 )
  reconObjectFlag[objectId]=1;

  /*if ( sqrt( (objectList[objectId].x()-reconObjectList[objectId].x())*(objectList[objectId].x()-reconObjectList[objectId].x())+
	         (objectList[objectId].y()-reconObjectList[objectId].y())*(objectList[objectId].y()-reconObjectList[objectId].y())+
             (objectList[objectId].z()-reconObjectList[objectId].z())*(objectList[objectId].z()-reconObjectList[objectId].z()) ) > 0.5 )
  { printf("%d\n ", objectId);
    printf("%f %f %f\n%f %f %f\n\n",objectList[objectId].x(),objectList[objectId].y(),objectList[objectId].z(),
		                            reconObjectList[objectId].x(),reconObjectList[objectId].y(),reconObjectList[objectId].z());
  }*/
}

void SolveForError( float tol, VMArray<VM3Point> & objectList, VMArray<VM4Matrix> & imageMatList, VMArray<OMSceneProjEntry> & projList, VMArray<AnchorId> & anchorIdList, 
				    VMArray<VM3Point> & reconObjectList, VMArray<int> & reconObjectFlag, VMArray<float> & Angle )
{ int i, j;
  float ran[3];
  VM3Point r;
  VMArray<OMSceneProjEntry> ProjList[3];

  for ( i=0; i<reconObjectList.size(); i++ )
  { if ( reconObjectFlag[i]!=0 ) 
    {  Angle[i]=1.0;
       continue;
	}
    for ( j=0; j<3; j++ )
      ProjList[j].resize(projList.size());    
    CheckForProjs( objectList, imageMatList, projList, i, ProjList, anchorIdList );
    try {
      SolveForOneError( tol, objectList, imageMatList, projList, ProjList, i, reconObjectList, reconObjectFlag, Angle );
    }
    catch( const exception & e ) {
      printf( e.what() );
    }
  }
}
#endif
