#include "vm3matrix.h"



/*****************************************************************************

Rotate the matrix by 'radians'.

*****************************************************************************/
void 
VM3Matrix::rotate(float radians)
{
   float sinTheta = SIN(radians);
   float cosTheta = COS(radians);
   VM3Matrix m;

   m.mat[0][0] = cosTheta;
   m.mat[1][1] = cosTheta;
   m.mat[0][1] = -sinTheta;
   m.mat[1][0] = sinTheta;

   (*this) = (*this) * m;
}


/*****************************************************************************

Below is the method invert() and its auxiliary routines. This routine is
efficient but does not assume homogeneous-type matrices.

*****************************************************************************/

#define SMALL 1e-8		/* Small enough to be considered zero */
#define MYSWAP(A,B,T)		T=B,B=A,A=T

/*
 * Shuffles rows in inverse of 3x3.  See comment in Matrix3_inv3_second_col().
 */
static void
Matrix3_inv3_swap(float inv[3][3], int row0, int row1, int row2)
{
   register int i, tempi;
   float	temp;

#define MYSWAP_ROWS(a, b) \
   for (i = 0; i < 3; i++) MYSWAP(inv[a][i], inv[b][i], temp); \
   MYSWAP(a, b, tempi)

   if (row0 != 0){
      if (row1 == 0) {
	 MYSWAP_ROWS(row0, row1);
      }
      else {
	 MYSWAP_ROWS(row0, row2);
      }
   }

   if (row1 != 1) {
      MYSWAP_ROWS(row1, row2);
   }
}

/*
 * Does Gaussian elimination on second column.
 */

static int
Matrix3_inv3_second_col(float source[3][3], float inv[3][3], int row0)
{
   register int row1, row2, i1, i2, i;
   float	temp;
   float	a, b;

   /* Find which row to use */
   if	   (row0 == 0)	i1 = 1, i2 = 2;
   else if (row0 == 1)	i1 = 0, i2 = 2;
   else 		i1 = 0, i2 = 1;

   /* Find which is larger in abs. val.:the entry in [i1][1] or [i2][1]	*/
   /* and use that value for pivoting.					*/

   a = source[i1][1]; if (a < 0) a = -a;
   b = source[i2][1]; if (b < 0) b = -b;
   if (a > b) 	row1 = i1;
   else		row1 = i2;
   row2 = (row1 == i1 ? i2 : i1);

   /* Scale row1 in source */
   if ((source[row1][1] < SMALL) && (source[row1][1] > -SMALL)) return(FALSE);
   temp = 1.0 / source[row1][1];
   source[row1][1]  = 1.0;
   source[row1][2] *= temp;	/* source[row1][0] is zero already */

   /* Scale row1 in inv */
   inv[row1][row1]  = temp;	/* it used to be a 1.0 */
   inv[row1][row0] *= temp;

   /* Clear column one, source, and make corresponding changes in inv */

   for (i = 0; i < 3; i++) if (i != row1) {	/* for i = all rows but row1 */
   temp = -source[i][1];
      source[i][1]  = 0.0;
      source[i][2] += temp * source[row1][2];

      inv[i][row1]  = temp * inv[row1][row1];
      inv[i][row0] += temp * inv[row1][row0];
   }

   /* Scale row2 in source */
   if ((source[row2][2] < SMALL) && (source[row2][2] > -SMALL)) return(FALSE);
   temp = 1.0 / source[row2][2];
   source[row2][2] = 1.0;	/* source[row2][*] is zero already */

   /* Scale row2 in inv */
   inv[row2][row2]  = temp;	/* it used to be a 1.0 */
   inv[row2][row0] *= temp;
   inv[row2][row1] *= temp;

   /* Clear column one, source, and make corresponding changes in inv */
   for (i = 0; i < 3; i++) if (i != row2) {	/* for i = all rows but row2 */
   temp = -source[i][2];
      source[i][2] = 0.0;
      inv[i][row0] += temp * inv[row2][row0];
      inv[i][row1] += temp * inv[row2][row1];
      inv[i][row2] += temp * inv[row2][row2];
   }

   /*
    * Now all is done except that the inverse needs to have its rows shuffled.
    * row0 needs to be moved to inv[0][*], row1 to inv[1][*], etc.
    *
    * We *didn't* do the swapping before the elimination so that we could more
    * easily keep track of what ops are needed to be done in the inverse.
    */
   Matrix3_inv3_swap(inv, row0, row1, row2);

   return(TRUE);
}

/*
 * Fast inversion routine for 3 x 3 matrices.	- Written by jfh.
 *
 * This takes 30 multiplies/divides, as opposed to 39 for Cramer's Rule.
 * The algorithm consists of performing fast gaussian elimination, by never
 * doing any operations where the result is guaranteed to be zero, or where
 * one operand is guaranteed to be zero. This is done at the cost of clarity,
 * alas.
 *
 * Returns 1 if the inverse was successful, 0 if it failed.
 */

static int
Matrix3_invert3(float source[3][3], float inv[3][3])
{
   register int i, row0;
   float	temp;
   float	a, b, c;

   inv[0][0] = inv[1][1] = inv[2][2] = 1.0;
   inv[0][1] = inv[0][2] = inv[1][0] = inv[1][2] = inv[2][0] = inv[2][1] = 0.0;

   /* attempt to find the largest entry in first column to use as pivot */
   a = source[0][0]; if (a < 0) a = -a;
   b = source[1][0]; if (b < 0) b = -b;
   c = source[2][0]; if (c < 0) c = -c;

   if (a > b) {
      if (a > c) row0 = 0;
      else row0 = 2;
   }
   else { 
      if (b > c) row0 = 1;
      else row0 = 2;
   }

   /* Scale row0 of source */
   if ((source[row0][0] < SMALL) && (source[row0][0] > -SMALL)) return(FALSE);
   temp = 1.0 / source[row0][0];
   source[row0][0]  = 1.0;
   source[row0][1] *= temp;
   source[row0][2] *= temp;

   /* Scale row0 of inverse */
   inv[row0][row0] = temp;	/* other entries are zero -- no effort	*/

   /* Clear column zero of source, and make corresponding changes in inverse */

   for (i = 0; i < 3; i++) if (i != row0) {	/* for i = all rows but row0 */
      temp = -source[i][0];
      source[i][0]  = 0.0;
      source[i][1] += temp * source[row0][1];
      source[i][2] += temp * source[row0][2];
      inv[i][row0]  = temp * inv[row0][row0];
   }

   /*
    * We've now done gaussian elimination so that the source and
    * inverse look like this:
    *
    *	1 * *		* 0 0
    *	0 * *		* 1 0
    *	0 * *		* 0 1
    *
    * We now proceed to do elimination on the second column.
    */
   if (! Matrix3_inv3_second_col(source, inv, row0)) return(FALSE);

   return(TRUE);
}


/*****************************************************************************
*****************************************************************************/
int
VM3Matrix::invert()
{
   float inv[3][3];
   int   ret;

   /* INVERT CODE IS ROW MAJOR, SO TRANSPOSE NOW, AND AGAIN AT THE END */
   this->transpose(); 

   if (ret = Matrix3_invert3(mat,inv))
      BCOPY(inv, mat, sizeof(float)*9);

   /* INVERT CODE IS ROW MAJOR, SO RE-TRANSPOSE NOW */
   this->transpose(); 

   return (ret);
}
