#include "vm3vector.h"
#include <vm3point.h>
#include <vm3matrix.h>
#include <vm4matrix.h>


/*****************************************************************************
*****************************************************************************/
VM3Vector::VM3Vector(VM3Point &pt) 
{ 
  vx = pt.x(); vy = pt.y(); vz = pt.z();
}


/*****************************************************************************
*****************************************************************************/
VM3Vector 
VM3Vector::perpendicular() 
{
   float   vec[3],result_vec[3],abs_vec[3];
   int     min,other1,other2;

   /* Compute the absolute value of the components of the input vector. */
   vec[0] = vx;
   vec[1] = vy;
   vec[2] = vz;
   abs_vec[0] = ABS(vec[0]);
   abs_vec[1] = ABS(vec[1]+0.0000001);
   abs_vec[2] = ABS(vec[2]-0.0000001);

   /* Determine the component with the minimum absolute value, saving its
      index in "min".  Save the indexes of the other two components in
      "other1" and "other2". */

   if (abs_vec[0] <= abs_vec[1])
      if (abs_vec[0] <= abs_vec[2]) {
         min = 0; other1 = 1; other2 = 2;
      }
      else {
         min = 2; other1 = 0; other2 = 1;
      }
   else
      if (abs_vec[1] <= abs_vec[2]) {
         min = 1; other1 = 0; other2 = 2;
      }
      else {
         min = 2; other1 = 0; other2 = 1;
      }

   /* Generate the perpendicular vector by setting the "min" compenent to
      0, and swapping the "other1" and "other2' components from "vec",
      negating one of these. */

   result_vec[min]    = 0.0;
   result_vec[other1] = vec[other2];
   result_vec[other2] = -vec[other1];

   /* If the input vector has unit length and the component with the mimimum
      absolute value is 0, the computed perpendicular vector already has
      unit length.  Otherwise, we must normalize it. */
   return VM3Vector(result_vec[0], result_vec[1], result_vec[2]);
}


/*****************************************************************************
*****************************************************************************/
VM3Vector operator*(const VM4Matrix& m, const VM3Vector &v) 
{
   register int i,j;
   float   vec[4];
   float   res[4];

   vec[0] = v.x(); 
   vec[1] = v.y(); 
   vec[2] = v.z(); 
   vec[3] = 0.0;       // <<<<<<<<<<<<< THIS SHOULD BE ZERO!!!!!! fixed 9/7/05
   for (i = 0; i < 4; i++) {
      res[i] = 0.0;
      for (j = 0; j < 4; j++)
         res[i] += m(i,j) * vec[j];
   }
   if (res[3] != 1.0 && res[3] != 0.0) {
      res[0] /= res[3];
      res[1] /= res[3];
      res[2] /= res[3];
   }
         
   return VM3Vector(res[0],res[1],res[2]);
}

